@@ -1,6 +1,6 @@
 /**********************************************************************
 zyGrib: meteorological GRIB file viewer
-Copyright (C) 2008-2010 - Jacques Zaninetti - http://www.zygrib.org
+Copyright (C) 2008-2012 - Jacques Zaninetti - http://www.zygrib.org
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -30,22 +30,26 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 //===========================================================================
 // DialogGraphicsParams
 //===========================================================================
-DialogChooseLang::DialogChooseLang (QString defaultlang)
-		: DialogBoxColumn ( 1, 
+DialogChooseLang::DialogChooseLang (QWidget *parent, QString defaultlang)
+		: DialogBoxColumn ( parent, 1, 
 							"Welcome to zyGrib", 
 						    "Choose your language",   
 						   1 )
 {
 	btGroup = new QButtonGroup (this);
 	assert (btGroup);
-	bt_de = addLanguage ("Deutsch",  "img/drapeau_de.png");
-	bt_en = addLanguage ("English",  "img/drapeau_en.png");
-	bt_es = addLanguage ("Español",  "img/drapeau_es.png");
-	bt_fr = addLanguage ("Français", "img/drapeau_fr.png");
-	bt_it = addLanguage ("Italiano", "img/drapeau_it.png");
-	bt_nl = addLanguage ("Dutch",    "img/drapeau_nl.png");
-	bt_ru = addLanguage ("Русско",   "img/drapeau_ru.png");
-	bt_cz = addLanguage ("Čeština", "img/drapeau_cz.png");
+	// A. Degwerth [Cassidian]: added Arabic language selection
+	bt_ar = addLanguage ("العربية",   Util::pathImg("drapeau_ar.png"));
+	bt_cz = addLanguage ("Čeština",   Util::pathImg("drapeau_cz.png"));
+	bt_de = addLanguage ("Deutsch",   Util::pathImg("drapeau_de.png"));
+	bt_en = addLanguage ("English",   Util::pathImg("drapeau_en.png"));
+	bt_es = addLanguage ("Español",   Util::pathImg("drapeau_es.png"));
+	bt_fi = addLanguage ("Finnish",   Util::pathImg("drapeau_fi.png"));
+	bt_fr = addLanguage ("Français",  Util::pathImg("drapeau_fr.png"));
+	bt_it = addLanguage ("Italiano",  Util::pathImg("drapeau_it.png"));
+	bt_nl = addLanguage ("Dutch",     Util::pathImg("drapeau_nl.png"));
+	bt_pt = addLanguage ("Português (Brasil)", Util::pathImg("drapeau_pt.png"));
+	bt_ru = addLanguage ("Русско",    Util::pathImg("drapeau_ru.png"));
 	
 	if (defaultlang == "de")
 		bt_de->setChecked (true);
@@ -53,16 +57,23 @@ DialogChooseLang::DialogChooseLang (QString defaultlang)
 		bt_en->setChecked (true);
 	else if (defaultlang == "es")
 		bt_es->setChecked (true);
+	else if (defaultlang == "fi")
+		bt_fi->setChecked (true);
 	else if (defaultlang == "fr")
 		bt_fr->setChecked (true);
 	else if (defaultlang == "it")
 		bt_it->setChecked (true);
 	else if (defaultlang == "nl")
 		bt_nl->setChecked (true);
+	else if (defaultlang == "pt")
+		bt_pt->setChecked (true);
 	else if (defaultlang == "ru")
 		bt_ru->setChecked (true);
 	else if (defaultlang == "cz")
 		bt_ru->setChecked (true);
+	// A. Degwerth [Cassidian]: added Arabic language selection
+	else if (defaultlang == "ar")
+		bt_ar->setChecked (true);
 	else
 		bt_en->setChecked (true);
 }	
@@ -75,16 +86,23 @@ QString DialogChooseLang::getLanguage ()
 		return "en";
 	else if (bt_es->isChecked())
 		return "es";
+	else if (bt_fi->isChecked())
+		return "fi";
 	else if (bt_fr->isChecked())
 		return "fr";
 	else if (bt_it->isChecked())
 		return "it";
 	else if (bt_nl->isChecked())
 		return "nl";
+	else if (bt_pt->isChecked())
+		return "pt";
 	else if (bt_ru->isChecked())
 		return "ru";
 	else if (bt_cz->isChecked())
 		return "cz";
+	// A. Degwerth [Cassidian]: added Arabic language selection
+	else if (bt_ar->isChecked())
+		return "ar";
 	else
 		return "en";
 }
@@ -104,7 +122,7 @@ QRadioButton * DialogChooseLang::addLanguage (QString name, QString iconfile)
 //===========================================================================
 // DialogGraphicsParams
 //===========================================================================
-DialogGraphicsParams::DialogGraphicsParams ()
+DialogGraphicsParams::DialogGraphicsParams (QWidget *parent) : DialogBoxBase (parent)
 {
 	setupUi(this);
 	initInterface();
@@ -130,6 +148,10 @@ void DialogGraphicsParams::slotBtOK()
 	Util::setSetting("isobarsLineColor",  inputIsobarsLine->getLineColor());
 	Util::setSetting("isotherms0LineWidth",  inputIsotherms0Line->getLineWidth());
 	Util::setSetting("isotherms0LineColor",  inputIsotherms0Line->getLineColor());
+	Util::setSetting("isotherms_LineWidth",  inputIsotherms_Line->getLineWidth());
+	Util::setSetting("isotherms_LineColor",  inputIsotherms_Line->getLineColor());
+	Util::setSetting("linesThetaE_LineWidth",  inputThetaE_Line->getLineWidth());
+	Util::setSetting("linesThetaE_LineColor",  inputThetaE_Line->getLineColor());
 	
 	Util::setSetting("cloudsColorMode", inputCloudsColorMode->itemData(inputCloudsColorMode->currentIndex()).toString());
     
@@ -175,6 +197,14 @@ void DialogGraphicsParams::initInterface()
 				Util::getSetting("isotherms0LineWidth", 1.6).toDouble(),
 				Util::getSetting("isotherms0LineColor", QColor(200,120,100)).value<QColor>(),
 				1.6,  QColor(200,120,100) );
+	inputIsotherms_Line->setValues (
+				Util::getSetting("isotherms_LineWidth", 1.6).toDouble(),
+				Util::getSetting("isotherms_LineColor", QColor(40,40,150)).value<QColor>(),
+				1.6,  QColor(40,40,150) );
+	inputThetaE_Line->setValues (
+				Util::getSetting("linesThetaE_LineWidth", 1.6).toDouble(),
+				Util::getSetting("linesThetaE_LineColor", QColor(40,140,40)).value<QColor>(),
+				1.6,  QColor(40,140,40) );
 
 	inputCloudsColorMode->addItem (tr("100 % -> white"), "white");
 	inputCloudsColorMode->addItem (tr("100 % -> dark"), "black");
