@@ -1,6 +1,6 @@
 /**********************************************************************
 zyGrib: meteorological GRIB file viewer
-Copyright (C) 2008-2010 - Jacques Zaninetti - http://www.zygrib.org
+Copyright (C) 2008-2012 - Jacques Zaninetti - http://www.zygrib.org
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -32,14 +32,13 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <QDoubleSpinBox>
 
 #include "ui_GraphicsParamsDialog.h"
-#include "DialogBox.h"
 #include "DialogBoxColumn.h"
 
 //-------------------------------------------
 class DialogChooseLang : public DialogBoxColumn 
 { Q_OBJECT
 	public:
-		DialogChooseLang (QString defaultlang);
+		DialogChooseLang (QWidget *parent, QString defaultlang);
 		QString getLanguage ();
 		
 	private:
@@ -49,21 +48,25 @@ class DialogChooseLang : public DialogBoxColumn
 		QRadioButton *bt_de;
 		QRadioButton *bt_en;
 		QRadioButton *bt_es;
+		QRadioButton *bt_fi;
 		QRadioButton *bt_fr;
 		QRadioButton *bt_it;
 		QRadioButton *bt_nl;
+		QRadioButton *bt_pt;
 		QRadioButton *bt_ru;
 		QRadioButton *bt_cz;
+		// A. Degwerth [Cassidian]: added RadioButton for Arabic language
+		QRadioButton *bt_ar;
 };
 
 
 //==============================================================
 class DialogGraphicsParams
-		: public  DialogBox,
+		: public  DialogBoxBase,
 		  private Ui_GraphicsParamsDialog
 { Q_OBJECT
     public:
-        DialogGraphicsParams ();
+        DialogGraphicsParams (QWidget *parent=NULL);
     
     public slots:
         void slotBtOK ();
