@@ -1,6 +1,6 @@
 /**********************************************************************
 zyGrib: meteorological GRIB file viewer
-Copyright (C) 2008-2010 - Jacques Zaninetti - http://www.zygrib.org
+Copyright (C) 2008-2012 - Jacques Zaninetti - http://www.zygrib.org
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -25,39 +25,47 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <QLabel>
 #include <QSpinBox>
 #include <QCheckBox>
+#include <QRadioButton>
 #include <QComboBox>
 #include <QPushButton>
 #include <QProgressBar>
 #include <QFileDialog>
 
-#include "DialogBox.h"
+#include "DataDefines.h"
+#include "DialogBoxBase.h"
 #include "FileLoaderGRIB.h"
 
-class DialogLoadGRIB : public DialogBox
+class DialogLoadGRIB : public DialogBoxBase
 { Q_OBJECT
     public:
-        DialogLoadGRIB();
-        ~DialogLoadGRIB();
+        DialogLoadGRIB (QNetworkAccessManager *manager, QWidget *parent);
+        ~DialogLoadGRIB ();
         
-        void setZone(float x0, float y0, float x1, float y1);
+        void setZone (double x0, double y0, double x1, double y1);
+		
+		static QString getFile (QNetworkAccessManager *manager, QWidget *parent,
+						double x0, double y0, double x1, double y1);
 
     public slots:
         void slotBtOK();
         void slotBtCancel();
         void slotBtServerStatus();
+        void slotBtProxy();
         void slotGribDataReceived(QByteArray *content, QString fileName);
         void slotGribReadProgress(int step, int done, int total);
         void slotGribFileError(QString error);
         void slotGribMessage(QString msg);
         void slotGribStartLoadData();
         void slotParameterUpdated();
-        void slotAltitudeAll();
-
-    signals:
-        void signalMeteoFileReceived(QString fileName);
+        void slotAltitude_All ();
+        void slotFnmocWW3_All ();
+		void slotFinished (int result);
 
     private:
         FileLoaderGRIB    *loadgrib;
+		QNetworkAccessManager *networkManager;
+		QString  savedFileName;
+		
         bool     loadInProgress;
         QTime    timeLoad;
         QCursor  oldcursor;
@@ -70,49 +78,75 @@ class DialogLoadGRIB : public DialogBox
         QComboBox *cbResolution;
         QComboBox *cbInterval;
         QComboBox *cbDays;
+        QComboBox *cbRunGFS;
 
         QCheckBox *chkWind;
         QCheckBox *chkPressure;
         QCheckBox *chkRain;
         QCheckBox *chkCloud;
+        QCheckBox *chkCloudLayers;
         QCheckBox *chkTemp;
         QCheckBox *chkHumid;
         QCheckBox *chkIsotherm0;
-        QCheckBox *chkTempPot;
         QCheckBox *chkTempMin;
         QCheckBox *chkTempMax;
         QCheckBox *chkSnowCateg;
         QCheckBox *chkFrzRainCateg;
         QCheckBox *chkSnowDepth;
         QCheckBox *chkCAPEsfc;
+        QCheckBox *chkCINsfc;
+        QCheckBox *chkGUSTsfc;
+        QCheckBox *chkSUNSDsfc;
 		
-        QCheckBox *chkAltitudeAll;
+        QCheckBox *chkAltitude_SkewT;
+        QCheckBox *chkAltitude_All;
         QCheckBox *chkAltitude200;
         QCheckBox *chkAltitude300;
+        QCheckBox *chkAltitude400;
         QCheckBox *chkAltitude500;
+        QCheckBox *chkAltitude600;
         QCheckBox *chkAltitude700;
         QCheckBox *chkAltitude850;
+        QCheckBox *chkAltitude925;
+		
+        QCheckBox *chkFnmocWW3_All;
+        QCheckBox *chkFnmocWW3_sig;
+        QCheckBox *chkFnmocWW3_max;
+        QCheckBox *chkFnmocWW3_swl;
+        QCheckBox *chkFnmocWW3_wnd;
+        QCheckBox *chkFnmocWW3_prim;
+        QCheckBox *chkFnmocWW3_scdy;
+        QCheckBox *chkFnmocWW3_wcap;
+		
+		DataCenterModel waveDataModel;
+		QRadioButton *bt_FNMOC_WW3_GLB;
+		QRadioButton *bt_FNMOC_WW3_MED;
         
         QPushButton *btOK;
         QPushButton *btCancel;
         QPushButton *btServerStatus;
+        QPushButton *btProxy;
 
         QProgressBar *progressBar;
         QLabel       *labelMsg;
         
         QFrame *createFrameButtonsZone(QWidget *parent);
         
-        float   xmin,ymin,xmax,ymax,resolution;
+        double   xmin,ymin,xmax,ymax,resolution;
         int     interval,days;
         
         bool    rain, cloud, pressure, wind, temp, humid, isotherm0;
-        bool	tempPot, tempMin, tempMax, snowDepth, snowCateg, frzRainCateg;
-        bool 	CAPEsfc;
+        bool	tempMin, tempMax, snowDepth, snowCateg, frzRainCateg;
+        bool 	CAPEsfc, CINsfc, cloudLayers;
+		bool 	GUSTsfc;
+		bool 	SUNSDsfc;
 		
         void    updateParameters();
         void    addSeparator (QLayout *layout, char orientation);	// 'H' or 'V'
+        QFrame *newSeparator (char orientation);	// 'H' or 'V'
+        QString createStringParameters ();
                 
-
+        void    closeEvent (QCloseEvent *event);
 };
 
 
