@@ -1,6 +1,6 @@
 /**********************************************************************
 zyGrib: meteorological GRIB file viewer
-Copyright (C) 2008-2010 - Jacques Zaninetti - http://www.zygrib.org
+Copyright (C) 2008-2012 - Jacques Zaninetti - http://www.zygrib.org
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -30,12 +30,12 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <QRadioButton>
 #include <QLineEdit>
 
-#include "DialogBox.h"
+#include "DialogBoxBase.h"
 
-class DialogProxy : public DialogBox
+class DialogProxy : public DialogBoxBase
 { Q_OBJECT
     public:
-        DialogProxy();
+        DialogProxy (QWidget *parent=NULL);
     
     public slots:
         void slotBtOK();
@@ -43,7 +43,8 @@ class DialogProxy : public DialogBox
 
     private slots:
         void slotUseProxyChanged();
-    
+        void slotTrueHttpDownloadChanged();
+		
     private:
         QFrame *frameGui;
         QGridLayout *layout;
@@ -53,10 +54,20 @@ class DialogProxy : public DialogBox
         
         QRadioButton *btUseProxy;
         QRadioButton *btDontUseProxy;
+		
+		QComboBox    *cbProxyType;
         QLineEdit    *lineProxyHostname;
         QLineEdit    *lineProxyPort;
-        QLineEdit    *lineProxyUsername;
+        QLineEdit    *lineProxyUserName;
         QLineEdit    *lineProxyUserPassword;
+		
+        QRadioButton *btStandardDownload;
+        QRadioButton *btStrictHttpDownload;
+        QLineEdit    *lineZygribForumUserName;
+        QLineEdit    *lineZygribForumUserPassword;
+		
+		QList <QLabel*> listProxyLabels;
+		QList <QLabel*> listDownloadLabels;
 
         QFrame * createFrameGui(QWidget *parent);
 };
