@@ -1,6 +1,6 @@
 /**********************************************************************
 zyGrib: meteorological GRIB file viewer
-Copyright (C) 2008-2010 - Jacques Zaninetti - http://www.zygrib.org
+Copyright (C) 2008-2012 - Jacques Zaninetti - http://www.zygrib.org
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -27,43 +27,58 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <QtNetwork>
 #include <QBuffer>
 
-#include "DialogBox.h"
+#include "DialogBoxBase.h"
 
-class DialogServerStatus : public  DialogBox
+class DialogServerStatus : public  DialogBoxBase
 { Q_OBJECT
     public:
-        DialogServerStatus();
-        ~DialogServerStatus();
+        DialogServerStatus (QNetworkAccessManager *manager, 
+							QWidget *parent=NULL);
+        ~DialogServerStatus ();
     
     public slots:
         void slotBtOK();
         void slotBtCancel();
-        void requestFinished (int id, bool error);
-    
+        void downloadProgress_step1 (qint64 done, qint64 total);
+		void slotNetworkError (QNetworkReply::NetworkError);
     
     private:
-        QString getNoaaRunDate (QStringList lsbuf);
+		QHash <QString,QString> readData (const QByteArray &data);
+		QString getData (const QHash <QString,QString> &data, const QString &key);
+        
+		QString getNoaaRunDate (QStringList lsbuf);
         QString getNoaaRunHour (QStringList lsbuf);
         QString getServerCurrentJob (QStringList lsbuf);
+        QString getClient (QStringList lsbuf);
         
         QFrame *frameGui;
         QGridLayout *layout;
-        
         QPushButton *btOK;
-        
-        QHttp    *http;
-        QString  host;
-        QBuffer  ioBuffer;
-        int      idHttpGetFile;
+		
+		QString errorMessage;
+		QNetworkReply *reply_step1;
+		bool downloadError;
         
         QTime   timeLoad;
         QLabel *lbResponseStatus;
-        QLabel *lbResponseTime;
         QLabel *lbRunDate;
-        QLabel *lbRunHour;
-        QLabel *lbMessage;
         QLabel *lbCurrentJob;
+        QLabel *lbGfsUpdateTime;
+		
+        QLabel *lbFnmocWW3_RunDate;
+        QLabel *lbFnmocWW3_UpdateTime;
+        QLabel *lbFnmocWW3_CurrentJob;
+		
+        QLabel *lbFnmocWW3_Med_RunDate;
+        QLabel *lbFnmocWW3_Med_UpdateTime;
+        QLabel *lbFnmocWW3_Med_CurrentJob;
+		
+        QLabel *lbMblueNMM4RunDate;
+        QLabel *lbMblueNMM4UpdateTime;
+        QLabel *lbMblueNMM4CurrentJob;
         
+        QLabel *lbClient;
+        QLabel *lbMessage;
         QFrame * createFrameGui(QWidget *parent);
 };
 
