@@ -1,6 +1,6 @@
 /**********************************************************************
 zyGrib: meteorological GRIB file viewer
-Copyright (C) 2008-2010 - Jacques Zaninetti - http://www.zygrib.org
+Copyright (C) 2008-2012 - Jacques Zaninetti - http://www.zygrib.org
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -30,12 +30,12 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <QRadioButton>
 #include <QLineEdit>
 
-#include "DialogBox.h"
+#include "DialogBoxBase.h"
 
-class DialogUnits : public DialogBox
+class DialogUnits : public DialogBoxBase
 { Q_OBJECT
     public:
-        DialogUnits();
+        DialogUnits (QWidget *parent=NULL);
     
     public slots:
         void slotBtOK();
@@ -52,6 +52,7 @@ class DialogUnits : public DialogBox
         QPushButton *btCancel;
 
         QComboBox *cbWindSpeedUnit;
+        QComboBox *cbCurrentSpeedUnit;
         QComboBox *cbTempUnit;
         QComboBox *cbPositionUnit;
         QComboBox *cbDistanceUnit;
