@@ -1,6 +1,6 @@
 /**********************************************************************
 zyGrib: meteorological GRIB file viewer
-Copyright (C) 2008-2010 - Jacques Zaninetti - http://www.zygrib.org
+Copyright (C) 2008-2012 - Jacques Zaninetti - http://www.zygrib.org
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 /*************************
 
-Download GRIB File on zygrib.org
+Download GRIB File on zygrib server
 
 *************************/
 
@@ -34,7 +34,7 @@ Download GRIB File on zygrib.org
 class FileLoaderGRIB : public QObject, FileLoader
 { Q_OBJECT
     public:
-        FileLoaderGRIB();
+        FileLoaderGRIB (QNetworkAccessManager *manager, QWidget *parent);
         ~FileLoaderGRIB();
         
         void getGribFile(
@@ -42,30 +42,31 @@ class FileLoaderGRIB : public QObject, FileLoader
 				float resolution, int interval, int days,
 				bool wind, bool pressure, bool rain,
 				bool cloud, bool temp, bool humid, bool isotherm0,
-				bool tempPot, bool tempMin, bool tempMax, bool snowDepth,
+				bool tempMin, bool tempMax, bool snowDepth,
 				bool snowCateg, bool frzRainCateg,
-				bool CAPEsfc, 
+				bool CAPEsfc, bool CINsfc,
 				bool altitudeData200,
 				bool altitudeData300,
+				bool altitudeData400,
 				bool altitudeData500,
+				bool altitudeData600,
 				bool altitudeData700,
-				bool altitudeData850
+				bool altitudeData850,
+				bool altitudeData925,
+				bool skewTData,
+				bool cloudLayers,
+				bool GUSTsfc,
+				bool SUNSDsfc
 			);
         void stop();
         
     private:
-        QHttp   *http;
-        QString host;
 		QString scriptpath;
 		QString scriptname;
 		QString scriptstock;
         QByteArray arrayContent;
+        QWidget *parent;
         
-        int     idFileName;    // id des requêtes
-        int     idGribContent;
-        bool	fileLoaded;
-        QString  statusErrorString;
-        QString GribContent;
         QString fileName;
         QString checkSumSHA1;
         int     step;
@@ -74,19 +75,22 @@ class FileLoaderGRIB : public QObject, FileLoader
         QString zygriblog;
         QString zygribpwd;
 
-        QBuffer    ioBuffer;
+		QNetworkReply *reply_step1;
+		QNetworkReply *reply_step2;
+		bool downloadError;
 
     public slots:
-        void requestFinished (int id, bool error);
-        void dataReadProgress (int done, int total );
-        void slotDone (bool error);
+        void downloadProgress (qint64 done, qint64 total);
+		void slotNetworkError (QNetworkReply::NetworkError);
+		void slotFinished_step1 ();
+		void slotFinished_step2 ();
 
     signals:
-        void signalGribDataReceived(QByteArray *content, QString);
-        void signalGribReadProgress(int step, int done, int total);
-        void signalGribSendMessage(QString msg);
-        void signalGribStartLoadData();
-        void signalGribLoadError(QString msg);
+        void signalGribDataReceived (QByteArray *content, QString);
+        void signalGribReadProgress (int step, int done, int total);
+        void signalGribSendMessage (QString msg);
+        void signalGribStartLoadData ();
+        void signalGribLoadError (QString msg);
 };
 
 
