@@ -1,6 +1,6 @@
 /**********************************************************************
 zyGrib: meteorological GRIB file viewer
-Copyright (C) 2008 - Jacques Zaninetti - http://www.zygrib.org
+Copyright (C) 2008-2012 - Jacques Zaninetti - http://www.zygrib.org
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -32,10 +32,12 @@ Lecture mise en mémoire d'un fichier GRIB
 class GribReader : public RegularGridReader
 {
     public:
-        GribReader();
-        GribReader(const std::string fname);
-        ~GribReader();
-
+        GribReader ();
+        ~GribReader ();
+		
+        void  openFile (const std::string fname,
+						LongTaskProgress *taskProgress);
+		
 		virtual FileDataType getReaderFileDataType () 
 					{return DATATYPE_GRIB;};
 
@@ -44,6 +46,11 @@ class GribReader : public RegularGridReader
         
         GribRecord * getFirstGribRecord ();
         GribRecord * getFirstGribRecord (DataCode dtc);
+		
+        virtual time_t  getFirstRefDate ();
+        virtual time_t  getRefDateForData (const DataCode &dtc);
+		virtual time_t  getRefDateForDataCenter (const DataCenterModel &dcm);
+		
 		virtual GriddedRecord *getFirstRecord () 
 							{return getFirstGribRecord();};
 		
@@ -73,30 +80,36 @@ class GribReader : public RegularGridReader
 		void  copyFirstCumulativeRecord   (DataCode dtc);
 		void  removeFirstCumulativeRecord (DataCode dtc);
 
-		virtual bool hasAltitudeData ()  {return hasAltitude;}
-		bool  hasAmbiguousHeader ()  {return ambiguousHeader;}
+		void  copyMissingWaveRecords   ();
+		void  removeMissingWaveRecords ();
+		void  copyMissingWaveRecords   (DataCode dtc);
+
+		virtual bool hasAltitudeData () const  {return hasAltitude;}
+		bool    hasAmbiguousHeader ()  {return ambiguousHeader;}
 
     private:
         ZUFILE *file;
         int	   dewpointDataStatus;
 		bool   hasAltitude;
 		bool   ambiguousHeader;
-
+		
         std::map < std::string,
         		   std::vector<GribRecord *>* >  mapGribRecords;
 
-        void storeRecordInMap(GribRecord *rec);
+        void storeRecordInMap (GribRecord *rec);
 		
-        void   openFile(const std::string fname);
-		void   readGribFileContent();
-		void   readAllGribRecords();
-        void   createListDates();
+        void   openFile (const std::string fname);
+		void   readGribFileContent (int nbrecs);
+		void   readAllGribRecords  (int nbrecs);
+        void   createListDates ();
+		int    countGribRecords (ZUFILE *f);
+		LongTaskProgress *taskProgress;
         
         void clean_vector(std::vector<GribRecord *> &ls);
         void clean_all_vectors();
         std::vector<GribRecord *> * getFirstNonEmptyList();
 		
-		double computeDewPoint(double lon, double lat, time_t date);
+		double   computeDewPoint(double lon, double lat, time_t date);
 
 		// Interpolation entre 2 GribRecord
 		double 	get2GribsInterpolatedValueByDate (
