@@ -1,6 +1,6 @@
 /**********************************************************************
 zyGrib: meteorological GRIB file viewer
-Copyright (C) 2008-2010 - Jacques Zaninetti - http://www.zygrib.org
+Copyright (C) 2008-2012 - Jacques Zaninetti - http://www.zygrib.org
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -16,22 +16,24 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ***********************************************************************/
 
+#include <time.h>
+
 #include "GribRecord.h"
 
 //-------------------------------------------------------------------------------
-// Adjust data type from different mete center
+// Adjust data type from different meteo center
 //-------------------------------------------------------------------------------
-void  GribRecord::translateDataType()
+void  GribRecord::translateDataType ()
 {
 	this->knownData = true;
-
 	//------------------------
 	// NOAA GFS
 	//------------------------
 	if (   idCenter==7
 		&& (idModel==96 || idModel==81)		// NOAA
-		&& (idGrid==4 || idGrid==255))		// Saildocs, Ocens
+		&& (idGrid==3|| idGrid==4 || idGrid==255))	// Saildocs, Ocens
 	{
+		dataCenterModel = NOAA_GFS;
 		if (dataType == GRB_PRECIP_TOT) {	// mm/period -> mm/h
 			if (periodP2 > periodP1)
 				multiplyAllData( 1.0/(periodP2-periodP1) );
@@ -40,6 +42,16 @@ void  GribRecord::translateDataType()
 			if (periodP2 > periodP1)
 				multiplyAllData( 3600.0 );
 		}
+		// NOAA GFS product table differs from NCEP WW3 product table
+		// data: http://nomads.ncdc.noaa.gov/data/gfsanl/
+		if ((idCenter==7 && idModel==81 && idGrid==3)) {
+			if (   dataType==GRB_WAV_MAX_DIR
+				|| dataType==GRB_WAV_MAX_PER
+				|| dataType==GRB_WAV_MAX_HT
+			)
+				dataType = GRB_TYPE_NOT_DEFINED;
+		}
+		
 	}
 	//------------------------
 	// NOAA Waves
@@ -49,9 +61,22 @@ void  GribRecord::translateDataType()
 			 || (idCenter==7 && idModel==124 && idGrid==253)  // enp.all.grb
 			 || (idCenter==7 && idModel==123 && idGrid==244)  // nah.all.grb
 			 || (idCenter==7 && idModel==125 && idGrid==253)  // nph.all.grb
-			 || (idCenter==7 && idModel==88 && idGrid==233)	  // nwww3.all.grb
+			 || (idCenter==7 && idModel==88 && idGrid==233)   // nwww3.all.grb
 			 || (idCenter==7 && idModel==121 && idGrid==238)  // wna.all.grb
+			 || (idCenter==7 && idModel==88 && idGrid==255)   // saildocs
 	) {
+		dataCenterModel = NOAA_NCEP_WW3;
+	}
+	//------------------------
+	// http://retro.met.no/data/maritim/DNMI-NEurope.grb
+	//------------------------
+	else if (   
+			    (idCenter==88 && idModel==255 && idGrid==255)
+			 || (idCenter==88 && idModel==230 && idGrid==255)
+			 || (idCenter==88 && idModel==200 && idGrid==255)
+			 || (idCenter==88 && idModel==67 && idGrid==255)
+	) {
+		dataCenterModel = NORWAY_METNO;
 	}
 	//------------------------
 	// WRF NMM grib.meteorologic.net
@@ -68,6 +93,28 @@ void  GribRecord::translateDataType()
 		}
 	}
 	//----------------------------------------------
+	// NCEP sea surface temperature
+	//----------------------------------------------
+	else if ((idCenter==7 && idModel==44 && idGrid==173)
+	        || (idCenter==7 && idModel==44 && idGrid==235))
+	{
+		dataCenterModel = NOAA_NCEP_SST;
+	}
+	//----------------------------------------------
+	// FNMOC WW3 mediterranean sea
+	//----------------------------------------------
+	else if (idCenter==58 && idModel==111 && idGrid==179)
+	{
+		dataCenterModel = FNMOC_WW3_MED;
+	}
+	//----------------------------------------------
+	// FNMOC WW3
+	//----------------------------------------------
+	else if (idCenter==58 && idModel==110 && idGrid==240)
+	{
+		dataCenterModel = FNMOC_WW3_MED;
+	}
+	//----------------------------------------------
 	// Meteorem (Scannav)
 	//----------------------------------------------
 	else if (idCenter==59 && idModel==78 && idGrid==255)
@@ -87,6 +134,19 @@ void  GribRecord::translateDataType()
 			levelValue = 0;
 		}
 	}
+	//----------------------------------------------
+	// NOGAPS
+	//----------------------------------------------
+	else if (idCenter==58 && idModel==58 && idGrid==240) // NOGAPS
+	{
+		dataCenterModel = NOGAPS;
+		if (getDataType()==133) {
+			DBG ("FIXME SEA TEMP");
+			dataType = GRB_TEMP;
+			levelType  = LV_GND_SURF;
+			levelValue = 0;
+		}
+	}
 	//------------------------------------------
 	// Others recognized grib suppliers
 	//------------------------------------------
@@ -94,6 +154,13 @@ void  GribRecord::translateDataType()
 		(idCenter==255 && idModel==1 && idGrid==255) //http://www.navcenter.com/grb/
 	 || (idCenter==7   && idModel==96 && idGrid==3)    // Maxsea
 	 || (idCenter==255 && idModel==255 && idGrid==255) // Maxsea: tide current
+	 || (idCenter==7   && idModel==127 && idGrid==255) // Meteoconsult
+	 || (idCenter==255 && idModel==220 && idGrid==255) // Actimar
+	 || (idCenter==7   && idModel==45  && idGrid==255) // saildocs.com RTOFS GulfStream
+	 || (idCenter==58 && idModel==22 && idGrid==179) // COAMPS
+	 || (idCenter==58 && idModel==22 && idGrid==158) // COAMPS
+	 || (idCenter==58 && idModel==22 && idGrid==255) // COAMPS via Saildocs
+	 || (idCenter==54 && idModel==47 && idGrid==255) // Canada GEM
 	) {
 	}
 	//------------------------
@@ -101,21 +168,40 @@ void  GribRecord::translateDataType()
 	//------------------------
 	else
 	{
-		this->print("translateDataType() : Unknown GribRecord");
+		this->print("translateDataType() : Unknown Center GribRecord");
 		this->knownData = false;
 	}
 	
+	//===================================================================
 	if (this->knownData) {
-		if ( (getDataType()==GRB_WIND_VX || getDataType()==GRB_WIND_VY)
-			&& getLevelType()== LV_ABOV_GND
-			&& getLevelValue()== 0)
-		{
-			levelType  = LV_GND_SURF;
+		switch (getDataType()) {
+			case GRB_WAV_SIG_HT:
+			case GRB_WAV_WND_DIR:
+			case GRB_WAV_WND_HT:
+			case GRB_WAV_WND_PER:
+			case GRB_WAV_SWL_DIR:
+			case GRB_WAV_SWL_HT:
+			case GRB_WAV_SWL_PER:
+			case GRB_WAV_PRIM_DIR:
+			case GRB_WAV_PRIM_PER:
+			case GRB_WAV_SCDY_DIR:
+			case GRB_WAV_SCDY_PER:
+			case GRB_WAV_WHITCAP_PROB:
+			case GRB_WAV_MAX_DIR:
+			case GRB_WAV_MAX_PER:
+			case GRB_WAV_MAX_HT:
+				waveData = true;
+				levelType  = LV_GND_SURF;
+				levelValue = 0;
+				break;
+			default:
+				waveData = false;
+		}
+		if (getLevelType()== LV_GND_SURF) {
 			levelValue = 0;
 		}
-		else if ( (getDataType()==GRB_WIND_VX || getDataType()==GRB_WIND_VY)
-			&& getLevelType()== LV_GND_SURF)
-		{
+		if (getLevelType()== LV_ABOV_GND && getLevelValue()== 0) {
+			levelType  = LV_GND_SURF;
 			levelValue = 0;
 		}
 	}
@@ -164,7 +250,7 @@ bool GribRecord::verticalDataAreMirrored() {
 		}
 	}
 	else {
-		print ("verticalDataAreMirrored()");
+		//print ("verticalDataAreMirrored()");
 	}
 	
 	return false;
@@ -177,9 +263,9 @@ fprintf(stderr,"====== GribRecord %d : %s\n", id, title);
 fprintf(stderr,"idCenter=%d idModel=%d idGrid=%d\n", idCenter,idModel,idGrid);
 fprintf(stderr,"dataType=%d levelType=%d levelValue=%d\n", dataType, levelType,levelValue);
 fprintf(stderr,"hour=%g\n", (curDate-refDate)/3600.0);
-fprintf(stderr,"sav   xmin=%f xmax=%f    ymin=%f ymax=%f\n", savXmin,savXmax,savYmin,savYmax);
+fprintf(stderr,"sav   xmin=%f xmax=%f    ymin=%f ymax=%f   savDi=%f savDj=%f\n", savXmin,savXmax,savYmin,savYmax, savDi,savDj);
 fprintf(stderr,"final xmin=%f xmax=%f    ymin=%f ymax=%f\n", xmin,xmax,ymin,ymax);
-fprintf(stderr,"Ni=%d Nj=%d\n", Ni,Nj);
+fprintf(stderr,"Ni=%d Nj=%d    entireWorldInLongitude=%d\n", Ni,Nj, (int)entireWorldInLongitude);
 fprintf(stderr,"hasDiDj=%d Di,Dj=(%f %f)\n", hasDiDj, savDi,savDj);
 fprintf(stderr,"final     Di,Dj=(%f %f)\n", Di,Dj);
 fprintf(stderr,"hasBMS=%d isScanIpositive=%d isScanJpositive=%d isAdjacentI=%d\n",
@@ -189,7 +275,7 @@ fprintf(stderr,"hasBMS=%d isScanIpositive=%d isScanJpositive=%d isAdjacentI=%d\n
 //-------------------------------------------------------------------------------
 // Lecture depuis un fichier
 //-------------------------------------------------------------------------------
-GribRecord::GribRecord(ZUFILE* file, int id_)
+GribRecord::GribRecord (ZUFILE* file, int id_)
 {
     id = id_;
     seekStart = zu_tell(file);
@@ -200,29 +286,33 @@ GribRecord::GribRecord(ZUFILE* file, int id_)
 	knownData = true;
 	editionNumber = 0;
 	verticalOrientationIsAmbiguous = false;
+	setDuplicated (false);
+	waveData = false;
+	dataCenterModel = OTHER_DATA_CENTER;
 
-    ok = readGribSection0_IS(file);
+    ok = readGribSection0_IS (file);
     if (ok) {
-        ok = readGribSection1_PDS(file);
+        ok = readGribSection1_PDS (file);
         zu_seek(file, fileOffset1+sectionSize1, SEEK_SET);
     }
     if (ok) {
-        ok = readGribSection2_GDS(file);
+        ok = readGribSection2_GDS (file);
         zu_seek(file, fileOffset2+sectionSize2, SEEK_SET);
     }
     if (ok) {
-        ok = readGribSection3_BMS(file);
+        ok = readGribSection3_BMS (file);
         zu_seek(file, fileOffset3+sectionSize3, SEEK_SET);
     }
     if (ok) {
-        ok = readGribSection4_BDS(file);
+        ok = readGribSection4_BDS (file);
         zu_seek(file, fileOffset4+sectionSize4, SEEK_SET);
     }
     if (ok) {
-        ok = readGribSection5_ES(file);
+        ok = readGribSection5_ES (file);
     }
+    
     if (ok) {
-        zu_seek(file, seekStart+totalSize, SEEK_SET);
+//        zu_seek (file, seekStart+totalSize, SEEK_SET);
     }
 	
 	if (ok && hasBMS) {
@@ -240,14 +330,22 @@ GribRecord::GribRecord(ZUFILE* file, int id_)
     if (ok) {
 		translateDataType ();
 		setDataType (dataType);
+		entireWorldInLongitude = fabs(xmax-360.0+Di-xmin) < fabs(Di/20);
+		//DBG("%g %g %g %g %d : ", xmin, xmax, Di, fabs(xmax-360.0+Di-xmin),(int)entireWorldInLongitude);
+		//this->print("");
+	}
+	else {
+		//this->print("");
 	}
 }
 //-------------------------------------------------------------------------------
 // Constructeur de recopie
 //-------------------------------------------------------------------------------
-GribRecord::GribRecord(const GribRecord &rec)
+GribRecord::GribRecord (const GribRecord &rec)
+	: RegularGridRecord ()
 {
     *this = rec;
+	setDuplicated (true);
     // recopie les champs de bits
     if (rec.data != NULL) {
         int size = rec.Ni*rec.Nj;
@@ -292,13 +390,21 @@ GribRecord::~GribRecord()
 //------------------------------------------------------------------------------
 void  GribRecord::checkOrientation ()
 {
-	if (!ok || !data || ymin==ymax  ||  xmin==xmax
+	if (!ok || !data || ymin==ymax
 		|| Ni<=1 || Nj<=1
 	) {
 		ok = false;
 		return;
 	}
-	double v;
+	if (xmin==xmax) {
+		if (Di >= 0)
+			xmin = xmin-360.0;
+		else
+			xmin = xmin+360.0;
+		Di = 360.0/Ni;
+	}
+		
+		double v;
 	if (xmin > xmax) 
 	{
 		reverseData ('H');
@@ -396,14 +502,18 @@ void  GribRecord::multiplyAllData(double k)
 //----------------------------------------------
 // SECTION 0: THE INDICATOR SECTION (IS)
 //----------------------------------------------
-bool GribRecord::readGribSection0_IS(ZUFILE* file) {
+bool GribRecord::readGribSection0_IS (ZUFILE* file) {
     char strgrib[4];
     fileOffset0 = zu_tell(file);
 
-	// Cherche le 1er 'G'
+// DBG ("START ftell=%ld %lx", zu_tell (file), zu_tell (file));
+	// Cherche le 1er 'G' de 'GRIB'
+	strgrib[0] = 'X';
 	while ( (zu_read(file, strgrib, 1) == 1)
 		   		&&  (strgrib[0] != 'G') )
-	{ }
+	{ 
+// 		fprintf(stderr,"%02X ", (unsigned char) strgrib[0]);
+	}
 
     if (strgrib[0] != 'G') {
         ok = false;
@@ -415,11 +525,6 @@ bool GribRecord::readGribSection0_IS(ZUFILE* file) {
         eof = true;
         return false;
     }
-/*    if (zu_read(file, strgrib, 4) != 4) {
-        ok = false;
-        eof = true;
-        return false;
-    }*/
     if (strncmp(strgrib, "GRIB", 4) != 0)  {
         erreur("readGribSection0_IS(): Unknown file header : %c%c%c%c",
                     strgrib[0],strgrib[1],strgrib[2],strgrib[3]);
@@ -549,7 +654,7 @@ bool GribRecord::readGribSection2_GDS(ZUFILE* file) {
 		Di = (xmax-xmin) / (Ni-1);
 		Dj = (ymax-ymin) / (Nj-1);
 	}
-
+// printf ("Ni=%d Nj=%d\n", Ni, Nj);
 //print ("readGribSection2_GDS");	
 
     return ok;
@@ -593,11 +698,13 @@ bool GribRecord::readGribSection4_BDS(ZUFILE* file) {
     nbBitsInPack = readChar(file);			// byte 11
     scaleFactorEpow2 = pow(2,scaleFactorE);
     unusedBitsEndBDS = flags & 0x0F;
-    isGridData      = (flags&0x80) ==0;
-    isSimplePacking = (flags&0x80) ==0;
-    isFloatValues   = (flags&0x80) ==0;
+	
+    isGridData      = (flags&0x80) == 0;
+    isSimplePacking = (flags&0x40) == 0;
+    isFloatValues   = (flags&0x20) == 0;
+    hasAdditionalFlags = (flags&0x10) != 0;
 
-//printf("BDS type=%3d - bits=%02d - level %3d - %d\n", dataType, nbBitsInPack, levelType,levelValue);
+// printf("BDS type=%3d - bits=%02d - level %3d - %d\n", dataType, nbBitsInPack, levelType,levelValue);
 
     if (! isGridData) {
         erreur("Record %d: need grid data",id);
@@ -611,6 +718,10 @@ bool GribRecord::readGribSection4_BDS(ZUFILE* file) {
         erreur("Record %d: need double values",id);
         ok = false;
     }
+    if (hasAdditionalFlags) {
+        erreur("Record %d: can't process additional flags",id);
+        ok = false;
+    }
 
     if (!ok) {
         return ok;
@@ -626,6 +737,11 @@ bool GribRecord::readGribSection4_BDS(ZUFILE* file) {
     zuint  startbit  = 0;
     int  datasize = sectionSize4-11;
     zuchar *buf = new zuchar[datasize+4];  // +4 pour simplifier les décalages ds readPackedBits
+	
+	// to make valgrind happy
+	for (int i=datasize; i<datasize+4; i++)
+		buf[i] = 0;
+	
     if (!buf) {
         erreur("Record %d: out of memory",id);
         ok = false;
@@ -638,6 +754,7 @@ bool GribRecord::readGribSection4_BDS(ZUFILE* file) {
     if (!ok) {
         return ok;
     }
+    
 
     // Read data in the order given by isAdjacentI
     int i, j, x;
@@ -695,15 +812,19 @@ bool GribRecord::readGribSection4_BDS(ZUFILE* file) {
 //----------------------------------------------
 // SECTION 5: END SECTION (ES)
 //----------------------------------------------
-bool GribRecord::readGribSection5_ES(ZUFILE* file) {
+bool GribRecord::readGribSection5_ES (ZUFILE* file) {
+	
+// DBG ("7777? ftell=%ld %lx", zu_tell (file), zu_tell (file));
+
     char str[4];
     if (zu_read(file, str, 4) != 4) {
+		DBG("can't read 4 chars");
         ok = false;
         eof = true;
         return false;
     }
     if (strncmp(str, "7777", 4) != 0)  {
-        erreur("Final 7777 not read: %c%c%c%c",str[0],str[1],str[2],str[3]);
+        DBG ("Final 7777 not read: %c%c%c%c",str[0],str[1],str[2],str[3]);
         ok = false;
         return false;
     }
@@ -882,32 +1003,61 @@ zuint GribRecord::periodSeconds(zuchar unit,zuchar P1,zuchar P2,zuchar range) {
 double GribRecord::getInterpolatedValue (double px, double py, bool interpolate) const
 {
     double val; 
+	double eps = 1e-4;
+    double pi, pj;     // coord. in grid unit
+    // 00 10      point is in a square
+    // 01 11
+    int i0, j0, i1, j1;
+	double ddx, ddy;
+	
     if (!ok || Di==0 || Dj==0) {
         return GRIB_NOTDEF;
     }
-    if (!isPointInMap(px,py)) {
-        px += 360.0;               // tour du monde à droite ?
-        if (!isPointInMap(px,py)) {
-            px -= 2*360.0;              // tour du monde à gauche ?
-            if (!isPointInMap(px,py)) {
-                return GRIB_NOTDEF;
-            }
-        }
+    if (!isYInMap(py)) {
+		return GRIB_NOTDEF;
     } 
-    double pi, pj;     // coord. in grid unit
-    pi = (px-xmin)/Di;
-    pj = (py-ymin)/Dj;
-    // 00 10      point is in a square
-    // 01 11
-    int i0 = (int) floor(pi);  // point 00
-    int j0 = (int) floor(pj);
-
+    if (!isXInMap(px)) {
+		if (! entireWorldInLongitude) {
+			px += 360.0;               // tour du monde à droite ?
+			if (!isXInMap(px)) {
+				px -= 2*360.0;              // tour du monde à gauche ?
+				if (!isXInMap(px)) {
+					return GRIB_NOTDEF;
+				}
+			}
+			pi = (px-xmin)/Di;
+			i0 = (int) floor(pi);  // point 00
+			i1 = i0+1;
+		}
+		else {
+			while (px< 0)
+				px += 360;
+			if (px <= xmax) {
+				pi = (px-xmin)/Di;
+				i0 = (int) floor(pi);  // point 00
+				i1 = i0+1;
+			}
+			else {
+				pi = (px-xmin)/Di;
+				i0 = (int) floor(pi);  // point 00
+				i1 = 0;
+			}
+		}
+    } 
+    else {
+		pi = (px-xmin)/Di;
+		i0 = (int) floor(pi);  // point 00
+		i1 = i0+1;
+	}
+	pj = (py-ymin)/Dj;
+	j0 = (int) floor(pj);
+	j1 = j0+1;
+	
 	// value very close to a grid point ?
-	double eps = 1e-4;
-	double ddx = fabs (pi-i0);
-	double ddy = fabs (pj-j0);
-	int ii = (ddx<eps) ? i0 : ((1-ddx)<eps) ? i0+1 : -1;
-	int jj = (ddy<eps) ? j0 : ((1-ddy)<eps) ? j0+1 : -1;
+	ddx = fabs (pi-i0);
+	ddy = fabs (pj-j0);
+	int ii = (ddx<eps) ? i0 : ((1-ddx)<eps) ? i1 : -1;
+	int jj = (ddy<eps) ? j0 : ((1-ddy)<eps) ? j1 : -1;
 	if (ii>=0 && jj>=0) {
 		if (hasValue(ii,jj))
 			return getValue (ii, jj);
@@ -917,13 +1067,13 @@ double GribRecord::getInterpolatedValue (double px, double py, bool interpolate)
 
     bool h00,h01,h10,h11;
     int nbval = 0;     // how many values in grid ?
-    if ((h00=hasValue(i0,   j0)))
+    if ((h00=hasValue(i0, j0)))
         nbval ++;
-    if ((h10=hasValue(i0+1, j0)))
+    if ((h10=hasValue(i1, j0)))
         nbval ++;
-    if ((h01=hasValue(i0,   j0+1)))
+    if ((h01=hasValue(i0, j1)))
         nbval ++;
-    if ((h11=hasValue(i0+1, j0+1)))
+    if ((h11=hasValue(i1, j1)))
         nbval ++;
 
     if (nbval <3) {
@@ -938,15 +1088,15 @@ double GribRecord::getInterpolatedValue (double px, double py, bool interpolate)
 	{
 		if (dx < 0.5) {
 			if (dy < 0.5)
-				val = getValue(i0,   j0);
+				val = getValue(i0, j0);
 			else
-				val = getValue(i0,   j0+1);
+				val = getValue(i0, j1);
 		}
 		else {
 			if (dy < 0.5)
-				val = getValue(i0+1,   j0);
+				val = getValue(i1, j0);
 			else
-				val = getValue(i0+1,   j0+1);
+				val = getValue(i1, j1);
 		}
 		return val;
 	}
@@ -963,10 +1113,10 @@ double GribRecord::getInterpolatedValue (double px, double py, bool interpolate)
     // ky = distance(xa,y)
     if (nbval == 4)
     { 
-        double x00 = getValue(i0,   j0);
-        double x01 = getValue(i0,   j0+1);
-        double x10 = getValue(i0+1, j0);
-        double x11 = getValue(i0+1, j0+1);
+        double x00 = getValue(i0, j0);
+        double x01 = getValue(i0, j1);
+        double x10 = getValue(i1, j0);
+        double x11 = getValue(i1, j1);
         double x1 = (1.0-dx)*x00 + dx*x10;
         double x2 = (1.0-dx)*x01 + dx*x11;
         val =  (1.0-dy)*x1 + dy*x2;
@@ -976,33 +1126,33 @@ double GribRecord::getInterpolatedValue (double px, double py, bool interpolate)
         // here nbval==3, check the corner without data
         if (!h00) {
             //printf("! h00  %f %f\n", dx,dy);
-            xa = getValue(i0+1, j0+1);   // A = point 11
-            xb = getValue(i0,   j0+1);   // B = point 01
-            xc = getValue(i0+1, j0);     // C = point 10
+            xa = getValue(i1, j1);   // A = point 11
+            xb = getValue(i0, j1);   // B = point 01
+            xc = getValue(i1, j0);   // C = point 10
             kx = 1-dx;
             ky = 1-dy;
         }
         else if (!h01) {
             //printf("! h01  %f %f\n", dx,dy);
-            xa = getValue(i0+1, j0);     // A = point 10
-            xb = getValue(i0+1, j0+1);   // B = point 11
-            xc = getValue(i0,   j0);     // C = point 00
+            xa = getValue(i1, j0);   // A = point 10
+            xb = getValue(i1, j1);   // B = point 11
+            xc = getValue(i0, j0);   // C = point 00
             kx = dy;
             ky = 1-dx;
         }
         else if (!h10) {
             //printf("! h10  %f %f\n", dx,dy);
-            xa = getValue(i0,   j0+1);     // A = point 01
-            xb = getValue(i0,   j0);       // B = point 00
-            xc = getValue(i0+1, j0+1);     // C = point 11
+            xa = getValue(i0, j1);     // A = point 01
+            xb = getValue(i0, j0);     // B = point 00
+            xc = getValue(i1, j1);     // C = point 11
             kx = 1-dy;
             ky = dx;
         }
         else {
             //printf("! h11  %f %f\n", dx,dy);
-            xa = getValue(i0,   j0);    // A = point 00
-            xb = getValue(i0+1, j0);    // B = point 10
-            xc = getValue(i0,   j0+1);  // C = point 01
+            xa = getValue(i0, j0);  // A = point 00
+            xb = getValue(i1, j0);  // B = point 10
+            xc = getValue(i0, j1);  // C = point 01
             kx = dx;
             ky = dy;
         }
