@@ -1,6 +1,6 @@
 /**********************************************************************
 zyGrib: meteorological GRIB file viewer
-Copyright (C) 2008-2010 - Jacques Zaninetti - http://www.zygrib.org
+Copyright (C) 2008-2012 - Jacques Zaninetti - http://www.zygrib.org
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ***********************************************************************/
 
-// Analyse MeteoBlue data file
+// Analyse Meteoblue data file
 
 #include <iostream>
 #include <list>
@@ -29,18 +29,116 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "MblueRecord.h"
 #include "Util.h"
+#include "DataQString.h" 
 
+int comptepts = 0;
+int comptedestroy = 0;
 
-
-//===============================================================================
 //===============================================================================
 MbluePoint::MbluePoint () 
 {
 	ok = false;
-	for (int i=0; i<MB_NBVALUES; i++) {
-		values[i] = GRIB_NOTDEF;
+}
+//--------------------------------------------------------------------------------
+MbluePoint::~MbluePoint () 
+{
+/*comptedestroy ++;
+if (comptedestroy%1000 == 0) DBG("destroy %d", comptedestroy);*/
+	mapvalues.clear();
+}
+//--------------------------------------------------------------------------------
+MbluePoint::MbluePoint (
+				const std::vector <uint32_t> datacodes, 
+				const MbzLine *mbzline)
+{
+// comptepts ++; 
+// if (comptepts%1000 == 0) DBG("nb=%d", comptepts);
+// mbzline->print();
+
+	ok = true;
+	// position
+	x = mbzline->x;
+	y = mbzline->y;
+	if (x<-360 || x>360 || y<-90 || y>90)
+		ok = false;
+	// data
+	//DBGS ("=======================");
+	for (size_t i=0; i<datacodes.size(); i++) 
+	{
+		if (i < mbzline->data.size()) 
+		{
+			uint32_t code = datacodes [i];
+			float    val  = mbzline->data [i];
+			DataCode dtc (code);
+			//DBG ("%s : %f", qPrintable(DataCodeStr::toString(dtc)), val );
+			//DBGN (dtc.dataType);
+			
+			// adjust some values
+			switch (dtc.dataType) {
+				case GRB_TEMP:
+				case GRB_TEMP_POT:
+				case GRB_TMAX:
+				case GRB_TMIN:
+				case GRB_DEWPOINT:
+					val += 273.15;	// kelvin
+					break;
+				case GRB_HUMID_REL:
+				case GRB_CLOUD_TOT:
+					if (val<0 || val>100)
+							val = GRIB_NOTDEF;
+					break;
+				case GRB_PRESSURE_MSL:
+					if (val<84000 || val>112000)
+							val = GRIB_NOTDEF;
+					break;
+				case GRB_WIND_GUST:
+					code = DataCode(GRB_WIND_GUST,LV_GND_SURF,0).toInt32();
+					break;
+			}
+			mapvalues.insert ( std::pair<uint32_t, float>(code,val) );
+		}
+		else
+			ok = false;
+	}
+	
+// DBGN ( getValue(DataCode(GRB_PRESSURE_MSL,LV_MSL,0)) );
+	//------------------------------------
+	// Computed data
+	//------------------------------------
+	// dewpoint
+	if (ok 
+		&& getValue (DataCode(GRB_DEWPOINT,LV_ABOV_GND,2)) == GRIB_NOTDEF) 
+	{
+		float t  = getValue (DataCode(GRB_TEMP,LV_ABOV_GND,2));
+		float rh = getValue (DataCode(GRB_HUMID_REL,LV_ABOV_GND,2));
+		if (t != GRIB_NOTDEF && rh != GRIB_NOTDEF) {
+			float val = DataRecordAbstract::dewpointHardy (t, rh);
+			DataCode dtcd (GRB_DEWPOINT,LV_ABOV_GND,2);
+			if (val != GRIB_NOTDEF) {
+				mapvalues.insert ( std::pair<uint32_t, float>(dtcd.toInt32(),val) );
+			}
+		}
+	}
+	// total cloud cover
+	if (ok  
+		&& getValue (DataCode(GRB_CLOUD_TOT,LV_ATMOS_ALL,0)) == GRIB_NOTDEF)
+	{
+		float ch = getValue (DataCode(GRB_CLOUD_TOT,LV_CLOUD_HIG_LAYER,0));
+		float cm = getValue (DataCode(GRB_CLOUD_TOT,LV_CLOUD_MID_LAYER,0));
+		float cl = getValue (DataCode(GRB_CLOUD_TOT,LV_CLOUD_LOW_LAYER,0));
+		
+		if (ch != GRIB_NOTDEF && cm != GRIB_NOTDEF && cl != GRIB_NOTDEF) {
+			float val  = 100 * (1 - (1-ch/100)*(1-cm/100)*(1-cl/100));
+			if (val>100) val=100;
+			else if (val<0) val=0;
+			DataCode dtcd (GRB_CLOUD_TOT,LV_ATMOS_ALL,0);
+			if (val != GRIB_NOTDEF) {
+				mapvalues.insert ( std::pair<uint32_t, float>(dtcd.toInt32(),val) );
+			}
+		}
 	}
 }
+
 //--------------------------------------------------------------------------------
 MbluePoint::MbluePoint ( double px, double py,
 					 int  nbPoints,
@@ -49,15 +147,25 @@ MbluePoint::MbluePoint ( double px, double py,
 					 MbluePoint *p3,
 					 MbluePoint *p4 )
 {
+// comptepts ++; 
+// if (comptepts%1000 == 0) DBG("nb=%d", comptepts);
+
 	ok = true;
 	double d1, d2, d3, d4;
 	double v1, v2, v3, v4;
 	double k1, k2, k3, k4;
 	
-	if (nbPoints < 2) {
+	DataCode dtc;
+	uint32_t   code;
+	float val;
+	std::map <uint32_t, float>::iterator iter;
+	
+	if (nbPoints == 1) {
 		ok = false;
-		for (int i=0; i<MB_NBVALUES; i++) {
-			values[i] = GRIB_NOTDEF;
+		for (iter=p1->mapvalues.begin(); iter!=p1->mapvalues.end(); iter++) {
+			code = iter->first;
+			dtc = DataCode(code);
+			mapvalues.insert ( std::pair<uint32_t, float> (code, GRIB_NOTDEF) );
 		}
 		return;
 	}
@@ -70,15 +178,17 @@ MbluePoint::MbluePoint ( double px, double py,
 			k2 = 1.0/(d2*d2+1e-12);
 			k3 = 1.0/(d3*d3+1e-12);
 			k4 = 1.0/(d4*d4+1e-12);
-			for (int i=0; i<MB_NBVALUES; i++) {
-				v1 = p1->getValue (i);
-				v2 = p2->getValue (i);
-				v3 = p3->getValue (i);
-				v4 = p4->getValue (i);
+			for (iter=p1->mapvalues.begin(); iter!=p1->mapvalues.end(); iter++) {
+				dtc = iter->first;
+				v1 = p1->getValue (dtc);
+				v2 = p2->getValue (dtc);
+				v3 = p3->getValue (dtc);
+				v4 = p4->getValue (dtc);
 				if (v1==GRIB_NOTDEF || v2==GRIB_NOTDEF || v3==GRIB_NOTDEF || v4==GRIB_NOTDEF)
-					values[i] = GRIB_NOTDEF;
+					val = GRIB_NOTDEF;
 				else
-					values[i] =  (k1*v1 + k2*v2 + k3*v3 + k4*v4)/(k1+k2+k3+k4);
+					val =  (k1*v1 + k2*v2 + k3*v3 + k4*v4)/(k1+k2+k3+k4);
+				mapvalues.insert ( std::pair<uint32_t, float>(dtc.toInt32(), val) );
 			}
 	}
 	else if (nbPoints==3) {
@@ -88,14 +198,16 @@ MbluePoint::MbluePoint ( double px, double py,
 			k1 = 1.0/(d1*d1+1e-12);
 			k2 = 1.0/(d2*d2+1e-12);
 			k3 = 1.0/(d3*d3+1e-12);
-			for (int i=0; i<MB_NBVALUES; i++) {
-				v1 = p1->getValue (i);
-				v2 = p2->getValue (i);
-				v3 = p3->getValue (i);
+			for (iter=p1->mapvalues.begin(); iter!=p1->mapvalues.end(); iter++) {
+				dtc = iter->first;
+				v1 = p1->getValue (dtc);
+				v2 = p2->getValue (dtc);
+				v3 = p3->getValue (dtc);
 				if (v1==GRIB_NOTDEF || v2==GRIB_NOTDEF || v3==GRIB_NOTDEF)
-					values[i] = GRIB_NOTDEF;
+					val = GRIB_NOTDEF;
 				else
-					values[i] =  (k1*v1 + k2*v2 + k3*v3)/(k1+k2+k3);
+					val =  (k1*v1 + k2*v2 + k3*v3)/(k1+k2+k3);
+				mapvalues.insert ( std::pair<uint32_t, float>(dtc.toInt32(), val) );
 			}
 	}
 	else if (nbPoints==2) {
@@ -103,106 +215,31 @@ MbluePoint::MbluePoint ( double px, double py,
 			d2 = (px-p2->x)*(px-p2->x)+(py-p2->y)*(py-p2->y);
 			k1 = 1.0/(d1*d1+1e-12);
 			k2 = 1.0/(d2*d2+1e-12);
-			for (int i=0; i<MB_NBVALUES; i++) {
-				v1 = p1->getValue (i);
-				v2 = p2->getValue (i);
+			for (iter=p1->mapvalues.begin(); iter!=p1->mapvalues.end(); iter++) {
+				dtc = iter->first;
+				v1 = p1->getValue (dtc);
+				v2 = p2->getValue (dtc);
 				if (v1==GRIB_NOTDEF || v2==GRIB_NOTDEF)
-					values[i] = GRIB_NOTDEF;
+					val = GRIB_NOTDEF;
 				else
-					values[i] =  (k1*v1 + k2*v2)/(k1+k2);
+					val =  (k1*v1 + k2*v2)/(k1+k2);
+				mapvalues.insert ( std::pair<uint32_t, float>(dtc.toInt32(), val) );
 			}
 	}
 }
-
-//--------------------------------------------------------------------------------
-MbluePoint::MbluePoint (ZUFILE *file, int nbdata) 
-{
-	ok = true;
-	nbvalues = nbdata;
-	for (int i=0; i<MB_NBVALUES; i++) {
-		values[i] = GRIB_NOTDEF;
-	}
-	int nb =0;
-	if (nb<nbdata)  { read_float32 (file, &x); nb++; }
-	if (nb<nbdata)  { read_float32 (file, &y); nb++; }
-	if (nb<nbdata)  { read_float32 (file, &values [MB_temp2m]); nb++; }
-	if (nb<nbdata)  { read_float32 (file, &values [MB_rh2m]); nb++; }
-	if (nb<nbdata)  { read_float32 (file, &values [MB_precip]); nb++; }
-	if (nb<nbdata)  { read_float32 (file, &values [MB_windu10m]); nb++; } 
-	if (nb<nbdata)  { read_float32 (file, &values [MB_windv10m]); nb++; }
-	if (nb<nbdata)  { read_float32 (file, &values [MB_clouds]); nb++; } 
-	if (nb<nbdata)  { read_float32 (file, &values [MB_gust]); nb++; }
-	if (nb<nbdata)  { read_float32 (file, &values [MB_pmsl]); nb++; } 
-	if (nb<nbdata)  { read_float32 (file, &values [MB_tempsfc]); nb++; } 
-	if (nb<nbdata)  { read_float32 (file, &values [MB_rainprob]); nb++; }
-	float tmp;
-	for ( ; nb<nbdata; nb++)
-		read_float32 (file, &tmp);	// flush data if necessary
-
-	// Adjust values
-	if (x<-360 || x>360 || y<-90 || y>90)
-		ok = false;
-	
-	if (values[MB_temp2m] != GRIB_NOTDEF)
-		values[MB_temp2m] += 273.15;	// kelvin
-	if (values[MB_tempsfc] != GRIB_NOTDEF)
-		values[MB_tempsfc] += 273.15;	// kelvin
-
-	if (values[MB_rh2m]<0 || values[MB_rh2m]>100)
-			values[MB_rh2m] = GRIB_NOTDEF;
-	if (values[MB_clouds]<0 || values[MB_clouds]>100)
-			values[MB_clouds] = GRIB_NOTDEF;
-	if (values[MB_rainprob]<0 || values[MB_rainprob]>100)
-			values [MB_rainprob] = GRIB_NOTDEF;
-	
-	if (values[MB_pmsl]<84000 || values[MB_pmsl]>112000)
-			values[MB_pmsl] = GRIB_NOTDEF;
-	
-	// Compute dew point with the Magnus-Tetens formula
-	if (values[MB_temp2m] != GRIB_NOTDEF && values[MB_rh2m] != GRIB_NOTDEF)
-		values[MB_dewpoint2m] 
-				= DataRecordAbstract::dewpointMagnusTetens (values[MB_temp2m], values[MB_rh2m]);
-}
-
 //-------------------------------------------------------------
-int MbluePoint::getDataIndex (DataCode dtc)
+bool   MbluePoint::hasData  (const DataCode &dtc) const
 {
-	if (dtc.equals (GRB_WIND_VX, LV_ABOV_GND, 10))
-		return MB_windu10m;
-	else if (dtc.equals (GRB_WIND_VY, LV_ABOV_GND, 10))
-		return MB_windv10m;
-	else if (dtc.equals (GRB_PRESSURE, LV_MSL, 0))
-		return MB_pmsl;
-	else if (dtc.equals (GRB_CLOUD_TOT, LV_ATMOS_ALL, 0))
-		return MB_clouds;
-	else if (dtc.equals (GRB_PRECIP_TOT, LV_GND_SURF, 0))
-		return MB_precip;
-	else if (dtc.equals (GRB_TEMP, LV_ABOV_GND, 2))
-		return MB_temp2m;
-	else if (dtc.equals (GRB_TEMP, LV_GND_SURF, 0))
-		return MB_tempsfc;
-	else if (dtc.equals (GRB_HUMID_REL, LV_ABOV_GND, 2))
-		return MB_rh2m;
-	else if (dtc.equals (GRB_DEWPOINT, LV_ABOV_GND, 2))
-		return MB_dewpoint2m;
-	else {
-		//DBG("unknown data %d %d %d", dataType,levelType,levelValue);
-		return -1;
-	}
+	return (mapvalues.find (dtc.toInt32()) != mapvalues.end());
 }	
 
 //-------------------------------------------------------------
-double MbluePoint::getValue (DataCode dtc)
+double MbluePoint::getValue (const DataCode &dtc) const
 {
-	return getValue (getDataIndex (dtc));
-}	
-
-//-------------------------------------------------------------
-double MbluePoint::getValue (int index)
-{
-	if (index>=0 && index<MB_NBVALUES) {
-		return values [index];
-	}
+	std::map <uint32_t, float>::const_iterator iter;
+	iter = mapvalues.find (dtc.toInt32());
+	if (iter != mapvalues.end())
+		return iter->second;
 	else
 		return GRIB_NOTDEF;
 }	
@@ -216,29 +253,67 @@ double MbluePoint::getValue (int index)
 //===========================================================
 //===========================================================
 //===========================================================
-MblueRecord::MblueRecord (ZUFILE *file)
+MblueRecord::MblueRecord (const MbzFile &mbzfile, 
+						  time_t curDate,
+						  bool fastInterpolation 
+						 )
 {
+	ok = false;
+	this->fastInterpolation = fastInterpolation;
 	clusters = NULL;
 	regularGrid = NULL;
 	smoothPressureGrid = NULL;
+	dataCenterModel = METEOBLUE_NMM4;
 	Ni = 0;
 	Nj = 0;
-	if (file != NULL) 
+	if (mbzfile.isOk()) 
 	{
-		ok = readFile (file);
-		if (ok) {
-			makeClusters ();
-			DBG("nb points : %d", allPoints.size());
-			makeVirtualRegularGrid ();
-			makeSmoothPressureGrid ();
-		}
+		this->refDate = UTC_mktime (mbzfile.year,mbzfile.month,mbzfile.day,mbzfile.href,0,0);
+		this->curDate = curDate;
+		xmin = mbzfile.xmin;
+		xmax = mbzfile.xmax;
+		ymin = mbzfile.ymin;
+		ymax = mbzfile.ymax;
+		ok = true;
 	}
 }
+//--------------------------------------------------------------	
+void MblueRecord::finalize ()
+{
+	if (ok) {
+		makeClusters ();
+		//DBG("nb points : %d", allPoints.size());
+		makeVirtualRegularGrid ();
+		makeSmoothPressureGrid ();
+// 		if (clusters != NULL) {
+// 			delete [] clusters;
+// 			clusters = NULL;
+// 		}
+	}
+}
+//--------------------------------------------------------------	
+void MblueRecord::addMbluePoint (MbluePoint *pt)
+{
+	allPoints.push_back (pt);
+}
+//--------------------------------------------------------------	
+bool MblueRecord::hasData (const DataCode &dtc) const
+{
+	if (allPoints.size() == 0)
+		return false;
+	
+	MbluePoint *pt = *(allPoints.begin());
+	return pt->hasData (dtc);
+	
+}
 //-----------------------------------------------------
 MblueRecord::~MblueRecord ()
 {
-	if (clusters != NULL)
+// DBGS("Destroy MblueRecord");	
+	if (clusters != NULL) {
 		delete [] clusters;
+		clusters = NULL;
+	}
 	if (regularGrid != NULL) {
 		for (int i=0; i<Ni*Nj; i++)
 				delete regularGrid [i];
@@ -246,7 +321,8 @@ MblueRecord::~MblueRecord ()
 	}
 	if (smoothPressureGrid != NULL)
 		delete [] smoothPressureGrid;
-	Util::cleanListPointers (allPoints);
+	
+	Util::cleanVectorPointers (allPoints);
 }
 //-----------------------------------------------------
 void MblueRecord::makeSmoothPressureGrid ()
@@ -257,20 +333,19 @@ void MblueRecord::makeSmoothPressureGrid ()
 	assert (smoothPressureGrid);
 	int i, j;
 	float v0,v1,v2,v3,v4,v5,v6,v7,v8;
-	DataCode dtc (GRB_PRESSURE,LV_MSL,0);
-	int ind = MbluePoint::getDataIndex (dtc);
+	DataCode dtc (GRB_PRESSURE_MSL,LV_MSL,0);
 	
     for (j=1; j<Nj-1; j++) {
         for (i=1; i<Ni-1; i++) {
-            v0 = getValueOnRegularGrid (ind, i, j );
-            v1 = getValueOnRegularGrid (ind, i-1, j-1 );
-            v2 = getValueOnRegularGrid (ind, i  , j-1 );
-            v3 = getValueOnRegularGrid (ind, i+1, j-1 );
-            v4 = getValueOnRegularGrid (ind, i-1, j );
-            v5 = getValueOnRegularGrid (ind, i+1, j );
-            v6 = getValueOnRegularGrid (ind, i-1, j+1 );
-            v7 = getValueOnRegularGrid (ind, i  , j+1 );
-            v8 = getValueOnRegularGrid (ind, i+1, j+1 );
+            v0 = getRawPressureOnRegularGrid (i, j );
+            v1 = getRawPressureOnRegularGrid (i-1, j-1 );
+            v2 = getRawPressureOnRegularGrid (i  , j-1 );
+            v3 = getRawPressureOnRegularGrid (i+1, j-1 );
+            v4 = getRawPressureOnRegularGrid (i-1, j );
+            v5 = getRawPressureOnRegularGrid (i+1, j );
+            v6 = getRawPressureOnRegularGrid (i-1, j+1 );
+            v7 = getRawPressureOnRegularGrid (i  , j+1 );
+            v8 = getRawPressureOnRegularGrid (i+1, j+1 );
 			if (   v0 != GRIB_NOTDEF  && v1 != GRIB_NOTDEF  && v2 != GRIB_NOTDEF
 				&& v3 != GRIB_NOTDEF  && v4 != GRIB_NOTDEF  && v5 != GRIB_NOTDEF  
 				&& v6 != GRIB_NOTDEF  && v7 != GRIB_NOTDEF  && v8 != GRIB_NOTDEF 
@@ -284,12 +359,12 @@ void MblueRecord::makeSmoothPressureGrid ()
 	}
     j = 0;
 	for (i=1; i<Ni-1; i++) {
-		v0 = getValueOnRegularGrid (ind, i, j );
-		v4 = getValueOnRegularGrid (ind, i-1, j );
-		v5 = getValueOnRegularGrid (ind, i+1, j );
-		v6 = getValueOnRegularGrid (ind, i-1, j+1 );
-		v7 = getValueOnRegularGrid (ind, i  , j+1 );
-		v8 = getValueOnRegularGrid (ind, i+1, j+1 );
+		v0 = getRawPressureOnRegularGrid (i, j );
+		v4 = getRawPressureOnRegularGrid (i-1, j );
+		v5 = getRawPressureOnRegularGrid (i+1, j );
+		v6 = getRawPressureOnRegularGrid (i-1, j+1 );
+		v7 = getRawPressureOnRegularGrid (i  , j+1 );
+		v8 = getRawPressureOnRegularGrid (i+1, j+1 );
 		if (v0 != GRIB_NOTDEF  && v4 != GRIB_NOTDEF && v5 != GRIB_NOTDEF  
 				&& v6 != GRIB_NOTDEF && v7 != GRIB_NOTDEF  && v8 != GRIB_NOTDEF)
 			smoothPressureGrid [i+j*Ni] = (v0 +v4+v5+v6+v7+v8) / 6.0;
@@ -298,12 +373,12 @@ void MblueRecord::makeSmoothPressureGrid ()
 	}
     j = Nj-1;
 	for (i=1; i<Ni-1; i++) {
-		v0 = getValueOnRegularGrid (ind, i, j );
-		v1 = getValueOnRegularGrid (ind, i-1, j-1 );
-		v2 = getValueOnRegularGrid (ind, i  , j-1 );
-		v3 = getValueOnRegularGrid (ind, i+1, j-1 );
-		v4 = getValueOnRegularGrid (ind, i-1, j );
-		v5 = getValueOnRegularGrid (ind, i+1, j );
+		v0 = getRawPressureOnRegularGrid (i, j );
+		v1 = getRawPressureOnRegularGrid (i-1, j-1 );
+		v2 = getRawPressureOnRegularGrid (i  , j-1 );
+		v3 = getRawPressureOnRegularGrid (i+1, j-1 );
+		v4 = getRawPressureOnRegularGrid (i-1, j );
+		v5 = getRawPressureOnRegularGrid (i+1, j );
 		if (v0 != GRIB_NOTDEF  && v1 != GRIB_NOTDEF  && v2 != GRIB_NOTDEF
 				&& v3 != GRIB_NOTDEF  && v4 != GRIB_NOTDEF  && v5 != GRIB_NOTDEF)
 			smoothPressureGrid [i+j*Ni] = (v0 +v1+v2+v3+v4+v5) / 6.0;
@@ -312,12 +387,12 @@ void MblueRecord::makeSmoothPressureGrid ()
 	}
 	i = 0;
     for (j=1; j<Nj-1; j++) {
-		v0 = getValueOnRegularGrid (ind, i, j );
-		v2 = getValueOnRegularGrid (ind, i  , j-1 );
-		v3 = getValueOnRegularGrid (ind, i+1, j-1 );
-		v5 = getValueOnRegularGrid (ind, i+1, j );
-		v7 = getValueOnRegularGrid (ind, i  , j+1 );
-		v8 = getValueOnRegularGrid (ind, i+1, j+1 );
+		v0 = getRawPressureOnRegularGrid (i, j );
+		v2 = getRawPressureOnRegularGrid (i  , j-1 );
+		v3 = getRawPressureOnRegularGrid (i+1, j-1 );
+		v5 = getRawPressureOnRegularGrid (i+1, j );
+		v7 = getRawPressureOnRegularGrid (i  , j+1 );
+		v8 = getRawPressureOnRegularGrid (i+1, j+1 );
 		if (v0 != GRIB_NOTDEF  && v2 != GRIB_NOTDEF  && v3 != GRIB_NOTDEF
 				&& v5 != GRIB_NOTDEF  && v7 != GRIB_NOTDEF  && v8 != GRIB_NOTDEF)
 			smoothPressureGrid [i+j*Ni] = (v0 +v2+v3+v5+v7+v8) / 6.0;
@@ -326,12 +401,12 @@ void MblueRecord::makeSmoothPressureGrid ()
 	}
 	i = Ni-1;
     for (j=1; j<Nj-1; j++) {
-		v0 = getValueOnRegularGrid (ind, i, j );
-		v1 = getValueOnRegularGrid (ind, i-1, j-1 );
-		v2 = getValueOnRegularGrid (ind, i  , j-1 );
-		v4 = getValueOnRegularGrid (ind, i-1, j );
-		v6 = getValueOnRegularGrid (ind, i-1, j+1 );
-		v7 = getValueOnRegularGrid (ind, i  , j+1 );
+		v0 = getRawPressureOnRegularGrid (i, j );
+		v1 = getRawPressureOnRegularGrid (i-1, j-1 );
+		v2 = getRawPressureOnRegularGrid (i  , j-1 );
+		v4 = getRawPressureOnRegularGrid (i-1, j );
+		v6 = getRawPressureOnRegularGrid (i-1, j+1 );
+		v7 = getRawPressureOnRegularGrid (i  , j+1 );
 		if (v0 != GRIB_NOTDEF  && v1 != GRIB_NOTDEF  && v2 != GRIB_NOTDEF
 				&& v4 != GRIB_NOTDEF  && v6 != GRIB_NOTDEF  && v7 != GRIB_NOTDEF)
 			smoothPressureGrid [i+j*Ni] = (v0 +v1+v2+v4+v6+v7) / 6.0;
@@ -340,30 +415,30 @@ void MblueRecord::makeSmoothPressureGrid ()
 	}
 	i = 0;
 	j = 0;
-		v0 = getValueOnRegularGrid (ind, i, j );
-		v5 = getValueOnRegularGrid (ind, i+1, j );
-		v7 = getValueOnRegularGrid (ind, i  , j+1 );
-		v8 = getValueOnRegularGrid (ind, i+1, j+1 );
+		v0 = getRawPressureOnRegularGrid (i, j );
+		v5 = getRawPressureOnRegularGrid (i+1, j );
+		v7 = getRawPressureOnRegularGrid (i  , j+1 );
+		v8 = getRawPressureOnRegularGrid (i+1, j+1 );
 		if (v0 != GRIB_NOTDEF && v5 != GRIB_NOTDEF && v7 != GRIB_NOTDEF && v8 != GRIB_NOTDEF)
 			smoothPressureGrid [i+j*Ni] = (v0 +v5+v7+v8) / 4.0;
 		else
 			smoothPressureGrid [i+j*Ni] = GRIB_NOTDEF;
 	i = Ni-1;
 	j = 0;
-		v0 = getValueOnRegularGrid (ind, i, j );
-		v4 = getValueOnRegularGrid (ind, i-1, j );
-		v6 = getValueOnRegularGrid (ind, i-1, j+1 );
-		v7 = getValueOnRegularGrid (ind, i  , j+1 );
+		v0 = getRawPressureOnRegularGrid (i, j );
+		v4 = getRawPressureOnRegularGrid (i-1, j );
+		v6 = getRawPressureOnRegularGrid (i-1, j+1 );
+		v7 = getRawPressureOnRegularGrid (i  , j+1 );
 		if (v0 != GRIB_NOTDEF && v4 != GRIB_NOTDEF && v6 != GRIB_NOTDEF && v7 != GRIB_NOTDEF)
 			smoothPressureGrid [i+j*Ni] = (v0 +v4+v6+v7) / 4.0;
 		else
 			smoothPressureGrid [i+j*Ni] = GRIB_NOTDEF;
 	i = 0;
 	j = Nj-1;
-		v0 = getValueOnRegularGrid (ind, i, j );
-		v2 = getValueOnRegularGrid (ind, i  , j-1 );
-		v3 = getValueOnRegularGrid (ind, i+1, j-1 );
-		v5 = getValueOnRegularGrid (ind, i+1, j );
+		v0 = getRawPressureOnRegularGrid (i, j );
+		v2 = getRawPressureOnRegularGrid (i  , j-1 );
+		v3 = getRawPressureOnRegularGrid (i+1, j-1 );
+		v5 = getRawPressureOnRegularGrid (i+1, j );
 		if (v0 != GRIB_NOTDEF && v2 != GRIB_NOTDEF && v3 != GRIB_NOTDEF && v5 != GRIB_NOTDEF)
 			smoothPressureGrid [i+j*Ni] = (v0 +v2+v3+v5) / 4.0;
 		else
@@ -371,10 +446,10 @@ void MblueRecord::makeSmoothPressureGrid ()
 		
 	i = Ni-1;
 	j = Nj-1;
-		v0 = getValueOnRegularGrid (ind, i, j );
-		v1 = getValueOnRegularGrid (ind, i-1, j-1 );
-		v2 = getValueOnRegularGrid (ind, i  , j-1 );
-		v4 = getValueOnRegularGrid (ind, i-1, j );
+		v0 = getRawPressureOnRegularGrid (i, j );
+		v1 = getRawPressureOnRegularGrid (i-1, j-1 );
+		v2 = getRawPressureOnRegularGrid (i  , j-1 );
+		v4 = getRawPressureOnRegularGrid (i-1, j );
 		if (v0 != GRIB_NOTDEF && v1 != GRIB_NOTDEF  && v2 != GRIB_NOTDEF && v4 != GRIB_NOTDEF)
 			smoothPressureGrid [i+j*Ni] = (v0 +v1+v2+v4) / 4.0;
 		else
@@ -387,8 +462,9 @@ void MblueRecord::makeVirtualRegularGrid ()
 		return;
 	double dens = allPoints.size() / ((xmax-xmin)*(ymax-ymin));
 	double dt = 1 / sqrt (dens);
-	if (dt<0.1)
-			dt = 0.1;
+// 	dt *= 1.5;		// minimize grid size
+	if (dt<0.01)
+			dt = 0.01;
 	// Virtual grid of size dt x dt
 	Ni = (int) ceil ((xmax-xmin)/dt);
 	Nj = (int) ceil ((ymax-ymin)/dt);
@@ -400,7 +476,7 @@ void MblueRecord::makeVirtualRegularGrid ()
 	Di = (xmax-xmin)/(Ni-1) - 1e-12;
 	Dj = (ymax-ymin)/(Nj-1) - 1e-12;
 	
-	DBG("Ni=%d, Nj=%d,  Ni*Nj=%d,   Di=%f Dj=%f ", Ni, Nj, Ni*Nj, Di,Dj);
+// 	DBG("Ni=%d, Nj=%d,  Ni*Nj=%d,   Di=%f Dj=%f ", Ni, Nj, Ni*Nj, Di,Dj);
 
 	regularGrid = new MbluePoint* [Ni*Nj];
 	assert (regularGrid);
@@ -423,19 +499,25 @@ void MblueRecord::makeVirtualRegularGrid ()
 //-----------------------------------------------------
 void MblueRecord::makeClusters ()
 {
-	int clustersize = 3;	// mean number of points in each cluster
+	int clustersize = 20;	// mean number of points in each cluster
 	int nbclusters = allPoints.size()/clustersize;
 	double k = (xmax-xmin)/(ymax-ymin);
-	clustersNBY = (int) (sqrt (nbclusters/k));
-	clustersNBX = (int) (k*clustersNBY);
-	if (clustersNBX<1)
-			clustersNBX = 1;
-	if (clustersNBY<1)
-			clustersNBY = 1;
-DBG("clustersNBX %d   clustersNBY %d  tot=%d", clustersNBX,clustersNBY, clustersNBX*clustersNBY)	;
-	clusters = new std::list <MbluePoint *> [clustersNBX*clustersNBY];
+	clustersNj = (int) (sqrt (nbclusters/k));
+	clustersNi = (int) (k*clustersNj);
+	if (clustersNi<1)
+			clustersNi = 1;
+	if (clustersNj<1)
+			clustersNj = 1;
+
+	clusterWidth = (xmax-xmin)/clustersNi;
+	clusterHight = (ymax-ymin)/clustersNj;
+	
+// 	DBG("clustersNi=%d   clustersNj=%d  tot=%d", clustersNi,clustersNj, clustersNi*clustersNj)	;
+// 	DBG("nb points=%d    per cluster=%.2f", allPoints.size(), (double)allPoints.size()/(clustersNi*clustersNj))	;
+
+	clusters = new std::vector <MbluePoint *> [clustersNi*clustersNj];
 	assert (clusters);
-	std::list <MbluePoint *>::iterator it;
+	std::vector <MbluePoint *>::iterator it;
 	for (it=allPoints.begin(); it !=allPoints.end(); it++)
 	{
 		MbluePoint *p = *it;
@@ -444,60 +526,118 @@ DBG("clustersNBX %d   clustersNBY %d  tot=%d", clustersNBX,clustersNBY, clusters
 }
 
 //-----------------------------------------------------
-std::list <MbluePoint *> & MblueRecord::getCluster (double x, double y) const
+std::vector <MbluePoint *> & MblueRecord::getCluster (double x, double y) const
 {
-	double dx = (xmax-xmin)/clustersNBX;
-	double dy = (ymax-ymin)/clustersNBY;
+	double dx = (xmax-xmin)/clustersNi;
+	double dy = (ymax-ymin)/clustersNj;
 	int i, j;
 	i = (int) floor((x-xmin)/dx);
 	j = (int) floor((y-ymin)/dy);
 	// avoid rounding errors
 	if (i<0) i=0;
 	if (j<0) j=0;
-	if (i>=clustersNBX) i=clustersNBX-1;
-	if (j>=clustersNBY) j=clustersNBY-1;
-	return clusters[j*clustersNBX+i];
+	if (i>=clustersNi) i=clustersNi-1;
+	if (j>=clustersNj) j=clustersNj-1;
+	return clusters [j*clustersNi+i];
 }
 
 //-----------------------------------------------------
-std::list <std::list <MbluePoint *> > * MblueRecord::getClustersList (
+std::vector <std::vector <MbluePoint *> > * MblueRecord::getClustersList (
 					double x, double y) const
 {
- 	std::list <std::list <MbluePoint *> > *listall;
-	listall = new std::list <std::list <MbluePoint *> >;
-	assert(listall);
-	
-	double dx = (xmax-xmin)/clustersNBX;
-	double dy = (ymax-ymin)/clustersNBY;
+// 	return getClustersList_9max (x,y);
+	return getClustersList_4max (x,y);
+}
+//-----------------------------------------------------
+std::vector <std::vector <MbluePoint *> > * MblueRecord::getClustersList_4max (
+					double x, double y) const
+{
+ 	std::vector <std::vector <MbluePoint *> > *listclusters;
+	listclusters = new std::vector <std::vector <MbluePoint *> >;
+	assert(listclusters);
+	listclusters->reserve (4);
+	double px = (x-xmin)/clusterWidth;
+	double py = (y-ymin)/clusterHight;
 	int i, j;
-	i = (int) floor((x-xmin)/dx);
-	j = (int) floor((y-ymin)/dy);
+	i = (int) floor (px);
+	j = (int) floor (py);
+	double dx = px-i;
+	double dy = py-j;
 	// avoid rounding errors
 	if (i<0) i=0;
+	else if (i>=clustersNi) i=clustersNi-1;
 	if (j<0) j=0;
-	if (i>=clustersNBX) i=clustersNBX-1;
-	if (j>=clustersNBY) j=clustersNBY-1;
+	else if (j>=clustersNj) j=clustersNj-1;
+	//--------------------------------------------------------------
+	listclusters->push_back (clusters[j*clustersNi+i]);
+	//--------------------------------------------------------------
+	double ecmin=0.33;
+	double ecmax=1-ecmin;
+	if (dx>ecmax && i<clustersNi-1) {
+		listclusters->push_back (clusters[j*clustersNi+i+1]);
+		if (dy>ecmax && j<clustersNj-1) {
+			listclusters->push_back (clusters[(j+1)*clustersNi+i+1]);
+		}
+		else if (dy<ecmin && j>0) {
+			listclusters->push_back (clusters[(j-1)*clustersNi+i+1]);
+		}
+	}
+	else if (dx<ecmin && i>0) {
+		listclusters->push_back (clusters[j*clustersNi+i-1]);
+		if (dy>ecmax && j<clustersNj-1) {
+			listclusters->push_back (clusters[(j+1)*clustersNi+i-1]);
+		}
+		else if (dy<ecmin && j>0) {
+			listclusters->push_back (clusters[(j-1)*clustersNi+i-1]);
+		}
+	}
+	//--------------------------------------------------------------
+	if (dy>ecmax && j<clustersNj-1) {
+		listclusters->push_back (clusters[(j+1)*clustersNi+i]);
+	}
+	else if (dy<ecmin && j>0) {
+		listclusters->push_back (clusters[(j-1)*clustersNi+i]);
+	}
+	return listclusters;
+}
+//-----------------------------------------------------
+std::vector <std::vector <MbluePoint *> > * MblueRecord::getClustersList_9max (
+					double x, double y) const
+{
+ 	std::vector <std::vector <MbluePoint *> > *listclusters;
+	listclusters = new std::vector <std::vector <MbluePoint *> >;
+	assert(listclusters);
+	listclusters->reserve (9);
+	
+	int i, j;
+	i = (int) floor ((x-xmin)/clusterWidth);
+	j = (int) floor ((y-ymin)/clusterHight);
+	// avoid rounding errors
+	if (i<0) i=0;
+	else if (i>=clustersNi) i=clustersNi-1;
+	if (j<0) j=0;
+	else if (j>=clustersNj) j=clustersNj-1;
 
-	listall->push_back (clusters[j*clustersNBX+i]);
+	listclusters->push_back (clusters[j*clustersNi+i]);
 	if (i>0) 
-		listall->push_back (clusters[j*clustersNBX+(i-1)]);
-	if (i<clustersNBX-1) 
-		listall->push_back (clusters[j*clustersNBX+(i+1)]);
+		listclusters->push_back (clusters[j*clustersNi+(i-1)]);
+	if (i<clustersNi-1) 
+		listclusters->push_back (clusters[j*clustersNi+(i+1)]);
 	if (j>0) 
-		listall->push_back (clusters[(j-1)*clustersNBX+i]);
-	if (j<clustersNBY-1) 
-		listall->push_back (clusters[(j+1)*clustersNBX+i]);
+		listclusters->push_back (clusters[(j-1)*clustersNi+i]);
+	if (j<clustersNj-1) 
+		listclusters->push_back (clusters[(j+1)*clustersNi+i]);
 
 	if (i>0 && j>0) 
-		listall->push_back (clusters[(j-1)*clustersNBX+(i-1)]);
-	if (i<clustersNBX-1 && j<clustersNBY-1) 
-		listall->push_back (clusters[(j+1)*clustersNBX+(i+1)]);
-	if (i>0 && j<clustersNBY-1) 
-		listall->push_back (clusters[(j+1)*clustersNBX+(i-1)]);
-	if (i<clustersNBX-1 && j>0) 
-		listall->push_back (clusters[(j-1)*clustersNBX+(i+1)]);
+		listclusters->push_back (clusters[(j-1)*clustersNi+(i-1)]);
+	if (i<clustersNi-1 && j<clustersNj-1) 
+		listclusters->push_back (clusters[(j+1)*clustersNi+(i+1)]);
+	if (i>0 && j<clustersNj-1) 
+		listclusters->push_back (clusters[(j+1)*clustersNi+(i-1)]);
+	if (i<clustersNi-1 && j>0) 
+		listclusters->push_back (clusters[(j-1)*clustersNi+(i+1)]);
 
-	return listall;
+	return listclusters;
 }
 
 //-----------------------------------------------------
@@ -511,14 +651,15 @@ int  MblueRecord::findNeighbour_clusters (
 	
 	*p1 = *p2 = *p3 = *p4 = NULL;
 
-	std::list <std::list <MbluePoint *> > *clusterslist = getClustersList (lon,lat);
-	std::list <std::list <MbluePoint *> >::iterator it; 
+	std::vector <std::vector <MbluePoint *> > *clusterslist;
+	clusterslist = getClustersList (lon,lat);
+	std::vector <std::vector <MbluePoint *> >::iterator it; 
 	int nb=0;
 	if (clusterslist != NULL) 
 	{
 		for (it=clusterslist->begin(); it!=clusterslist->end(); it++)
 		{
-			std::list <MbluePoint *> cluster = *it;
+			std::vector <MbluePoint *> cluster = *it;
 			nb += findBestNeighboursInCluster (
 						lon, lat, cluster,
 						p1, p2, p3, p4,  
@@ -534,7 +675,7 @@ int  MblueRecord::findNeighbour_clusters (
 //-------------------------------------------------------
 int MblueRecord::findBestNeighboursInCluster (
 						double lon, double lat,
-						std::list <MbluePoint *> cluster,
+						std::vector <MbluePoint *> cluster,
 						MbluePoint **p1, MbluePoint **p2, 
 						MbluePoint **p3, MbluePoint **p4, 
 						double *d1, double *d2, double *d3, double *d4
@@ -542,7 +683,7 @@ int MblueRecord::findBestNeighboursInCluster (
 {	
 	double d;
 	int nb=0;
-	std::list <MbluePoint *>::iterator it;
+	std::vector <MbluePoint *>::iterator it;
 	for (it=cluster.begin(); it !=cluster.end(); it++)
 	{
 		MbluePoint *p = *it;
@@ -586,119 +727,6 @@ int MblueRecord::findBestNeighboursInCluster (
 	return nb;
 }
 
-//-----------------------------------------------------
-bool MblueRecord::read_int32 (ZUFILE *file, int *val)
-{
-	unsigned char buf[16];
-	if (zu_read(file,buf,4) != 4)
-		return false;
-	*val = ((int)(buf[0])<<24) + ((int)(buf[1])<<16) + ((int)(buf[2])<<8) + (int)buf[3];
-	//DBG("%02x %02x %02x %02x -> %d", buf[0],buf[1],buf[2],buf[3], *val);
-	return true;
-}
-
-//-----------------------------------------------------
-bool MbluePoint::read_float32 (ZUFILE *file, float *val)
-{
-	char buf[16];
-	if (zu_read(file,buf,4) != 4)
-		return false;
-	*val = *( (float*)((void*)buf) );
-	return true;
-}
-
-//-----------------------------------------------------
-bool MblueRecord::readFile (ZUFILE *file)
-{
-	char buf[1024];
-	int   year,month,day,href,hcur; 
-	
-	if (zu_read(file,buf,10) != 8)
-		return false;
-	buf[8] = '\0';
-	if (strcmp(buf, "MBZYGRIB") != 0)
-		return false;
-	
-	if (zu_read(file,buf,4) != 4)
-		return false;
-	buf[4] = '\0';
-	year = atoi (buf);
-	if (year <1970 || year > 3333) 	// here is the year 3333 bug
-		return false;
-
-	if (zu_read(file,buf,2) != 2)
-		return false;
-	buf[2] = '\0';
-	month = atoi (buf);
-	if (month <1 || month > 12) 
-		return false;
-
-	if (zu_read(file,buf,2) != 2)
-		return false;
-	buf[2] = '\0';
-	day = atoi (buf);
-	if (day<1 || day> 31) 
-		return false;
-
-	if (zu_read(file,buf,2) != 2)
-		return false;
-	buf[2] = '\0';
-	href = atoi (buf);
-	if (href<0 || href>24) 
-		return false;
-
-	if (zu_read(file,buf,3) != 3)
-		return false;
-	buf[3] = '\0';
-	hcur = atoi (buf);
-	if (hcur<0 || hcur>999) 
-		return false;
-	
-	refDate = UTC_mktime (year,month,day, href,0,0);
-	curDate = UTC_mktime (year,month,day, hcur,0,0);
-
-	if (zu_read(file,buf,8) != 8)
-		return false;
-
-	int nblines, nbdata;
-	if (!read_int32(file, &nblines))
-		return false;
-	if (!read_int32(file, &nbdata))
-		return false;
-	
-	if (nblines<1 || nblines >1000000000) 
-		return false;
-	if (nbdata<1 || nbdata>100) 
-		return false;
-	
-	xmin =  1e6;
-	xmax = -1e6;
-	ymin =  1e6;
-	ymax = -1e6;
-	for (int i=0; i<nblines; i++)
-	{
-		MbluePoint *point = new MbluePoint (file, nbdata);
-		assert (point);
-		if (point->isOk()) {
-			allPoints.push_back (point);
-			if (xmin > point->x) xmin=point->x;
-			if (xmax < point->x) xmax=point->x;
-			if (ymin > point->y) ymin=point->y;
-			if (ymax < point->y) ymax=point->y;
-		}
-		else
-			delete point;
-		
-	}
-	
-	if (   (xmax - xmin) < 0.01
-		|| (ymax - ymin) < 0.01
-		|| allPoints.size() < 4
-	) {
-		ok = false;
-	}
-	return true;
-}	
 
 //--------------------------------------------------------------------
 bool MblueRecord::getZoneExtension (double *x0,double *y0, double *x1,double *y1)
@@ -710,8 +738,8 @@ bool MblueRecord::getZoneExtension (double *x0,double *y0, double *x1,double *y1
 	return ok;
 }
 //--------------------------------------------------------------------
-double  MblueRecord::getInterpolatedValue (
-				int index, 
+double  MblueRecord::getInterpolatedValueWithoutGrid (
+				DataCode dtc, 
 				double px, double py,
 				bool interpolateValues) const
 {
@@ -737,7 +765,7 @@ double  MblueRecord::getInterpolatedValue (
 		return GRIB_NOTDEF;
 	
 	if (! interpolateValues) {
-			return p1->getValue (index);
+			return p1->getValue (dtc);
 	}
 	if (nb==4) {
 			d1 = (px-p1->x)*(px-p1->x)+(py-p1->y)*(py-p1->y);
@@ -748,10 +776,10 @@ double  MblueRecord::getInterpolatedValue (
 			k2 = 1.0/(d2+1e-12);
 			k3 = 1.0/(d3+1e-12);
 			k4 = 1.0/(d4+1e-12);
-			v1 = p1->getValue (index);
-			v2 = p2->getValue (index);
-			v3 = p3->getValue (index);
-			v4 = p4->getValue (index);
+			v1 = p1->getValue (dtc);
+			v2 = p2->getValue (dtc);
+			v3 = p3->getValue (dtc);
+			v4 = p4->getValue (dtc);
 			if (v1!=GRIB_NOTDEF && v2!=GRIB_NOTDEF && v3!=GRIB_NOTDEF && v4!=GRIB_NOTDEF)
 				return (k1*v1 + k2*v2 + k3*v3 + k4*v4)/(k1+k2+k3+k4);
 	}
@@ -762,9 +790,9 @@ double  MblueRecord::getInterpolatedValue (
 			k1 = 1.0/(d1+1e-12);
 			k2 = 1.0/(d2+1e-12);
 			k3 = 1.0/(d3+1e-12);
-			v1 = p1->getValue (index);
-			v2 = p2->getValue (index);
-			v3 = p3->getValue (index);
+			v1 = p1->getValue (dtc);
+			v2 = p2->getValue (dtc);
+			v3 = p3->getValue (dtc);
 			if (v1!=GRIB_NOTDEF && v2!=GRIB_NOTDEF && v3!=GRIB_NOTDEF)
 				return (k1*v1 + k2*v2 + k3*v3)/(k1+k2+k3);
 	}
@@ -773,8 +801,8 @@ double  MblueRecord::getInterpolatedValue (
 			d2 = (px-p2->x)*(px-p2->x)+(py-p2->y)*(py-p2->y);
 			k1 = 1.0/(d1+1e-12);
 			k2 = 1.0/(d2+1e-12);
-			v1 = p1->getValue (index);
-			v2 = p2->getValue (index);
+			v1 = p1->getValue (dtc);
+			v2 = p2->getValue (dtc);
 			if (v1!=GRIB_NOTDEF && v2!=GRIB_NOTDEF)
 				return (k1*v1 + k2*v2)/(k1+k2);
 	}
@@ -787,14 +815,13 @@ double  MblueRecord::getInterpolatedValue (
 				double px, double py,
 				bool interpolate) const
 {
-	bool fastInterpolation = true;
 	if (fastInterpolation)
 			return getInterpolatedValueUsingRegularGrid (
 						dtc, 
 						px, py, interpolate );
 	else
-			return getInterpolatedValue (
-						MbluePoint::getDataIndex (dtc),
+			return getInterpolatedValueWithoutGrid (
+						dtc,
 						px, py, interpolate );
 
 }
@@ -804,7 +831,7 @@ double MblueRecord::getValueOnRegularGrid (DataCode dtc, int i, int j) const
 {	
 	if (i>=0 && i<Ni && j>=0 && j<Nj) 
 	{
-		if (dtc.equals (GRB_PRESSURE, LV_MSL, 0)) {
+		if (dtc.equals (GRB_PRESSURE_MSL, LV_MSL, 0)) {
 			return getSmoothPressureMSL (i, j);
 		}
 		else {
@@ -815,18 +842,18 @@ double MblueRecord::getValueOnRegularGrid (DataCode dtc, int i, int j) const
 	else
 		return GRIB_NOTDEF;
 } 
+
 //--------------------------------------------------------------------	
-double MblueRecord::getValueOnRegularGrid (int ind, int i, int j) const
+double MblueRecord::getRawPressureOnRegularGrid (int i, int j) const
 {	
 	if (i>=0 && i<Ni && j>=0 && j<Nj) 
 	{
 		MbluePoint *pt = regularGrid [i+j*Ni];
-		return pt->getValue (ind); 
+		return pt->getValue (DataCode(GRB_PRESSURE_MSL,LV_MSL,0)); 
 	}
-	else {
+	else
 		return GRIB_NOTDEF;
-	}
-}  
+} 
 
 //--------------------------------------------------------------------	
 int     MblueRecord::getNi ()  const
