@@ -1,6 +1,6 @@
 /**********************************************************************
 zyGrib: meteorological GRIB file viewer
-Copyright (C) 2008 - Jacques Zaninetti - http://www.zygrib.org
+Copyright (C) 2008-2012 - Jacques Zaninetti - http://www.zygrib.org
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -26,28 +26,34 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "MeteoTableWidget.h"
 #include "MeteotableOptionsDialog.h"
+#include "SylkFile.h"
 
 //===================================================================
 // MeteoTable : dialog + MeteoTableWidget
 //===================================================================
-class MeteoTableDialog : public QDialog
+class MeteoTableDialog : public QWidget
 { Q_OBJECT
     public:
         MeteoTableDialog (
 					GriddedPlotter *plotter, 
-					double lon, double lat, QString posName="");
+					double lon, double lat, QString locationName="");
         ~MeteoTableDialog();
 		
-		bool saveFileSYLK (QFile &file);
+		void saveFileSYLK (SylkFile &slk);
     
     private:
     	GriddedPlotter *plotter;
     	double   lon, lat;
-    	QString  posName;
+    	QString  locationName;
 		
 		MeteoTableWidget 		*meteoTableWidget;
 		DialogMeteotableOptions *optionsDialog;
 		QScrollArea 			*scrollArea;
+		QWidget                 *createDataTable ();
+		QWidget * dataTable;
+		QVBoxLayout *mainLayout;
+		
+		QScrollArea *hdrscroll;
      	
      	QPushButton *btClose;
      	QPushButton *btOptions;
@@ -55,15 +61,16 @@ class MeteoTableDialog : public QDialog
      	QPushButton *btExport;
 		void closeEvent(QCloseEvent *) {delete this;};
 		
-		bool SYLK_saveFile (QFile &file);
-		int  SYLK_addData_gen  (QTextStream &out, int lig,int col, DataCode dtc);
-		int  SYLK_addData_wind (QTextStream &out, int lig,int col, DataCode dtc);
+		int  SYLK_addData_gen   (SylkFile &slk, int lig,int col, DataCode dtc);
+		int  SYLK_addData_wind  (SylkFile &slk, int lig,int col, DataCode dtc);
+		int  SYLK_addData_current  (SylkFile &slk, int lig,int col, DataCode dtc);
+		int  SYLK_addData_waves (SylkFile &slk, int lig,int col, DataCode dtc);
 		
 	private slots :
-		void slotBtOptions();
-		void slotBtExport();
-		void slotOptionsChanged();
-		void reject()	{delete this;};
+		void slotBtOptions ();
+		void slotBtExport ();
+		void slotOptionsChanged ();
+		void reject ()	{delete this;};
 };
 
 
