@@ -1,6 +1,6 @@
 /**********************************************************************
 zyGrib: meteorological GRIB file viewer
-Copyright (C) 2008-2010 - Jacques Zaninetti - http://www.zygrib.org
+Copyright (C) 2008-2012 - Jacques Zaninetti - http://www.zygrib.org
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -16,31 +16,101 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ***********************************************************************/
 
+#include <QTimer>
+
+#include "Astro.h"
 #include "Settings.h"
 #include "DataQString.h"
 #include "MeteoTableWidget.h"
 
 //-------------------------------------------------------------------------------
-MeteoTableWidget::MeteoTableWidget(GriddedPlotter *plotter, double lon, double lat, QWidget *parent)
+MeteoTableWidget::MeteoTableWidget 
+			(GriddedPlotter *plotter, 
+			 double lon, double lat, QString locationName, QWidget *parent)
 	: QWidget(parent)
 {
 	this->plotter = plotter;
+	this->parent = parent;
 	this->lon = lon;
 	this->lat = lat;
+	this->locationName = locationName;
 
-	layout = new QGridLayout(this);
-	layout->setContentsMargins(0,0,0,0);
-	layout->setSpacing(0);
+	layout = new QGridLayout (this);
+	assert (layout);
+	layout->setContentsMargins (0,0,0,0);
+	layout->setSpacing (0);
+	headerWidget = new QWidget ();
+	assert (headerWidget);
+	headerLayout = new QGridLayout (headerWidget);
+	assert (headerLayout);
+	headerLayout->setContentsMargins (0,0,0,0);
+	headerLayout->setSpacing (0);
 	
 	createTable();
 }
+//----------------------------------------------------------------
+void MeteoTableWidget::addCell_title_dataline (QString txt, bool bold,
+				int lig,int col)
+{
+	QColor bgcolor(200,200,255);
+	TableCell *cell2 = new TableCell (headerWidget, txt, bold, bgcolor,
+										Qt::AlignRight|Qt::AlignHCenter);	
+	assert (cell2);
+	cell2->setBorders (lig, col);
+	headerLayout->addWidget (cell2, lig,col, 1,1);
+}
+//------------------------------------------------------------------------
+QWidget * MeteoTableWidget::getDataHeaders ()
+{
+	return headerWidget;
+}
+//------------------------------------------------------------------------
+void MeteoTableWidget::showEvent (QShowEvent */*event*/)
+{
+	QRect r;
+	// Adjust header height to row height
+	for (int j=0; j<layout->rowCount(); j++) 
+	{
+		QLayoutItem *it1 = layout->itemAtPosition (j,1);
+		if (it1 && it1->widget()) {
+			QLayoutItem *it2 = headerLayout->itemAtPosition (j,0);
+			if (it2 && it2->widget()) 
+			{
+ 				it2->widget()->setMinimumHeight (it1->widget()->height());
+			}
+		}
+	}
+}
 
 //-------------------------------------------------------------------------------
 MeteoTableWidget::~MeteoTableWidget()
 {
-	Util::cleanListPointers (lspinfos);
+	Util::cleanVectorPointers (lspinfos);
 	qDeleteAll (listVisibleData);
 }
+//----------------------------------------------------------------
+void MeteoTableWidget::addCell_SunMoonAlmanac (time_t t, double lat, double lon, 
+				QGridLayout *layout,int lig,int col, int rowspan,int colspan)
+{
+	TableCell *cell = new TableCell (this);	
+	assert (cell);
+	cell->setBorders (lig, col);
+	
+	Almanac alm = Astro::getSunMoonAlmanac (t, lat, lon);
+
+	QString rs,st;
+	QString txt;
+	rs = alm.sunRise>0 ? Util::formatTime (alm.sunRise) : "----";
+	st = alm.sunSet>0 ? Util::formatTime (alm.sunSet) : "----";
+	txt = "⇑ "+rs+" ⇓ "+st+"\n";
+	rs = alm.moonRise>0 ? Util::formatTime (alm.moonRise) : "----";
+	st = alm.moonSet>0 ? Util::formatTime (alm.moonSet) : "----";
+	txt += "⇑ "+rs+" ⇓ "+st;
+	txt += "  "+QString("%1%").arg( (int)(100*alm.moonDisk+0.5));
+	cell->label->setText (txt);
+	layout->setAlignment (Qt::AlignLeft);
+	layout->addWidget (cell, lig,col, rowspan,colspan );
+}
 
 //-------------------------------------------------------------------------------
 void MeteoTableWidget::createTable() 
@@ -54,18 +124,19 @@ void MeteoTableWidget::createTable()
 	std::set<time_t>::iterator iter2;
 	int lig, col, colspan;
 	QString dstr;
+	bool showSunMoonAlmanac = Util::getSetting("MTABLE_showSunMoonAlmanac", true).toBool();
 	//-----------------------------------------------
 	// Titre 1 : une colonne par jour, regroupant plusieurs horaires
 	//-----------------------------------------------
 	col = 0;
 	lig = 0;
-	addCell_title("", true, layout, lig,col, 2,1);
+	addCell_title_dataline ("", true, lig,col);
 	col ++;
 	QString actuel = "";
 	for (iter=sdates.begin(); iter!=sdates.end(); iter++)
 	{
 		time_t daterecord = *iter;
-		dstr = Util::formatDateLong(daterecord);
+		dstr = Util::formatDateLong (daterecord);
 		if (dstr != actuel)
 		{
 			colspan = 0;
@@ -77,18 +148,50 @@ void MeteoTableWidget::createTable()
 				iter2 ++;
 				dstr = Util::formatDateLong(*iter2);
 			} while (actuel==dstr);
-			
-			addCell_title(actuel, true, layout, lig,col, 1,colspan);
+			addCell_title (actuel, true, layout, lig,col, 1,colspan);
 			col += colspan;
 		}
 	}
+			
+	//-----------------------------------------------
+	// Titre 1 bis: sun moon almanac if needed
+	//-----------------------------------------------
+	if (showSunMoonAlmanac)
+	{
+		col = 0;
+		lig ++;
+		addCell_title_dataline (tr("Sun")+"\n"+tr("Moon"), true, lig,col);
+		col ++;
+		QString actuel = "";
+		for (iter=sdates.begin(); iter!=sdates.end(); iter++)
+		{
+			time_t daterecord = *iter;
+			dstr = Util::formatDateLong (daterecord);
+			if (dstr != actuel)
+			{
+				colspan = 0;
+				actuel = dstr;
+				iter2 = iter;
+				do
+				{
+					colspan ++;
+					iter2 ++;
+					dstr = Util::formatDateLong(*iter2);
+				} while (actuel==dstr);
+				addCell_SunMoonAlmanac (daterecord, lat, lon, layout, lig,col, 1,colspan);
+				col += colspan;
+			}
+		}
+	}	
 	//-----------------------------------------------
 	// Titre 2 : une colonne par date+horaires
 	//-----------------------------------------------
 	DataPointInfo *pinfo;
 	time_t dateproche = reader->getClosestDateFromNow ();
-	col = 1;
-	lig = 1;
+	col = 0;
+	lig ++;
+	addCell_title_dataline ("", true, lig,col);
+	col ++;
 	for (iter=sdates.begin(); iter!=sdates.end(); iter++)
 	{
 		time_t date = *iter;
@@ -106,7 +209,6 @@ void MeteoTableWidget::createTable()
 	//-----------------------------------------------
 	lig ++;
 	createListVisibleGribData();
-	
 	QList <MTGribData *>::iterator it;
 	for (it=listVisibleData.begin(); it!=listVisibleData.end(); it++) {
 		MTGribData *gr = *it;
@@ -114,11 +216,15 @@ void MeteoTableWidget::createTable()
 		int levelType  = gr->dtc.levelType;
 		int levelValue = gr->dtc.levelValue;
 		
-		if (dataType==GRB_WIND_XY2D) {
+		if (dataType==GRB_PRV_WIND_XY2D) {
 			Altitude alt (levelType, levelValue);
 			addLine_Wind (alt, lig++);
 		}
-		else if (dataType==GRB_PRESSURE && levelType==LV_MSL && levelValue==0){
+		else if (dataType==GRB_PRV_CUR_XY2D) {
+			Altitude alt (levelType, levelValue);
+			addLine_Current (alt, lig++);
+		}
+		else if (dataType==GRB_PRESSURE_MSL && levelType==LV_MSL && levelValue==0){
 			Altitude alt (levelType, levelValue);
 			addLine_Pressure (alt, lig++);
 		}
@@ -128,11 +234,11 @@ void MeteoTableWidget::createTable()
 		else if (dataType==GRB_PRECIP_TOT && levelType==LV_GND_SURF && levelValue==0){
 			addLine_Rain (lig++);
 		}
-		else if (dataType==GRB_THETA_E) {
+		else if (dataType==GRB_PRV_THETA_E) {
 			Altitude alt (levelType, levelValue);
-			addLine_Temperature (alt, GRB_THETA_E, lig++);
+			addLine_Temperature (alt, GRB_PRV_THETA_E, lig++);
 		}
-		else if (dataType==GRB_TEMP) {
+		else if (dataType==GRB_TEMP && levelType!=LV_ATMOS_ALL) {
 			Altitude alt (levelType, levelValue);
 			addLine_Temperature (alt, GRB_TEMP, lig++);
 		}
@@ -163,9 +269,9 @@ void MeteoTableWidget::createTable()
 			Altitude alt (levelType, levelValue);
 			addLine_DewPoint (alt, lig++);
 		}
-		else if (dataType==GRB_DIFF_TEMPDEW && levelType==LV_ABOV_GND && levelValue==2){
+		else if (dataType==GRB_PRV_DIFF_TEMPDEW && levelType==LV_ABOV_GND && levelValue==2){
 			Altitude alt (levelType, levelValue);
-			addLine_DeltaTemperature (alt, GRB_DIFF_TEMPDEW, lig++);
+			addLine_DeltaTemperature (alt, GRB_PRV_DIFF_TEMPDEW, lig++);
 		}
 		else if (dataType==GRB_SNOW_CATEG && levelType==LV_GND_SURF && levelValue==0){
 			addLine_Categorical (GRB_SNOW_CATEG, lig++);
@@ -179,6 +285,124 @@ void MeteoTableWidget::createTable()
 		else if (dataType==GRB_CAPE && levelType==LV_GND_SURF && levelValue==0){
 			addLine_CAPEsfc (lig++);
 		}
+		else if (dataType==GRB_CIN && levelType==LV_GND_SURF && levelValue==0){
+			addLine_CINsfc (lig++);
+		}
+		else if (dataType==GRB_TEMP && levelType==LV_ATMOS_ALL && levelValue==0){
+			addLine_SkewT (lig++);
+		}
+		else if (dataType==GRB_WIND_GUST && levelType==LV_GND_SURF && levelValue==0){
+			addLine_GUSTsfc (lig++);
+		}
+		//----------------------------------------------
+		// Waves
+		//----------------------------------------------
+		else if (dataType==GRB_WAV_SIG_HT
+				|| dataType==GRB_WAV_WND_HT
+				|| dataType==GRB_WAV_SWL_HT
+				|| dataType==GRB_WAV_MAX_HT
+		){
+			addLine_WaveHeight (dataType, lig++);
+		}
+		else if (dataType==GRB_PRV_WAV_MAX
+				|| dataType==GRB_PRV_WAV_WND
+				|| dataType==GRB_PRV_WAV_SWL
+				|| dataType==GRB_PRV_WAV_PRIM
+				|| dataType==GRB_PRV_WAV_SCDY
+		){
+			addLine_WaveCompleteCell (dataType, lig++);
+		}
+		else if (dataType==GRB_WAV_WHITCAP_PROB) {
+			addLine_WaveWhitecap (dataType, lig++);
+		}
+		
+		//----------------------------------------------
+		// Unknown data
+		//----------------------------------------------
+		else {
+			DBG ("%d %d %d", dataType,levelType,levelValue);
+			DBGQS (DataCodeStr::toString(gr->dtc) );
+		}
+	}
+}
+//-----------------------------------------------------------------
+void MeteoTableWidget::addLine_WaveWhitecap (int type, int lig)
+{
+	std::vector <time_t>::iterator it; 
+	std::vector <DataPointInfo *>::iterator iter; 
+	QColor    bgColor = Qt::white;
+	QString   txt;
+	int col = 0;
+	addCell_title_dataline  (tr("Whitecap (prob)"), true, lig,col);
+	col ++;
+	for (iter=lspinfos.begin(); iter!=lspinfos.end(); iter++, col++)
+	{
+		DataPointInfo * pinfo = *iter;
+		txt = "";
+		double v = pinfo->getWaveData (type);
+		if (v != GRIB_NOTDEF) {
+			txt = Util::formatPercentValue (v);
+			bgColor = QColor(plotter->getWhiteCapColor (v, true));
+		}
+		else
+			bgColor = Qt::white;
+		addCell_content(txt, layout,lig,col, 1,1, bgColor);
+	}
+}
+//-----------------------------------------------------------------
+void MeteoTableWidget::addLine_WaveCompleteCell (int prvtype, int lig)
+{
+	std::vector <DataPointInfo *>::iterator iter;
+	QColor    bgColor = Qt::white;
+	QString   txt;
+	int col = 0;
+	addCell_title_dataline (DataCodeStr::toString(prvtype), true, lig,col);
+	col ++;
+	for (iter=lspinfos.begin(); iter!=lspinfos.end(); iter++, col++)
+	{
+		DataPointInfo * pinfo = *iter;
+		txt = "";
+		float ht, per, dir;
+		pinfo->getWaveValues (prvtype, &ht, &per, &dir);
+		if (ht != GRIB_NOTDEF) {
+			txt = Util::formatWaveHeight (ht);
+			bgColor = QColor(plotter->getWaveHeightColor (ht, true));
+		}
+		else {
+			bgColor = Qt::white;
+		}
+		txt += "\n";
+		if (dir != GRIB_NOTDEF) {
+			txt += Util::formatWaveDirection (dir, true);
+		}
+		txt += "\n";
+		if (per != GRIB_NOTDEF) {
+			txt += Util::formatWavePeriod (per, true);
+		}
+		addCell_content (txt, layout,lig,col, 1,1, bgColor);
+	}
+}
+//-----------------------------------------------------------------
+void MeteoTableWidget::addLine_WaveHeight (int type, int lig)
+{
+	std::vector <DataPointInfo *>::iterator iter;
+	QColor    bgColor = Qt::white;
+	QString   txt;
+	int col = 0;
+	addCell_title_dataline (DataCodeStr::toString(type), true, lig,col);
+	col ++;
+	for (iter=lspinfos.begin(); iter!=lspinfos.end(); iter++, col++)
+	{
+		DataPointInfo * pinfo = *iter;
+		txt = "";
+		double v = pinfo->getWaveData (type);
+		if (v != GRIB_NOTDEF) {
+			txt = Util::formatWaveHeight (v);
+			bgColor = QColor(plotter->getWaveHeightColor (v, true));
+		}
+		else
+			bgColor = Qt::white;
+		addCell_content(txt, layout,lig,col, 1,1, bgColor);
 	}
 }
 
@@ -217,7 +441,7 @@ void MeteoTableWidget::createListVisibleGribData ()
 		// No MTableData_... in settings : initialize first list (arbitrary)
 		int pos = 0;
     	listVisibleData.append( new MTGribData (
-    				DataCode(GRB_WIND_XY2D,LV_ABOV_GND,10).toInt32(), pos++) );
+    				DataCode(GRB_PRV_WIND_XY2D,LV_ABOV_GND,10).toInt32(), pos++) );
     	listVisibleData.append( new MTGribData (
     				DataCode(GRB_CLOUD_TOT,LV_ATMOS_ALL,0).toInt32(), pos++) );
     	listVisibleData.append( new MTGribData (
@@ -225,11 +449,11 @@ void MeteoTableWidget::createListVisibleGribData ()
     	listVisibleData.append( new MTGribData (
     				DataCode(GRB_TEMP,LV_ABOV_GND,2).toInt32(), pos++) );
     	listVisibleData.append( new MTGribData (
-    				DataCode(GRB_DIFF_TEMPDEW,LV_ABOV_GND,2).toInt32(), pos++) );
+    				DataCode(GRB_PRV_DIFF_TEMPDEW,LV_ABOV_GND,2).toInt32(), pos++) );
     	listVisibleData.append( new MTGribData (
     				DataCode(GRB_GEOPOT_HGT,LV_ISOTHERM0,0).toInt32(), pos++) );
     	listVisibleData.append( new MTGribData (
-					DataCode(GRB_PRESSURE,LV_MSL,0).toInt32(), pos++) );
+					DataCode(GRB_PRESSURE_MSL,LV_MSL,0).toInt32(), pos++) );
 	}
 	// Sort visible data by position
 	qSort (listVisibleData.begin(), listVisibleData.end(), lessThanMTGribData);
@@ -237,14 +461,13 @@ void MeteoTableWidget::createListVisibleGribData ()
 //-----------------------------------------------------------------
 void MeteoTableWidget::addLine_Isotherm0Height(int lig)
 {
-	std::list<DataPointInfo *>::iterator iter; 
+	std::vector <DataPointInfo *>::iterator iter; 
 	QColor    bgColor = Qt::white;
 	QString   txt;
 	int col = 0;
-	addCell_title(tr("Isotherm 0°C"), true, layout, lig,col);
+	addCell_title_dataline (tr("Isotherm 0°C"), true, lig,col);
 	col ++;
-	for (iter=lspinfos.begin(); iter!=lspinfos.end(); iter++, col++)
-	{
+	for (iter=lspinfos.begin(); iter!=lspinfos.end(); iter++, col++)	{
 		DataPointInfo * pinfo = *iter;
 		txt = "";
 		if (pinfo->hasIsotherm0HGT()) {
@@ -252,21 +475,22 @@ void MeteoTableWidget::addLine_Isotherm0Height(int lig)
 			bgColor = QColor(plotter->getAltitudeColor
 						(pinfo->isotherm0HGT, Altitude(LV_ISOTHERM0,0), true));
 		}
+		else
+			bgColor = Qt::white;
 		addCell_content(txt, layout,lig,col, 1,1, bgColor);
 	}
 }
 //-----------------------------------------------------------------
 void MeteoTableWidget::addLine_GeopotentialAltitude(const Altitude &alt, int lig)
 {
-	std::list<DataPointInfo *>::iterator iter; 
+	std::vector <DataPointInfo *>::iterator iter; 
 	QColor    bgColor = Qt::white;
 	QString   txt;
 	int col = 0;
-	addCell_title(tr("Geopotential altitude") +" ("+AltitudeStr::toStringShort(alt)+")", 
-				  true, layout, lig,col);
+	addCell_title_dataline (tr("Geopotential altitude") +" ("+AltitudeStr::toStringShort(alt)+")", 
+				  true, lig,col);
 	col ++;
-	for (iter=lspinfos.begin(); iter!=lspinfos.end(); iter++, col++)
-	{
+	for (iter=lspinfos.begin(); iter!=lspinfos.end(); iter++, col++) {
 		DataPointInfo * pinfo = *iter;
 		txt = "";
 		float v = pinfo->getDataValue (DataCode(GRB_GEOPOT_HGT,alt));
@@ -274,101 +498,158 @@ void MeteoTableWidget::addLine_GeopotentialAltitude(const Altitude &alt, int lig
 			txt = Util::formatGeopotAltitude (v);
 			bgColor = QColor(plotter->getAltitudeColor(v, alt, true));
 		}
+		else
+			bgColor = Qt::white;
 		addCell_content(txt, layout,lig,col, 1,1, bgColor);
 	}
 }
 //-----------------------------------------------------------------
 void MeteoTableWidget::addLine_Pressure(const Altitude &alt, int lig)
 {
-	std::list<DataPointInfo *>::iterator iter; 
+	std::vector <DataPointInfo *>::iterator iter; 
 	QColor    bgColor = Qt::white;
 	QString   txt;
 	int col = 0;
-	addCell_title(tr("Pressure") +" ("+AltitudeStr::toStringShort(alt)+")", 
-				  true, layout, lig,col);
+	addCell_title_dataline (tr("Pressure") +" ("+AltitudeStr::toStringShort(alt)+")", 
+				  true, lig,col);
 	col ++;
-	for (iter=lspinfos.begin(); iter!=lspinfos.end(); iter++, col++)
-	{
+	for (iter=lspinfos.begin(); iter!=lspinfos.end(); iter++, col++) {
 		DataPointInfo * pinfo = *iter;
 		txt = "";
-		if (pinfo->hasPressure()) {
-			txt = Util::formatPressure (pinfo->pressure);
-			bgColor = QColor(plotter->getPressureColor(pinfo->pressure, true));
+		if (pinfo->hasPressureMSL()) {
+			txt = Util::formatPressure (pinfo->pressureMSL);
+			bgColor = QColor(plotter->getPressureColor(pinfo->pressureMSL, true));
 		}
+		else
+			bgColor = Qt::white;
 		addCell_content(txt, layout,lig,col, 1,1, bgColor);
 	}
 }
 //-----------------------------------------------------------------
 void MeteoTableWidget::addLine_Wind (const Altitude &alt, int lig)
 {
-	std::list<DataPointInfo *>::iterator iter; 
-	std::list<time_t>::iterator it; 
+	std::vector <DataPointInfo *>::iterator iter; 
+	std::vector <time_t>::iterator it; 
 	QColor    bgColor = Qt::white;
 	QString   txt;
 	int col = 0;
-	addCell_title (tr("Wind")+" ("+AltitudeStr::toStringShort(alt)+")", 
-					true, layout, lig,col);
+	addCell_title_dataline (tr("Wind")+" ("+AltitudeStr::toStringShort(alt)+")", 
+					layout, lig,col);
 	col ++;
-	for (iter=lspinfos.begin(); iter!=lspinfos.end(); iter++, col++)
-	{
+	for (iter=lspinfos.begin(); iter!=lspinfos.end(); iter++, col++) {
 		DataPointInfo * pf = *iter;
 		float v, dir;
 		txt = "";
+		bgColor = Qt::white;
 		if (pf->getWindValues (alt, &v, &dir)) {
-			QString tmp;
-			txt = "";
-			tmp.sprintf("%.0f", dir);
-			txt += tmp + tr(" °") + "\n";
-			txt += Util::formatSpeed(v);
-			if ( Util::getSetting("MTABLE_showWindBeauforts", true).toBool() ) {
-				tmp.sprintf("%2d", Util::kmhToBeaufort(v*3.6));
-				txt += "\n";
-				txt += tmp + tr(" Bf");
+			if (dir != GRIB_NOTDEF) {
+				QString tmp;
+				tmp.sprintf("%.0f", dir);
+				txt += tmp + tr(" °") + "\n";
+				txt += Util::formatSpeed_Wind(v);
+				if ( Util::getSetting("MTABLE_showWindBeauforts", true).toBool() ) {
+					tmp.sprintf("%2d", Util::msToBeaufort(v));
+					txt += "\n";
+					txt += tmp + tr(" Bf");
+				}
+				bgColor = QColor(plotter->getWindColor(v, true));
 			}
-			bgColor = QColor(plotter->getWindColor(v*3.6, true));
 		}
 		float vx, vy;
-		pf->getWindVxVy (alt, &vx, &vy);
+		pf->getWindVxVy (alt, &vx, &vy);   
 		addCell_content (txt, layout,lig,col, 1,1, bgColor, MTABLE_WIND_CELL,vx,vy);
 	}
 }
 //-----------------------------------------------------------------
+void MeteoTableWidget::addLine_Current (const Altitude &alt, int lig)
+{
+	std::vector <DataPointInfo *>::iterator iter; 
+	std::vector <time_t>::iterator it; 
+	QColor    bgColor = Qt::white;
+	QString   txt;
+	int col = 0;
+	addCell_title_dataline (tr("Current")+" ("+AltitudeStr::toStringShort(alt)+")", 
+					layout, lig,col);
+	col ++;
+	for (iter=lspinfos.begin(); iter!=lspinfos.end(); iter++, col++) {
+		DataPointInfo * pf = *iter;
+		float v, dir;
+		txt = "";
+		bgColor = Qt::white;
+		if (pf->getCurrentValues (&v, &dir)) {
+			if (dir != GRIB_NOTDEF) {
+				QString tmp;
+				tmp.sprintf("%.0f", dir);
+				txt += tmp + tr(" °") + "\n";
+				txt += Util::formatSpeed_Current(v);
+				bgColor = QColor(plotter->getCurrentColor(v, true));
+			}
+		}
+		float cx, cy;
+		pf->getCurrentCxCy (alt, &cx, &cy);   
+		addCell_content (txt, layout,lig,col, 1,1, bgColor, MTABLE_CURRENT_CELL,cx,cy);
+	}
+}
+//-----------------------------------------------------------------
+void MeteoTableWidget::addLine_GUSTsfc(int lig)
+{
+	std::vector <DataPointInfo *>::iterator iter;
+	QColor    bgColor = Qt::white;
+	QString   txt;
+	int col = 0;
+	addCell_title_dataline (tr("Wind gust"), true, lig,col);
+	col ++;
+	for (iter=lspinfos.begin(); iter!=lspinfos.end(); iter++, col++) {
+		DataPointInfo * pinfo = *iter;
+		txt = "";
+		if (pinfo->hasGUSTsfc()) {
+			double v = pinfo->GUSTsfc;
+			txt = Util::formatSpeed_Wind(v);
+			bgColor = QColor(plotter->getWindColor(v, true));
+		}
+		else
+			bgColor = Qt::white;
+		addCell_content(txt, layout,lig,col, 1,1, bgColor);
+	}
+}
+//-----------------------------------------------------------------
 void MeteoTableWidget::addLine_HumidRel (const Altitude &alt, int lig)
 {
-	std::list<time_t>::iterator it; 
-	std::list<DataPointInfo *>::iterator iter; 
+	std::vector <time_t>::iterator it; 
+	std::vector <DataPointInfo *>::iterator iter; 
 	QColor    bgColor = Qt::white;
 	QString   txt;
 	int col = 0;
 	//addCell_title(tr("Relative humidity"), true, layout, lig,col);
-	addCell_title (tr("Relative humidity")+" ("+AltitudeStr::toStringShort(alt)+")", 
-					true, layout, lig,col);
+	addCell_title_dataline  (tr("Relative humidity")+" ("+AltitudeStr::toStringShort(alt)+")", 
+					true, lig,col);
 	col ++;
 	for (it=lsdates.begin(); it!=lsdates.end(); it++, col++)
 	{
 		time_t date = *it;
 		txt = "";
 		double v = 0;
-		
 		v = reader->getDateInterpolatedValue (DataCode(GRB_HUMID_REL,alt), lon,lat, date);
-		
-		txt = Util::formatPercentValue(v);
-		bgColor = QColor(plotter->getHumidColor(v, true));
-		
+		if (v != GRIB_NOTDEF) {
+			txt = Util::formatPercentValue(v);
+			bgColor = QColor(plotter->getHumidColor(v, true));
+		}
+		else
+			bgColor = Qt::white;
 		addCell_content(txt, layout,lig,col, 1,1, bgColor);
 	}
 }
 //-----------------------------------------------------------------
 void MeteoTableWidget::addLine_Temperature(const Altitude &alt, uchar type, int lig)
 {
-	std::list<time_t>::iterator it; 
+	std::vector <time_t>::iterator it; 
 	QColor    bgColor = Qt::white;
 	QString   txt;
 	int col = 0;
 	double v;
 	QString title;
 	switch (type) {
-		case GRB_THETA_E:
+		case GRB_PRV_THETA_E:
 			title = tr("Theta-e"); break;
 		case GRB_TEMP:
 			title = tr("Temperature"); break;
@@ -380,14 +661,12 @@ void MeteoTableWidget::addLine_Temperature(const Altitude &alt, uchar type, int
 			title = tr("Temp. pot"); break;
 	}
 	title += " ("+AltitudeStr::toStringShort(alt)+")";
-	addCell_title(title, true, layout, lig,col);
+	addCell_title_dataline (title, true, lig,col);
 	col ++;
-	
-	for (it=lsdates.begin(); it!=lsdates.end(); it++, col++)
-	{
+	for (it=lsdates.begin(); it!=lsdates.end(); it++, col++) {
 		time_t date = *it;
-		if (type == GRB_THETA_E) {
-			int P = alt.levelValue;	// 850 700 500 300 200
+		if (type == GRB_PRV_THETA_E) {
+			int P = alt.levelValue;	// 925 850 700 600 500 400 300 200
 			double SH = reader->getDateInterpolatedValue (DataCode(GRB_HUMID_SPEC,alt), lon,lat,date);
 			double T = reader->getDateInterpolatedValue (DataCode(GRB_TEMP,alt), lon,lat,date);
 			v = DataRecordAbstract::computeThetaE (T,P,SH);
@@ -399,22 +678,24 @@ void MeteoTableWidget::addLine_Temperature(const Altitude &alt, uchar type, int
 			txt = Util::formatTemperature(v);
 			bgColor = QColor(plotter->getTemperatureColor(v, true));
 		}
+		else
+			bgColor = Qt::white;
 		addCell_content(txt, layout,lig,col, 1,1, bgColor);
 	}
 }
 //-----------------------------------------------------------------
 void MeteoTableWidget::addLine_DeltaTemperature(const Altitude &alt, uchar type, int lig)
 {
-	std::list<DataPointInfo *>::iterator iter; 
+	std::vector <DataPointInfo *>::iterator iter; 
 	QColor    bgColor = Qt::white;
 	QString   txt;
 	int col = 0;
 	double v;
 	switch (type) {
-		case GRB_DIFF_TEMPDEW:
+		case GRB_PRV_DIFF_TEMPDEW:
 		default:
-			addCell_title(tr("Gap temp-dew point")+" ("+AltitudeStr::toStringShort(alt)+")", 
-							true, layout, lig,col);
+			addCell_title_dataline (tr("Gap temp-dew point")+" ("+AltitudeStr::toStringShort(alt)+")", 
+							true, lig,col);
 			break;
 	}
 	col ++;
@@ -422,7 +703,7 @@ void MeteoTableWidget::addLine_DeltaTemperature(const Altitude &alt, uchar type,
 	{
 		DataPointInfo * pinfo = *iter;
 		switch (type) {
-			case GRB_DIFF_TEMPDEW:
+			case GRB_PRV_DIFF_TEMPDEW:
 			default:
 				v = fabs(pinfo->temp - pinfo->dewPoint);
 				break;
@@ -432,18 +713,20 @@ void MeteoTableWidget::addLine_DeltaTemperature(const Altitude &alt, uchar type,
 			txt = Util::formatTemperature(v + 273.15);
 			bgColor = QColor(plotter->getDeltaTemperaturesColor(v, true));
 		}
+		else
+			bgColor = Qt::white;
 		addCell_content(txt, layout,lig,col, 1,1, bgColor);
 	}
 }
 //-----------------------------------------------------------------
 void MeteoTableWidget::addLine_DewPoint(const Altitude &alt, int lig)
 {
-	std::list<DataPointInfo *>::iterator iter; 
+	std::vector <DataPointInfo *>::iterator iter; 
 	QColor    bgColor = Qt::white;
 	QString   txt;
 	int col = 0;
-	addCell_title(tr("Dew point")+" ("+AltitudeStr::toStringShort(alt)+")", 
-				  true, layout, lig,col);
+	addCell_title_dataline (tr("Dew point")+" ("+AltitudeStr::toStringShort(alt)+")", 
+				  true, lig,col);
 	col ++;
 	for (iter=lspinfos.begin(); iter!=lspinfos.end(); iter++, col++)
 	{
@@ -454,17 +737,19 @@ void MeteoTableWidget::addLine_DewPoint(const Altitude &alt, int lig)
 			txt = Util::formatTemperature(v);
 			bgColor = QColor(plotter->getTemperatureColor(v, true));
 		}
+		else
+			bgColor = Qt::white;
 		addCell_content(txt, layout,lig,col, 1,1, bgColor);
 	}
 }
 //-----------------------------------------------------------------
-void MeteoTableWidget::addLine_CAPEsfc(int lig)
+void MeteoTableWidget::addLine_CAPEsfc (int lig)
 {
-	std::list<DataPointInfo *>::iterator iter;
+	std::vector <DataPointInfo *>::iterator iter;
 	QColor    bgColor = Qt::white;
 	QString   txt;
 	int col = 0;
-	addCell_title(tr("CAPE (surface)"), true, layout, lig,col);
+	addCell_title_dataline (tr("CAPE (surface)"), true, lig,col);
 	col ++;
 	for (iter=lspinfos.begin(); iter!=lspinfos.end(); iter++, col++)
 	{
@@ -472,21 +757,47 @@ void MeteoTableWidget::addLine_CAPEsfc(int lig)
 		txt = "";
 		if (pinfo->hasCAPEsfc()) {
 			double v = pinfo->CAPEsfc;
-			txt.sprintf("%d ", (int) v);
+			txt.sprintf("%d ", qRound(v));
 			txt += tr("J/kg");
 			bgColor = QColor(plotter->getCAPEColor(v, true));
 		}
-		addCell_content(txt, layout,lig,col, 1,1, bgColor);
+		else
+			bgColor = Qt::white;
+		addCell_content (txt, layout,lig,col, 1,1, bgColor);
+	}
+}
+//-----------------------------------------------------------------
+void MeteoTableWidget::addLine_CINsfc (int lig)
+{
+	std::vector <DataPointInfo *>::iterator iter;
+	QColor    bgColor = Qt::white;
+	QString   txt;
+	int col = 0;
+	addCell_title_dataline (tr("CIN (surface)"), true, lig,col);
+	col ++;
+	for (iter=lspinfos.begin(); iter!=lspinfos.end(); iter++, col++)
+	{
+		DataPointInfo * pinfo = *iter;
+		txt = "";
+		if (pinfo->hasCINsfc()) {
+			double v = pinfo->CINsfc;
+			txt.sprintf("%d ", qRound(v));
+			txt += tr("J/kg");
+			bgColor = QColor(plotter->getCINColor(v, true));
+		}
+		else
+			bgColor = Qt::white;
+		addCell_content (txt, layout,lig,col, 1,1, bgColor);
 	}
 }
 //-----------------------------------------------------------------
 void MeteoTableWidget::addLine_Rain(int lig)
 {
-	std::list<DataPointInfo *>::iterator iter;
+	std::vector <DataPointInfo *>::iterator iter;
 	QColor    bgColor = Qt::white;
 	QString   txt;
 	int col = 0;
-	addCell_title(tr("Precipitation"), true, layout, lig,col);
+	addCell_title_dataline (tr("Precipitation"), true, lig,col);
 	col ++;
 	for (iter=lspinfos.begin(); iter!=lspinfos.end(); iter++, col++)
 	{
@@ -498,45 +809,49 @@ void MeteoTableWidget::addLine_Rain(int lig)
 			txt += tr("mm/h");
 			bgColor = QColor(plotter->getRainColor(v, true));
 		}
-		addCell_content(txt, layout,lig,col, 1,1, bgColor);
+		else
+			bgColor = Qt::white;
+		addCell_content (txt, layout,lig,col, 1,1, bgColor);
 	}
 }
 //-----------------------------------------------------------------
 void MeteoTableWidget::addLine_CloudCover (int lig)
 {
-	std::list<DataPointInfo *>::iterator iter; 
+	std::vector <DataPointInfo *>::iterator iter; 
 	QColor    bgColor = Qt::white;
 	QString   txt;
 	int col = 0;
-	addCell_title(tr("Cloud cover"), true, layout, lig,col);
+	addCell_title_dataline (tr("Cloud cover"), true, lig,col);
 	col ++;
 	for (iter=lspinfos.begin(); iter!=lspinfos.end(); iter++, col++)
 	{
 		DataPointInfo * pinfo = *iter;
 		txt = "";
 		double v = 0;
-		if (pinfo->hasCloud()) {
-			v = pinfo->cloud;
+		if (pinfo->hasCloudTotal()) {
+			v = pinfo->cloudTotal;
 			txt = Util::formatPercentValue(v);
 			bgColor = QColor(plotter->getCloudColor(v, true));
 		}
+		else
+			bgColor = Qt::white;
 		addCell_content(txt, layout,lig,col, 1,1, bgColor, MTABLE_CLOUD_CELL, v);
 	}
 }
 //-----------------------------------------------------------------
-void MeteoTableWidget::addLine_Categorical(uchar type, int lig)
+void MeteoTableWidget::addLine_Categorical (uchar type, int lig)
 {
-	std::list<DataPointInfo *>::iterator iter; 
+	std::vector <DataPointInfo *>::iterator iter; 
 	QColor    bgColor = Qt::white;
 	QString   txt;
 	int col = 0;
 	double v = -1000;
 	switch (type) {
 		case GRB_FRZRAIN_CATEG:
-			addCell_title(tr("Frozen rain possible"), true, layout, lig,col);
+			addCell_title_dataline (tr("Frozen rain possible"), true, lig,col);
 			break;
 		case GRB_SNOW_CATEG:
-			addCell_title(tr("Snowfall possible"), true, layout, lig,col);
+			addCell_title_dataline (tr("Snowfall possible"), true, lig,col);
 			break;
 	}
 	col ++;
@@ -551,20 +866,24 @@ void MeteoTableWidget::addLine_Categorical(uchar type, int lig)
 				v = pinfo->snowCateg;
 				break;
 		}
-		txt = Util::formatCategoricalData (v);
-		bgColor = QColor(plotter->getSnowDepthColor(v, true));
+		if (v != GRIB_NOTDEF) {
+			txt = Util::formatCategoricalData (v);
+			bgColor = QColor(plotter->getSnowDepthColor(v, true));
+		}
+		else
+			bgColor = Qt::white;
 		addCell_content(txt, layout,lig,col, 1,1, bgColor);
 	}
 }
 //-----------------------------------------------------------------
-void MeteoTableWidget::addLine_SnowDepth(int lig)
+void MeteoTableWidget::addLine_SnowDepth (int lig)
 {
-	std::list<DataPointInfo *>::iterator iter; 
+	std::vector <DataPointInfo *>::iterator iter; 
 	QColor    bgColor = Qt::white;
 	QString   txt;
 	int col = 0;
 	double v = -1000;
-	addCell_title(tr("Snow"), true, layout, lig,col);
+	addCell_title_dataline (tr("Snow"), true, lig,col);
 	col ++;
 	for (iter=lspinfos.begin(); iter!=lspinfos.end(); iter++, col++)
 	{
@@ -576,12 +895,37 @@ void MeteoTableWidget::addLine_SnowDepth(int lig)
 			txt = Util::formatSnowDepth(v);
 			bgColor = QColor(plotter->getSnowDepthColor(v, true));
 		}
+		else
+			bgColor = Qt::white;
 		addCell_content(txt, layout,lig,col, 1,1, bgColor);
 	}
 }
-
+//-----------------------------------------------------------------
+void MeteoTableWidget::addLine_SkewT (int lig)
+{
+	std::vector <DataPointInfo *>::iterator iter;
+	int col = 0;
+	addCell_title_dataline (tr("SkewT-LogP"), true, lig,col);
+	col ++;
+	for (iter=lspinfos.begin(); iter!=lspinfos.end(); iter++, col++)
+	{
+		DataPointInfo *pinfo = *iter;
+		addCell_SkewT (layout,lig,col, pinfo->date);
+	}
+}
 
 //===================================================================
+void MeteoTableWidget::addCell_SkewT (
+				QGridLayout *layout,int lig,int col,
+				time_t date
+				)
+{
+	TableCell *cell = new TableCell_SkewT (this, lon, lat, locationName, date, reader);	
+	assert (cell);
+	cell->setBorders (lig, col);
+	layout->addWidget (cell, lig,col, 1,1, Qt::AlignCenter);
+}
+//----------------------------------------------------------------
 void MeteoTableWidget::addCell_content (
 				QString txt,
 				QGridLayout *layout,int lig,int col,
@@ -595,29 +939,26 @@ void MeteoTableWidget::addCell_content (
 {
 	TableCell *cell;
  	if (cellType == MTABLE_WIND_CELL) {
-	 	cell = new TableCell_Wind(vx, vy, (lat<0), plotter,
+	 	cell = new TableCell_Wind (vx, vy, (lat<0), plotter,
+	 				this, txt, false, bgcolor);
+ 	}
+ 	else if (cellType == MTABLE_CURRENT_CELL) {
+	 	cell = new TableCell_Current (vx, vy, (lat<0), plotter,
 	 				this, txt, false, bgcolor);
  	}
  	else if (cellType == MTABLE_CLOUD_CELL) {
-	 	cell = new TableCell_Clouds ( vx, plotter,
+	 	cell = new TableCell_Clouds (vx, plotter,
 	 				this, txt, false, bgcolor);
  	}
  	else {
-	 	cell = new TableCell(this, txt, false, bgcolor);
+	 	cell = new TableCell (this, txt, false, bgcolor);
 	}
-	if (lig==0 && col==0)
-		cell->setBorders(TableCell::all);
-	else if (lig==0)
-		cell->setBorders(TableCell::south+TableCell::east+TableCell::north);
-	else if (col==0)
-		cell->setBorders(TableCell::west+TableCell::south+TableCell::east);
-	else
-		cell->setBorders(TableCell::south+TableCell::east);
-	layout->addWidget(cell, lig,col, rowspan,colspan );
+	assert (cell);
+	cell->setBorders (lig, col);
+	layout->addWidget (cell, lig,col, rowspan,colspan );
 }
-
 //----------------------------------------------------------------
-void MeteoTableWidget::addCell_title( QString txt, bool bold,
+void MeteoTableWidget::addCell_title (QString txt, bool bold,
 				QGridLayout *layout,int lig,int col, int rowspan,int colspan,
 				bool isNowDate)
 {
@@ -625,32 +966,26 @@ void MeteoTableWidget::addCell_title( QString txt, bool bold,
 	if (isNowDate) {
 		bgcolor = QColor(250,250,100);
 	}
-	
 	TableCell *cell = new TableCell(this, txt, bold, bgcolor);	
-	if (lig==0 && col==0)
-		cell->setBorders(TableCell::all);
-	else if (lig==0)
-		cell->setBorders(TableCell::south+TableCell::east+TableCell::north);
-	else if (col==0)
-		cell->setBorders(TableCell::west+TableCell::south+TableCell::east);
-	else
-		cell->setBorders(TableCell::south+TableCell::east);
-	layout->addWidget(cell, lig,col, rowspan,colspan );
+	assert (cell);
+	cell->setBorders (lig, col);
+	layout->addWidget (cell, lig,col, rowspan,colspan );
 }
-
 //===================================================================
 // TableCell : case seule
 //===================================================================
 TableCell::TableCell(QWidget *parent, QString txt, bool bold,
-						QColor bgcolor)
+						QColor bgcolor,
+						Qt::Alignment alignment
+					)
 	: QWidget(parent)
 {
 	this->bgcolor = bgcolor;
 	this->bordercolor = QColor(100,100,100);
 	this->borders = TableCell::none;
 	
-	label = new QLabel(txt, this);
-	label->setAlignment(Qt::AlignHCenter);
+	label = new QLabel (txt, this);
+//	label->setAlignment (alignment);
 	if (bold) {
 		QFont font;
 		font.setBold(true);
@@ -658,28 +993,34 @@ TableCell::TableCell(QWidget *parent, QString txt, bool bold,
 	}
 	setContrastedTextColor(this->bgcolor);
 	
-	QGridLayout *layout = new QGridLayout();
+	QGridLayout *layout = new QGridLayout (this);
 	int mh=2, mv=4;
 	layout->setContentsMargins (mh,mv,mh,mv);
-	layout->addWidget(label, 0,0, Qt::AlignHCenter|Qt::AlignBottom);
-	this->setLayout(layout);
+//	layout->addWidget(label, 0,0, Qt::AlignHCenter|Qt::AlignBottom);
+	layout->addWidget(label, 0,0, alignment);
 }
 //----------------------------------------------------------
 void    TableCell::setContrastedTextColor(QColor bgcolor)
 {
-	// Ecriture contrastée
-	double gris = 0.30*bgcolor.redF() + 0.59*bgcolor.greenF() + 0.11*bgcolor.blueF();
-	QColor fgcolor;
-	if (gris < 0.45)
-		fgcolor = QColor(230,230,230);
-	else
-		fgcolor = Qt::black;
+	QColor fgcolor = DataColors::getContrastedColor (bgcolor);
 	QPalette p;
 	p.setBrush(QPalette::Active, QPalette::WindowText, fgcolor);
 	p.setBrush(QPalette::Inactive, QPalette::WindowText, fgcolor);
 	label->setPalette(p);
 }
 //---------------------------------------------------------
+void TableCell::setBorders (int lig, int col)
+{
+	if (lig==0 && col==0)
+		this->setBorders (TableCell::all);
+	else if (lig==0)
+		this->setBorders (TableCell::south+TableCell::east+TableCell::north);
+	else if (col==0)
+		this->setBorders (TableCell::west+TableCell::south+TableCell::east);
+	else
+		this->setBorders (TableCell::south+TableCell::east);
+}
+//---------------------------------------------------------
 void TableCell::paintEvent(QPaintEvent * /*event*/)
 {
     QPainter pnt(this);
@@ -720,7 +1061,7 @@ TableCell_Wind::TableCell_Wind (double vx, double vy, bool south,
 	windArrowsColor = QColor(40,40,40);
 	showWindArrows = Util::getSetting("MTABLE_showWindArrows", true).toBool();
 
-	if (showWindArrows)
+	if (showWindArrows && vx!=GRIB_NOTDEF && vy!=GRIB_NOTDEF)
 		setMinimumHeight(label->minimumSizeHint().height()+50);
 }	
 //---------------------------------------------------------
@@ -738,6 +1079,40 @@ void TableCell_Wind::paintEvent(QPaintEvent * e)
 }
 
 
+//===================================================================
+// TableCell_Current : case seule spécialisée pour le courant (flêche)
+//===================================================================
+TableCell_Current::TableCell_Current (double cx, double cy, bool south,
+        			GriddedPlotter *plotter,
+        			QWidget *parent, QString txt, bool bold,
+        			QColor bgcolor )
+	: TableCell(parent, txt, bold, bgcolor)
+{
+	this->cx = cx;
+	this->cy = cy;
+	this->south = south;
+	this->plotter = plotter;
+
+	currentArrowsColor = QColor(40,40,40);
+	showCurrentArrows = Util::getSetting("MTABLE_showCurrentArrows", true).toBool();
+
+	if (showCurrentArrows && cx!=GRIB_NOTDEF && cy!=GRIB_NOTDEF)
+		setMinimumHeight(label->minimumSizeHint().height()+50);
+}	
+//---------------------------------------------------------
+void TableCell_Current::paintEvent(QPaintEvent * e)
+{
+	TableCell::paintEvent(e);
+    QPainter pnt(this);
+	pnt.setRenderHint(QPainter::Antialiasing, true);
+
+	if (showCurrentArrows && cx != GRIB_NOTDEF && cx != GRIB_NOTDEF)
+	{
+    	plotter->drawCurrentArrow (
+    			pnt, width()/2, 25, cx, cy, south, currentArrowsColor);
+	}
+}
+
 
 //===================================================================
 // TableCell_Clouds : case seule spécialisée pour la nébulosité
@@ -778,4 +1153,67 @@ void TableCell_Clouds::paintEvent(QPaintEvent * e)
 
 
 
+//===================================================================
+// TableCell_SkewT
+//===================================================================
+TableCell_SkewT::TableCell_SkewT (QWidget *parent,
+					double lon, double lat,  QString locationName,
+					time_t date,
+					GriddedReader *reader)
+	: TableCell (parent)
+{
+	this->lon = lon;
+	this->lat = lat;
+	this->locationName = locationName;
+	this->date = date;
+	this->reader = reader;
+	MiniSkewT miniskewt;
+	miniskewt.initFromGriddedReader (reader, lon, lat, date);
+	pixmap = miniskewt.createPixmap ();
+	setFixedSize (miniskewt.size());
+}
+//---------------------------------------------------------
+void TableCell_SkewT::paintEvent (QPaintEvent * e)
+{
+	TableCell::paintEvent (e);
+    QPainter pnt (this);
+	pnt.setRenderHint(QPainter::Antialiasing, true);
+	pnt.drawPixmap (0,0, pixmap);
+}
+//---------------------------------------------------------
+void TableCell_SkewT::enterEvent (QEvent *)
+{
+	setCursor (Qt::PointingHandCursor);
+}
+//---------------------------------------------------------
+void TableCell_SkewT::leaveEvent (QEvent *)
+{
+	unsetCursor ();
+}
+//---------------------------------------------------------
+void TableCell_SkewT::mouseReleaseEvent (QMouseEvent *e)
+{
+	if (this->rect().contains (e->pos())) {
+		SkewT *skewt = new SkewT ();
+		skewt->initFromGriddedReader (reader, lon, lat, date);
+		
+		SkewTWindow *sdial = new SkewTWindow (skewt);
+		
+		QString position = Util::formatPosition(lon, lat);
+		if (locationName == "") {
+			skewt->setLocation (position);
+			sdial->setWindowTitle ("SkewT - "+position);
+		}
+		else {
+			skewt->setLocation (position+" ("+locationName+")");
+			sdial->setWindowTitle ("SkewT - "+locationName);
+		}
+		sdial->show ();
+	}
+}
+
+	
+	
+	
+	
 
