@@ -1,6 +1,6 @@
 /**********************************************************************
 zyGrib: meteorological GRIB file viewer
-Copyright (C) 2008-2010 - Jacques Zaninetti - http://www.zygrib.org
+Copyright (C) 2008-2012 - Jacques Zaninetti - http://www.zygrib.org
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -23,6 +23,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <QMouseEvent>
 #include <QPaintEvent>
 #include <QPainter>
+#include <QProgressDialog>
+#include <QMessageBox>
 
 #include "Terrain.h"
 #include "Orthodromie.h"
@@ -32,15 +34,20 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 //---------------------------------------------------------
 // Constructeur
 //---------------------------------------------------------
-Terrain::Terrain (QWidget *parent, Projection *proj_, GshhsReader *gshhsReader)
+Terrain::Terrain (QWidget *parent, Projection *proj, GshhsReader *gshhsReader)
     : QWidget(parent)
 {
 	setSizePolicy (QSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding));
+    this->proj = proj;
+	
     quality = 0;
     selX0 = selY0 = 0;
     selX1 = selY1 = 0;
-    isResizing = true;
-    
+	lastMouseX = -1;
+	lastMouseY = -1;
+    setAutoFillBackground (false);
+	drawCartouche = true;
+	
     timerResize = new QTimer(this);
     assert(timerResize);
     timerResize->setSingleShot(true);
@@ -60,30 +67,49 @@ Terrain::Terrain (QWidget *parent, Projection *proj_, GshhsReader *gshhsReader)
     //----------------------------------------------------------------------------
     showOrthodromie   = Util::getSetting("showOrthodromie", false).toBool();
     duplicateFirstCumulativeRecord = Util::getSetting("duplicateFirstCumulativeRecord", true).toBool();
+    duplicateMissingWaveRecords = Util::getSetting("duplicateMissingWaveRecords", true).toBool();
     interpolateValues = Util::getSetting("interpolateValues", true).toBool();
     windArrowsOnGribGrid = Util::getSetting("windArrowsOnGribGrid", false).toBool();
+    currentArrowsOnGribGrid = Util::getSetting("currentArrowsOnGribGrid", false).toBool();
+	fastInterpolation_MBlue = Util::getSetting ("MBfastInterpolation", true).toBool();
     
     isEarthMapValid = false;
     mustRedraw = true;
-
-    proj = proj_;
+ 	mustShowSpecialZone = false;
+	firstDrawingIsDone = false;
+    isResizing = false;
+	pleaseWait = false;
 	
     //---------------------------------------------------------------
-	griddedPlot = new GribPlot ();
-	assert(griddedPlot);
-
-	iacPlot = new IacPlot();
-    assert(iacPlot);
+	griddedPlot = NULL;
+	iacPlot = NULL;
+	taskProgress = NULL;
 
     //---------------------------------------------------------------
 	updateGraphicsParameters();
     setIsobarsStep(drawer->isobarsStep);
     
     isSelectionZoneEnCours = false;
-    
+	isDraggingMapEnCours = false;
+
     setMouseTracking(true);
     setFocusPolicy(Qt::StrongFocus);
 	
+	createCrossCursor ();
+}
+//-------------------------------------------
+void Terrain::updateGraphicsParameters()
+{            
+    drawer->updateGraphicsParameters();
+	if (griddedPlot)
+		griddedPlot->updateGraphicsParameters();
+	isEarthMapValid = false;
+	mustRedraw = true;
+    update();
+}
+//-------------------------------------------------------
+void Terrain::createCrossCursor ()
+{            
 	//---------------------------------------------------
 	// Define new cross cursor, more visible
 	//---------------------------------------------------
@@ -117,24 +143,11 @@ Terrain::Terrain (QWidget *parent, Projection *proj_, GshhsReader *gshhsReader)
 	pm.end();
 	myCrossCursor = QCursor(*crossBits, *crossMask);
     
-//myCrossCursor=Qt::CrossCursor;
-    setCursor(myCrossCursor);
-    
-}
-//-------------------------------------------
-void Terrain::updateGraphicsParameters()
-{            
-    drawer->updateGraphicsParameters();
     int v = 180;
     selectColor     = QColor(v,v,v);
-	griddedPlot->setCloudsColorMode("cloudsColorMode");
-    
-	isEarthMapValid = false;
-	mustRedraw = true;
-    update();
+    setCursor(myCrossCursor);
 }
 
-
 //-------------------------------------------------------
 // Tracé récursif de la route orthodromique
 void Terrain::draw_OrthodromieSegment(QPainter &pnt,
@@ -203,18 +216,19 @@ void Terrain::draw_Orthodromie(QPainter &pnt)
 void Terrain::indicateWaitingMap()
 {
     pleaseWait = true;   // Affiche un message d'attente
-    update();
-    QCoreApplication::processEvents();
 }
 
 //-------------------------------------------------------
 void Terrain::setProjection(Projection *proj)
-{
-//printf("Terrain::setProjection\n");
+{ 
     indicateWaitingMap();
     this->proj = proj;
     proj->setScreenSize( width(), height());
 	
+    Util::setSetting("projectionCX", proj->getCX());
+    Util::setSetting("projectionCY", proj->getCY());
+    Util::setSetting("projectionScale",  proj->getScale());	
+	
 	QList<POI*> lpois = getListPOIs();
 	for (int i=0; i<lpois.size(); i++) {
 		lpois.at(i)->setProjection(proj);
@@ -225,19 +239,19 @@ void Terrain::setProjection(Projection *proj)
 }
 
 //=========================================================
-void Terrain::setDrawLonLatGrid(bool b) {
-    if (drawer->showLonLatGrid != b) {
-        drawer->showLonLatGrid = b;
-        Util::setSetting("showLonLatGrid", b);
+void Terrain::setDrawRivers(bool b) {
+    if (drawer->showRivers != b) {
+        drawer->showRivers = b;
+        Util::setSetting("showRivers", b);
         mustRedraw = true;
         update();
     }
 }
 //-------------------------------------------------------
-void Terrain::setDrawRivers(bool b) {
-    if (drawer->showRivers != b) {
-        drawer->showRivers = b;
-        Util::setSetting("showRivers", b);
+void Terrain::setDrawLonLatGrid(bool b) {
+    if (drawer->showLonLatGrid != b) {
+        drawer->showLonLatGrid = b;
+        Util::setSetting("showLonLatGrid", b);
         mustRedraw = true;
         update();
     }
@@ -252,15 +266,6 @@ void Terrain::slotTemperatureLabels(bool b) {
     }
 }
 //-------------------------------------------------------
-void Terrain::setDrawCountriesBorders(bool b) {
-    if (drawer->showCountriesBorders != b) {
-        drawer->showCountriesBorders = b;
-        Util::setSetting("showCountriesBorders", b);
-        mustRedraw = true;
-        update();
-    }
-}
-//-------------------------------------------------------
 void Terrain::setDrawOrthodromie(bool b) {
     if (showOrthodromie != b) {
         showOrthodromie = b;
@@ -270,6 +275,15 @@ void Terrain::setDrawOrthodromie(bool b) {
     }
 }
 //-------------------------------------------------------
+void Terrain::setDrawCountriesBorders(bool b) {
+    if (drawer->showCountriesBorders != b) {
+        drawer->showCountriesBorders = b;
+        Util::setSetting("showCountriesBorders", b);
+        mustRedraw = true;
+        update();
+    }
+}
+//-------------------------------------------------------
 void Terrain::setCountriesNames(bool b) {
     if (drawer->showCountriesNames != b) {
         drawer->showCountriesNames = b;
@@ -287,7 +301,15 @@ void Terrain::setCitiesNamesLevel  (int level) {
         update();
     }
 }
-
+//-------------------------------------------------------
+void Terrain::setWaveArrowsType  (int type) {
+    if (drawer->showWaveArrowsType != type) {
+        drawer->showWaveArrowsType = type;
+        Util::setSetting("waveArrowsType", type);
+        mustRedraw = true;
+        update();
+    }
+}
 //-------------------------------------------------------
 void Terrain::setMapQuality (int q) {
     indicateWaitingMap();
@@ -306,14 +328,12 @@ void Terrain::setMapQuality (int q) {
         pleaseWait = false;
     }
 }
-
 //-------------------------------------------------------
-void Terrain::setColorMapData (const DataCode &dtc)
-{
-    if (drawer->getColorMapData() != dtc)
-    {
-		Util::setSetting ("colorMapData", DataCodeStr::serialize(dtc));
-        drawer->setColorMapData (dtc);
+void Terrain::setDuplicateMissingWaveRecords (bool b) {
+    if (duplicateMissingWaveRecords != b) {
+        duplicateMissingWaveRecords = b;
+        Util::setSetting("duplicateMissingWaveRecords", b);
+	    griddedPlot->duplicateMissingWaveRecords (b);
         mustRedraw = true;
         update();
     }
@@ -349,6 +369,32 @@ void Terrain::setWindArrowsOnGribGrid (bool b) {
     }
 }
 //-------------------------------------------------------
+void Terrain::setColorMapData (const DataCode &dtc)
+{
+	//DBGQS (DataCodeStr::toString (dtc));
+    if (drawer)
+    {
+		Util::setSetting ("colorMapData", DataCodeStr::serialize(dtc));
+        drawer->setColorMapData (dtc);
+		if (griddedPlot!=NULL && griddedPlot->isReaderOk()) {
+			griddedPlot->setUseJetStreamColorMap (
+						Util::getSetting("useJetStreamColorMap", false).toBool());
+		}
+        mustRedraw = true;
+        update();
+    }
+}
+//-------------------------------------------------------
+void Terrain::setCurrentArrowsOnGribGrid (bool b) {
+    if (currentArrowsOnGribGrid != b) {
+        currentArrowsOnGribGrid = b;
+        Util::setSetting("currentArrowsOnGribGrid", b);
+	    griddedPlot->setCurrentArrowsOnGrid (b);
+        mustRedraw = true;
+        update();
+    }
+}
+//-------------------------------------------------------
 void Terrain::setColorMapSmooth (bool b) {
     if (drawer->colorMapSmooth != b) {
         drawer->colorMapSmooth = b;
@@ -358,6 +404,15 @@ void Terrain::setColorMapSmooth (bool b) {
     }
 }
 //-------------------------------------------------------
+void Terrain::setDrawCurrentArrows (bool b) {
+    if (drawer->showCurrentArrows != b) {
+        drawer->showCurrentArrows = b;
+        Util::setSetting("showCurrentArrows", b);
+        mustRedraw = true;
+        update();
+    }
+}
+//-------------------------------------------------------
 void Terrain::setDrawWindArrows (bool b) {
     if (drawer->showWindArrows != b) {
         drawer->showWindArrows = b;
@@ -376,6 +431,18 @@ void Terrain::setBarbules (bool b) {
     }
 }
 //-------------------------------------------------------
+void Terrain::setThinArrows (bool b) {
+	bool actual = Util::getSetting("thinWindArrows", false).toBool();
+    if (actual != b) {
+        Util::setSetting("thinWindArrows", b);
+		if (griddedPlot) {
+			griddedPlot->updateGraphicsParameters ();
+		}
+        mustRedraw = true;
+        update();
+    }
+}
+//-------------------------------------------------------
 void Terrain::setGribGrid (bool b) {
     if (drawer->showGribGrid != b) {
         drawer->showGribGrid = b;
@@ -395,7 +462,7 @@ void Terrain::setPressureMinMax (bool b) {
 }
 
 //-------------------------------------------------------
-void Terrain::setDrawIsobars(bool b) {
+void Terrain::setDrawIsobars (bool b) {
     if (drawer->showIsobars != b) {
         drawer->showIsobars = b;
         Util::setSetting("showIsobars", b);
@@ -404,10 +471,9 @@ void Terrain::setDrawIsobars(bool b) {
     }
 }
 //-------------------------------------------------------
-void Terrain::setIsobarsStep(double step)
+void Terrain::setIsobarsStep (double step)
 {
-    if (griddedPlot != NULL) {
-        griddedPlot->setIsobarsStep(step);
+    if (drawer->isobarsStep != step) {
         Util::setSetting("isobarsStep", step);
         drawer->isobarsStep = step;
         mustRedraw = true;
@@ -415,7 +481,7 @@ void Terrain::setIsobarsStep(double step)
     }
 }
 //-------------------------------------------------------
-void Terrain::setDrawIsobarsLabels(bool b) {
+void Terrain::setDrawIsobarsLabels (bool b) {
     if (drawer->showIsobarsLabels != b) {
         drawer->showIsobarsLabels = b;
         Util::setSetting("showIsobarsLabels", b);
@@ -423,9 +489,8 @@ void Terrain::setDrawIsobarsLabels(bool b) {
         update();
     }
 }
-
-//-------------------------------------------------------
-void Terrain::setDrawIsotherms0(bool b) {
+//=============================================================
+void Terrain::setDrawIsotherms0 (bool b) {
     if (drawer->showIsotherms0 != b) {
         drawer->showIsotherms0 = b;
         Util::setSetting("showIsotherms0", b);
@@ -434,10 +499,9 @@ void Terrain::setDrawIsotherms0(bool b) {
     }
 }
 //-------------------------------------------------------
-void Terrain::setIsotherms0Step(double step)
+void Terrain::setIsotherms0Step (double step)
 {
-    if (griddedPlot != NULL) {
-        griddedPlot->setIsotherms0Step(step);
+    if (drawer->isotherms0Step != step) {
         Util::setSetting("isotherms0Step", step);
         drawer->isotherms0Step = step;
         mustRedraw = true;
@@ -445,7 +509,7 @@ void Terrain::setIsotherms0Step(double step)
     }
 }
 //-------------------------------------------------------
-void Terrain::setDrawIsotherms0Labels(bool b) {
+void Terrain::setDrawIsotherms0Labels (bool b) {
     if (drawer->showIsotherms0Labels != b) {
         drawer->showIsotherms0Labels = b;
         Util::setSetting("showIsotherms0Labels", b);
@@ -453,6 +517,83 @@ void Terrain::setDrawIsotherms0Labels(bool b) {
         update();
     }
 }
+//=================================================================
+void Terrain::setIsotherms_Altitude (Altitude alt)
+{
+    if (drawer->isothermsAltitude != alt) {
+        drawer->isothermsAltitude = alt;
+		//DBGQS (AltitudeStr::toString (alt));
+        Util::setSetting ("isothermsAltitude", AltitudeStr::serialize(alt));
+        mustRedraw = true;
+        update();
+    }
+}
+//-------------------------------------------------------
+void Terrain::setDrawIsotherms (bool b) {
+    if (drawer->showIsotherms != b) {
+        drawer->showIsotherms = b;
+        Util::setSetting("showIsotherms", b);
+        mustRedraw = true;
+        update();
+    }
+}
+//-------------------------------------------------------
+void Terrain::setIsotherms_Step (double step)
+{
+    if (drawer->isotherms_Step != step) {
+		Util::setSetting("isotherms_Step", step);
+		drawer->isotherms_Step = step;
+		mustRedraw = true;
+		update();
+    }
+}
+//-------------------------------------------------------
+void Terrain::setDrawIsotherms_Labels (bool b) {
+    if (drawer->showIsotherms_Labels != b) {
+        drawer->showIsotherms_Labels = b;
+        Util::setSetting("showIsotherms_Labels", b);
+        mustRedraw = true;
+        update();
+    }
+}
+//=================================================================
+void Terrain::setLinesThetaE_Altitude (Altitude alt)
+{
+    if (drawer->linesThetaEAltitude != alt) {
+        drawer->linesThetaEAltitude = alt;
+		DBGQS (AltitudeStr::toString (alt));
+        Util::setSetting ("linesThetaEAltitude", AltitudeStr::serialize(alt));
+        mustRedraw = true;
+        update();
+    }
+}
+//-------------------------------------------------------
+void Terrain::setDrawLinesThetaE (bool b) {
+    if (drawer->showLinesThetaE != b) {
+        drawer->showLinesThetaE = b;
+        Util::setSetting("showLinesThetaE", b);
+        mustRedraw = true;
+        update();
+    }
+}
+//-------------------------------------------------------
+void Terrain::setLinesThetaE_Step (double step) {
+    if (drawer->linesThetaE_Step != step) {
+		Util::setSetting("linesThetaE_Step", step);
+		drawer->linesThetaE_Step = step;
+		mustRedraw = true;
+		update();
+    }
+}
+//-------------------------------------------------------
+void Terrain::setDrawLinesThetaE_Labels (bool b) {
+    if (drawer->showLinesThetaE_Labels != b) {
+        drawer->showLinesThetaE_Labels = b;
+        Util::setSetting("showLinesThetaE_Labels", b);
+        mustRedraw = true;
+        update();
+    }
+}
 
 //=================================================================
 bool  Terrain::getSelectedRectangle(double *x0, double *y0, double *x1, double *y1)
@@ -509,61 +650,110 @@ FileDataType Terrain::loadMeteoDataFile (QString fileName, bool zoom)
     indicateWaitingMap();
 	currentFileType = DATATYPE_NONE;
 	bool ok = false;
-	if (! ok) {		// try to load a GRIB file
+	
+	taskProgress = new LongTaskProgress (this);
+	assert (taskProgress);
+	taskProgress->continueDownload = true;
+	
+	if (griddedPlot != NULL) {
+		delete griddedPlot;
+		griddedPlot = NULL;
+	}
+	if (iacPlot != NULL) {
+		delete iacPlot;
+		iacPlot = NULL;
+	}
+	taskProgress->setMessage (LTASK_OPEN_FILE);
+	taskProgress->setValue (0);
+	
+	GriddedPlotter  *griddedPlot_Temp = NULL;
+	if (!ok && taskProgress->continueDownload) {	// try to load a GRIB file
 		//DBG("try to load a GRIB file");
-		if (griddedPlot != NULL)
-			delete griddedPlot;
-		griddedPlot = new GribPlot ();
-		assert(griddedPlot);
-		griddedPlot->loadFile( fileName );    // GRIB file ?
-		if (griddedPlot->isReaderOk()) {
+		taskProgress->setWindowTitle (tr("Open file")+" GRIB");
+		taskProgress->setVisible (true);
+		taskProgress->setValue (0);
+		griddedPlot_Temp = new GribPlot ();
+		assert(griddedPlot_Temp);
+		griddedPlot_Temp->loadFile (fileName, taskProgress);    // GRIB file ?
+		if (griddedPlot_Temp->isReaderOk()) {
 			currentFileType = DATATYPE_GRIB;
 			ok = true;
 		}
+		else {
+			delete griddedPlot_Temp;
+			griddedPlot_Temp = NULL;
+		}
 	}
-	if (! ok) {		// try to load a MBLUE file
+	if (!ok && taskProgress->continueDownload) {	// try to load a MBLUE file
 		//DBG("try to load a MBLUE file");
-		if (griddedPlot != NULL)
-			delete griddedPlot;
-		griddedPlot = new MbluePlot ();
-		assert(griddedPlot);
-		griddedPlot->loadFile( fileName );    // GRIB file ?
-		if (griddedPlot->isReaderOk()) {
+		taskProgress->setWindowTitle (tr("Open file")+" MBLUE");
+		taskProgress->setVisible (true);
+		taskProgress->setValue (0);
+		griddedPlot_Temp = new MbluePlot ();
+		assert(griddedPlot_Temp);
+// 		QTime now = QTime::currentTime ();
+		griddedPlot_Temp->loadFile (fileName, taskProgress);    // MBLUE file ?
+		
+// 		DBG ("openfile MbluePlot: %.2f s.", now.elapsed()/1000.0);
+		
+		if (griddedPlot_Temp->isReaderOk()) {
+			griddedPlot_Temp->setFastInterpolation (fastInterpolation_MBlue);
 			currentFileType = DATATYPE_MBLUE;
 			ok = true;
 		}
+		else {
+			delete griddedPlot_Temp;
+			griddedPlot_Temp = NULL;
+		}
 	}
-	if (! ok) {		// try to load a IAC file
+	if (!ok && taskProgress->continueDownload) {	// try to load a IAC file
 		//DBG("try to load a IAC file");
-		iacPlot->loadFile( fileName );      // IAC file ?
+		iacPlot = new IacPlot ();
+		assert(iacPlot);
+		iacPlot->loadFile (fileName);      // IAC file ?
 		if (iacPlot->isReaderOk()) {
 			currentFileType = DATATYPE_IAC;
 			ok = true;
 		}
+		else {
+			delete iacPlot;
+			iacPlot = NULL;
+		}
 	}
-
+	
+	taskProgress->setVisible (false);
+	
+	griddedPlot = griddedPlot_Temp;
 	if (ok) {	// initializes data plotter
 		switch (currentFileType) {
 			case DATATYPE_GRIB :
-				griddedPlot->setInterpolateValues (interpolateValues);
-				griddedPlot->setWindArrowsOnGrid  (windArrowsOnGribGrid);
-				griddedPlot->duplicateFirstCumulativeRecord (duplicateFirstCumulativeRecord);
-				griddedPlot->setCurrentDateClosestFromNow ();
+				griddedPlot_Temp->setInterpolateValues (interpolateValues);
+				griddedPlot_Temp->setWindArrowsOnGrid  (windArrowsOnGribGrid);
+				griddedPlot_Temp->setCurrentArrowsOnGrid  (currentArrowsOnGribGrid);
+				griddedPlot_Temp->duplicateFirstCumulativeRecord (duplicateFirstCumulativeRecord);
+				griddedPlot_Temp->duplicateMissingWaveRecords (duplicateMissingWaveRecords);
+				griddedPlot_Temp->setCurrentDateClosestFromNow ();
+				griddedPlot_Temp->setUseJetStreamColorMap (
+								Util::getSetting("useJetStreamColorMap", false).toBool());
 				break;
 			case DATATYPE_MBLUE :
-				griddedPlot->setInterpolateValues (interpolateValues);
-				griddedPlot->setWindArrowsOnGrid  (windArrowsOnGribGrid);
-				griddedPlot->setCurrentDateClosestFromNow ();
+				griddedPlot_Temp->setInterpolateValues (interpolateValues);
+				griddedPlot_Temp->setWindArrowsOnGrid  (windArrowsOnGribGrid);
+				griddedPlot_Temp->setCurrentArrowsOnGrid  (currentArrowsOnGribGrid);
+				griddedPlot_Temp->setCurrentDateClosestFromNow ();
+				griddedPlot_Temp->setUseJetStreamColorMap (
+								Util::getSetting("useJetStreamColorMap", false).toBool());
 				break;
 			default :
 				break;
 		}
 	}
 	else {
-		DBG("ERROR: unknown file type");
+		//DBG("ERROR: unknown file type");
 	}
 	
 	isSelectionZoneEnCours = false;
+	isDraggingMapEnCours = false;
     selX0 = selY0 = 0;
     selX1 = selY1 = 0;
     isEarthMapValid = false;
@@ -572,7 +762,15 @@ FileDataType Terrain::loadMeteoDataFile (QString fileName, bool zoom)
         zoomOnFileZone();    // Zoom sur la zone couverte par le fichier GRIB
     }
     update();
-	return currentFileType;
+
+	bool cancelled = ! taskProgress->continueDownload;
+	delete taskProgress;
+	taskProgress = NULL;
+	
+	if (cancelled)
+		return DATATYPE_CANCELLED;
+	else
+		return currentFileType;
 }
 
 //---------------------------------------------------------
@@ -591,15 +789,23 @@ GriddedPlotter *Terrain::getGriddedPlotter ()
 //---------------------------------------------------------
 void   Terrain::closeMeteoDataFile()
 {
+	if (griddedPlot != NULL) {
+		delete griddedPlot;
+		griddedPlot = NULL;
+	}
+	if (iacPlot != NULL) {
+		delete iacPlot;
+		iacPlot = NULL;
+	}
 	currentFileType = DATATYPE_NONE;
 	mustRedraw = true;
     update();
 }
 
 //---------------------------------------------------------
-void Terrain::zoomOnFileZone()
+void Terrain::zoomOnFileZone ()
 {
-	double x0,y0, x1,y1, mh, mv;
+	double x0,y0, x1,y1;
 	bool ok = false;
     if (   currentFileType == DATATYPE_GRIB
 		|| currentFileType == DATATYPE_MBLUE
@@ -614,14 +820,41 @@ void Terrain::zoomOnFileZone()
 		}
     }
 
-	if (ok)
-	{
-		mh = fabs(x0-x1)*0.05;
-		mv = fabs(y0-y1)*0.05;
-		proj->setVisibleArea(x0-mh,y0-mv, x1+mh,y1+mv);
-		setProjection(proj);
+	if (ok)	{
+		this->zoomOnZone (x0, y0, x1, y1);
 	}
 }
+//---------------------------------------------------------
+void  Terrain::setSpecialZone  (double x0, double y0, double x1, double y1)
+{
+	double tmp;
+	if (x0 > y0) {
+		tmp=x0; x0=x1; x1=tmp;
+	}
+	if (y0 > y0) {
+		tmp=y0; y0=y1; y1=tmp;
+	}
+	specialZoneX0 = x0;
+	specialZoneY0 = y0;
+	specialZoneX1 = x1;
+	specialZoneY1 = y1;
+}
+
+//---------------------------------------------------------
+void  Terrain::showSpecialZone (bool b)
+{
+ 	mustShowSpecialZone = b;
+}
+
+//---------------------------------------------------------
+void Terrain::zoomOnZone (double x0, double y0, double x1, double y1)
+{
+	double mh, mv;
+	mh = fabs(x0-x1)*0.05;
+	mv = fabs(y0-y1)*0.05;
+	proj->setVisibleArea (x0-mh,y0-mv, x1+mh,y1+mv);
+	setProjection (proj);
+}
 
 //---------------------------------------------------------
 void Terrain::slotMustRedraw()
@@ -654,8 +887,9 @@ void Terrain::enterEvent (QEvent * /*e*/) {
     setCursor(myCrossCursor);
 }
 //---------------------------------------------------------
-void Terrain::leaveEvent (QEvent * /*e*/) {
+void Terrain::leaveEvent (QEvent * e) {
 //printf("leave\n");
+	emit mouseLeave (e);
     setCursor(enterCursor);
 }
 
@@ -665,7 +899,7 @@ void  Terrain::keyPressEvent (QKeyEvent *e)
 //printf("Terrain::keyPressEvent\n");
 	keyModifiers = e->modifiers();
     if (keyModifiers == Qt::ControlModifier) {
-        setCursor(Qt::SizeAllCursor);
+        setCursor(Qt::ClosedHandCursor);
     }
     else if (keyModifiers == Qt::ShiftModifier) {
         setCursor(Qt::UpArrowCursor);
@@ -680,7 +914,7 @@ void  Terrain::keyReleaseEvent (QKeyEvent *e)
 //printf("keyReleaseEvent\n");
 	keyModifiers = e->modifiers();
     if (keyModifiers == Qt::ControlModifier) {
-        setCursor(Qt::SizeAllCursor);
+        setCursor(Qt::ClosedHandCursor);
     }
     else if (keyModifiers == Qt::ShiftModifier) {
         setCursor(Qt::UpArrowCursor);
@@ -696,13 +930,13 @@ void  Terrain::wheelEvent(QWheelEvent * e)
 	double k = 1.3;
 	
 	if (e->delta() > 0)
-		deltaZoomWheel /= k;
-	else
 		deltaZoomWheel *= k;
+	else
+		deltaZoomWheel /= k;
 	
 	// Le timer évite les multiples update() pendant les changements de taille
     timerZoomWheel->stop();	 // pas d'update() tout de suite
-    timerZoomWheel->start(150); // update() seulement après une petite inactivité
+    timerZoomWheel->start(100); // update() seulement après une petite inactivité
 }
 //---------------------------------------------------------
 void Terrain::slotTimerZoomWheel () {
@@ -711,6 +945,7 @@ void Terrain::slotTimerZoomWheel () {
 		proj->zoom(deltaZoomWheel);
 		deltaZoomWheel = 1;
 		setProjection(proj);
+		//DBGN(proj->getScale());
     }
 }
 
@@ -721,33 +956,43 @@ void Terrain::mousePressEvent (QMouseEvent * e) {
     if (e->button() == Qt::LeftButton)
     {
         // Début de sélection de zone rectangulaire
-        isSelectionZoneEnCours = true;
+		if (e->modifiers() == Qt::ControlModifier) {
+		// TODO use  tiles to drag map
+			isDraggingMapEnCours = true;
+		}
+		else {
+			isSelectionZoneEnCours = true;
+		}
         proj->screen2map(e->x(),e->y(), &selX1, &selY1);
-/*        selX1 = floor(selX1*12)/12;
-        selY1 = ceil (selY1*12)/12;*/
         selX0 = selX1;
         selY0 = selY1;
+		globalX0 = e->globalX();
+		globalY0 = e->globalY();
         update();
     }
 }
 //---------------------------------------------------------
 void Terrain::mouseReleaseEvent (QMouseEvent * e) {
-//printf("release\n");
     double x0, y0, x1, y1;
-/*proj->screen2map(e->x(), e->y(), &x0,&y0);
-printf("clic: i=%d j=%d   x=%f y=%f\n", e->x(), e->y(), x0, y0);*/
     
+    globalX0 = 0;
+    globalY0 = 0;
+	
+    if (isDraggingMapEnCours)
+    {
+		// TODO use  tiles to drag map
+        isDraggingMapEnCours = false;
+		setProjection (proj);
+    }
     if (isSelectionZoneEnCours)
     {
         isSelectionZoneEnCours = false;
         proj->screen2map(e->x(),e->y(), &selX1, &selY1);
-/*        selX1 = ceil (selX1*12)/12;
-        selY1 = floor(selY1*12)/12;*/
         if (getSelectedRectangle(&x0,&y0, &x1,&y1))
         {
             emit selectionOK(x0, y0, x1, y1);
-            // Zoom sur la sélection si Control appuyé
-            if (e->modifiers() == Qt::ControlModifier) {
+            // Zoom sur la sélection si Shift appuyé
+            if (e->modifiers() == Qt::ShiftModifier) {
                 this->slot_Zoom_Sel();
             }
         }
@@ -761,14 +1006,28 @@ printf("clic: i=%d j=%d   x=%f y=%f\n", e->x(), e->y(), x0, y0);*/
 }
 
 //---------------------------------------------------------
-void Terrain::mouseMoveEvent (QMouseEvent * e) {
-//DBG ("move %d %d",e->x(),e->y());
-    if (isSelectionZoneEnCours)
+void Terrain::mouseMoveEvent (QMouseEvent * e) 
+{
+    if (isDraggingMapEnCours)
     {
-        proj->screen2map(e->x(),e->y(), &selX1, &selY1);
-        update();
+		// TODO use  tiles to drag map
+		double x0, y0, x1, y1;
+		int mx = lastMouseX-e->x();
+		int my = lastMouseY-e->y();
+		proj->screen2map (mx, my,  &x0, &y0);
+		proj->screen2map (mx+width(), my+height(),  &x1, &y1);
+		proj->setVisibleArea (x0,y0, x1,y1);
+		setProjection (proj);
     }
-    emit mouseMoved(e);
+    else if (isSelectionZoneEnCours)
+    {
+		proj->screen2map (e->x(),e->y(), &selX1, &selY1);
+		update();	// draw selection rectangle
+    }
+    
+	lastMouseX = e->x();
+	lastMouseY = e->y();
+	emit mouseMoved(e);
 }
 
 //---------------------------------------------------------
@@ -776,38 +1035,200 @@ void Terrain::resizeEvent (QResizeEvent * /*e*/)
 {
 	static int cpt = 0;
 	cpt ++;
-
-//printf("resize %03d : %d %d\n", cpt, width(), height());
-
+	
     indicateWaitingMap();
 
 	int W = width();
 	int H = height();
 	double x, y;
 	proj->screen2map(0, 0, &x, &y);	// position du coin haut-gauche
-    
     proj->setScreenSize(W, H);
-    
+	
     if (cpt > 1) {   // pas de repositionnement lors de l'ouverture (1er resize)
     	proj->setMapPointInScreen(x,y, 0,0); // coin haut-gauche fixe
     }
-    
-    isEarthMapValid = false;
-	mustRedraw = true;
     isResizing = true;
 	
 	// Le timer évite les multiples update() pendant les changements de taille
     timerResize->stop();	 // pas d'update() tout de suite
-    timerResize->start(100); // update() seulement après une petite inactivité
+	timerResize->start(100); // update() seulement après une petite inactivité
 }
 //---------------------------------------------------------
 void Terrain::slotTimerResize () {
     if (isResizing) {
-//printf("timer update\n");
+		isEarthMapValid = false;
+		mustRedraw = true;
         isResizing = false;
         update();
     }
 }
+
+//---------------------------------------------------------
+// paintEvent
+//---------------------------------------------------------
+void Terrain::paintEvent(QPaintEvent *)
+{
+    QPainter pnt (this);
+    QColor transp;
+    int r = 100;
+    if (!isResizing || !firstDrawingIsDone)
+    {
+		firstDrawingIsDone = true;
+        // Draw the map and the GRIB data
+        QCursor oldcursor = cursor();
+        setCursor(Qt::WaitCursor);
+        
+        switch (currentFileType) {
+			case DATATYPE_GRIB :
+			case DATATYPE_MBLUE :
+				drawer->draw_GSHHS_and_GriddedData 
+					(pnt, mustRedraw, isEarthMapValid, proj, griddedPlot, drawCartouche);
+				break;
+			case DATATYPE_IAC :
+				drawer->draw_GSHHS_and_IAC 
+					(pnt, mustRedraw, isEarthMapValid, proj, iacPlot, drawCartouche);
+				break;
+			default :
+				drawer->draw_GSHHS (pnt, mustRedraw, isEarthMapValid, proj);
+        }
+		
+        setCursor(oldcursor);
+		isEarthMapValid = true;
+		mustRedraw = false;
+		pleaseWait = false;
+        
+        if (selX0!=selX1 && selY0!=selY1) {
+            // Draw the rectangle of the selected zone
+            pnt.setPen(selectColor);
+            transp = QColor(r,r,r, 80);
+            pnt.setBrush(transp);
+            int x0,y0, x1,y1;
+            proj->map2screen(selX0, selY0, &x0, &y0);
+            proj->map2screen(selX1, selY1, &x1, &y1);
+            pnt.drawRect(x0, y0, x1-x0, y1-y0);
+            
+            if (showOrthodromie)
+            {
+                QPen penLine(QColor(Qt::white));
+                penLine.setWidthF(1.6);
+                pnt.setPen(penLine);
+                draw_Orthodromie(pnt);
+            }
+        }
+    }
+    else {
+        switch (currentFileType) {
+			case DATATYPE_GRIB :
+			case DATATYPE_MBLUE :
+				drawer->draw_GSHHS_and_GriddedData 
+						(pnt, false, true, proj, griddedPlot);
+				break;
+			case DATATYPE_IAC :
+				drawer->draw_GSHHS_and_IAC 
+						(pnt, false, true, proj, iacPlot);
+				break;
+			default :
+				drawer->draw_GSHHS (pnt, mustRedraw, isEarthMapValid, proj);
+        }
+	}
+    
+    if (mustShowSpecialZone) {
+		if (specialZoneX0!=specialZoneX1 && specialZoneY0!=specialZoneY1) {
+			pnt.setPen(QColor(Qt::white));
+			r = 80;
+			transp = QColor(r,r,r, 50);
+			pnt.setBrush(transp);
+			int x0,y0, x1,y1;
+			proj->map2screen(specialZoneX0, specialZoneY0, &x0, &y0);
+			proj->map2screen(specialZoneX1, specialZoneY1, &x1, &y1);
+			pnt.drawRect(x0, y0, x1-x0, y1-y0);
+		}
+    }
+    
+    if (pleaseWait) {
+        // Write the message "please wait..." on the map
+        QFont fontWait = Font::getFont(FONT_MapWait);
+        QFontMetrics fmet(fontWait);
+        pnt.setPen(QColor(Qt::white));
+        r = 80;
+        transp = QColor(r,r,r, 80);
+        pnt.setFont(fontWait);
+        pnt.setBrush(transp);
+        QString txt = tr("  Please wait...  ");
+        QRect rect = fmet.boundingRect(txt);
+
+        rect.moveTo(20,20);
+        pnt.drawRect(rect);
+        pnt.drawText(rect, Qt::AlignHCenter|Qt::AlignVCenter , txt);
+    }
+}
+//------------------------------------------------------------------
+time_t Terrain::getCurrentDate()
+{
+	switch (currentFileType) {
+		case DATATYPE_GRIB :
+		case DATATYPE_MBLUE :
+			if (griddedPlot && griddedPlot->isReaderOk() )
+				return griddedPlot->getCurrentDate();
+			break;
+		case DATATYPE_IAC :
+			DBGS("TODO: get date IAC")
+			return 0;
+			break;
+		default :
+			return 0;
+	}
+	return 0;
+}
+//------------------------------------------------------------------------
+QPixmap * Terrain::createPixmap (time_t date, int width, int height)
+{
+	Projection *scaledproj = proj->clone(); 
+	MapDrawer  *scaleddrawer = new MapDrawer (*drawer);
+	QPixmap *pixmap = NULL;
+	
+	if (scaledproj && scaleddrawer)
+	{
+		double x0,y0, x1,y1;
+		scaledproj->getVisibleArea (&x0,&y0, &x1,&y1);
+		scaledproj->setScreenSize  (width, height);
+		scaledproj->setVisibleArea (x0,y0, x1,y1);
+		switch (currentFileType) {
+			case DATATYPE_GRIB :
+			case DATATYPE_MBLUE :
+				if (griddedPlot && griddedPlot->isReaderOk() )
+				{
+					pixmap = scaleddrawer->createPixmap_GriddedData ( 
+										date, 
+										false, 
+										griddedPlot, 
+										scaledproj, 
+										getListPOIs() );
+				}
+				break;
+			case DATATYPE_IAC :
+				if (iacPlot && iacPlot->isReaderOk() )
+				{
+					DBGS("TODO: save image IAC")
+				}
+				break;
+			default :	// draw only map
+				pixmap = scaleddrawer->createPixmap_GriddedData ( 
+									date, 
+									false, 
+									NULL, 
+									scaledproj, 
+									getListPOIs() );
+		}
+		delete scaledproj;
+		delete scaleddrawer;
+	}
+	else {
+		DBG("Memory problem... ");
+	}
+	return pixmap;
+}
+
 //---------------------------------------------------------
 void Terrain::slot_Zoom_In()
 {
@@ -927,140 +1348,28 @@ void Terrain::setGeopotentialStep (int step)
 		update();
     }
 }
-
-
-//---------------------------------------------------------
-// paintEvent
-//---------------------------------------------------------
-void Terrain::paintEvent(QPaintEvent * /*event*/)
+//-------------------------------------------------------
+void Terrain::setMBlueFastInterpolation (bool b)
 {
-    QPainter pnt(this);
-    QColor transp;
-    int r = 100;
-    if (!isResizing)
-    {
-        // Draw the map and the GRIB data
-        QCursor oldcursor = cursor();
-        setCursor(Qt::WaitCursor);
-        
-        switch (currentFileType) {
-			case DATATYPE_GRIB :
-			case DATATYPE_MBLUE :
-				drawer->draw_GSHHS_and_GriddedData 
-						(pnt, mustRedraw, isEarthMapValid, proj, 
-						 griddedPlot
-						);
-				break;
-			case DATATYPE_IAC :
-				drawer->draw_GSHHS_and_IAC 
-						(pnt, mustRedraw, isEarthMapValid, proj, iacPlot);
-				break;
-			default :
-				drawer->draw_GSHHS (pnt, mustRedraw, isEarthMapValid, proj);
-        }
-		
-        setCursor(oldcursor);
-		isEarthMapValid = true;
-		mustRedraw = false;
-		pleaseWait = false;
-        
-        if (selX0!=selX1 && selY0!=selY1) {
-            // Draw the rectangle of the selected zone
-            pnt.setPen(selectColor);
-            transp = QColor(r,r,r, 80);
-            pnt.setBrush(transp);
-            int x0,y0, x1,y1;
-            proj->map2screen(selX0, selY0, &x0, &y0);
-            proj->map2screen(selX1, selY1, &x1, &y1);
-            pnt.drawRect(x0, y0, x1-x0, y1-y0);
-            
-            if (showOrthodromie)
-            {
-                QPen penLine(QColor(Qt::white));
-                penLine.setWidthF(1.6);
-                pnt.setPen(penLine);
-                draw_Orthodromie(pnt);
-            }
-        }
-
-    }
-    
-    if (pleaseWait) {
-        // Write the message "please wait..." on the map
-        QFont fontWait = Font::getFont(FONT_MapWait);
-        QFontMetrics fmet(fontWait);
-        pnt.setPen(QColor(Qt::white));
-        r = 80;
-        transp = QColor(r,r,r, 80);
-        pnt.setFont(fontWait);
-        pnt.setBrush(transp);
-        QString txt = tr("  Please wait...  ");
-        QRect rect = fmet.boundingRect(txt);
-
-        rect.moveTo(20,20);
-        pnt.drawRect(rect);
-        pnt.drawText(rect, Qt::AlignHCenter|Qt::AlignVCenter , txt);
-    }
-}
-//------------------------------------------------------------------
-time_t Terrain::getCurrentDate()
-{
-	switch (currentFileType) {
-		case DATATYPE_GRIB :
-		case DATATYPE_MBLUE :
-			if (griddedPlot && griddedPlot->isReaderOk() )
-				return griddedPlot->getCurrentDate();
-			break;
-		case DATATYPE_IAC :
-			DBGS("TODO: get date IAC")
-			return 0;
-			break;
-		default :
-			return 0;
-	}
-	return 0;
-}
-//------------------------------------------------------------------------
-QPixmap * Terrain::createPixmap (time_t date, int width, int height)
-{
-	Projection *scaledproj = proj->clone(); 
-	MapDrawer  *scaleddrawer = new MapDrawer (*drawer);
-	
-	if (scaledproj && scaleddrawer)
-	{
-		double x0,y0, x1,y1;
-		scaledproj->getVisibleArea (&x0,&y0, &x1,&y1);
-		scaledproj->setScreenSize  (width, height);
-		scaledproj->setVisibleArea (x0,y0, x1,y1);
-		switch (currentFileType) {
-			case DATATYPE_GRIB :
-			case DATATYPE_MBLUE :
-				if (griddedPlot && griddedPlot->isReaderOk() )
-				{
-					QPixmap *pixmap = 
-						scaleddrawer->createPixmap_GriddedData ( 
-										date, 
-										false, 
-										griddedPlot, 
-										scaledproj, 
-										getListPOIs() );
-					return pixmap;
-				}
-				break;
-			case DATATYPE_IAC :
-				if (iacPlot && iacPlot->isReaderOk() )
-				{
-					DBGS("TODO: save image IAC")
-				}
-				break;
-			default :
-					DBGS("TODO: save image without data")
-				return NULL;
+	if (b != fastInterpolation_MBlue) {
+		Util::setSetting ("MBfastInterpolation", b);
+		fastInterpolation_MBlue = b;
+		if (griddedPlot) {
+			griddedPlot->setFastInterpolation (fastInterpolation_MBlue);
+			mustRedraw = true;
+			update();
 		}
-		delete scaledproj;
-		delete scaleddrawer;
 	}
-	return NULL;
+}
+//-------------------------------------------------------
+DataCode Terrain::getColorMapData ()
+{ 
+	DataCode dtc = drawer->getColorMapData();
+	if (dtc.dataType == GRB_PRV_WIND_XY2D) {
+		if (Util::getSetting("useJetStreamColorMap", false).toBool()) {
+			dtc.dataType = GRB_PRV_WIND_JET;
+		}
+	}
+	return dtc; 
 }
 
-
