@@ -1,6 +1,6 @@
 /**********************************************************************
 zyGrib: meteorological GRIB file viewer
-Copyright (C) 2008-2010 - Jacques Zaninetti - http://www.zygrib.org
+Copyright (C) 2008-2012 - Jacques Zaninetti - http://www.zygrib.org
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -20,13 +20,15 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #define VERSION_H
 
 
-#define ZYGRIB_VERSION_NUM   "3.9.2"
-#define ZYGRIB_VERSION_DATE  "2010-05-14"
+#define ZYGRIB_VERSION_NUM   "6.2.1"
+#define ZYGRIB_VERSION_DATE  "2013-11-25"
 
-#ifdef Q_OS_WIN32
+#if defined(Q_OS_WIN32)
 	#define ZYGRIB_APP_NAME   "zyGrib_win"
+#elif defined(Q_OS_MAC)
+	#define ZYGRIB_APP_NAME   "zyGrib_mac"
 #else
-	#define ZYGRIB_APP_NAME   "zyGrib"
+	#define ZYGRIB_APP_NAME   "zyGrib_unx"
 #endif
 
 
@@ -43,6 +45,9 @@ class Version {
         static QString getDate() {
             return QString(ZYGRIB_VERSION_DATE);
         }
+        static QString getShortName() {
+            return QString( "zyGrib-"+getVersion() );
+        }
         static QString getCompleteName() {
             return QString( getAppName()+"-"+getVersion() );
         }
