@@ -1,6 +1,6 @@
 /**********************************************************************
 zyGrib: meteorological GRIB file viewer
-Copyright (C) 2008 - Jacques Zaninetti - http://www.zygrib.org
+Copyright (C) 2008-2012 - Jacques Zaninetti - http://www.zygrib.org
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -17,35 +17,150 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ***********************************************************************/
 
 #include <unistd.h>
+#include <time.h>
+
 #include <QDir>
 #include <QStringList>
 
+#include <QUrl>
+#include <QNetworkProxy>
+#include <QNetworkProxyQuery>
+#include <QCryptographicHash>
+#include <QFileDialog>
+
 #include "Settings.h"
 #include "Util.h"
+#include "Version.h"
+
 
 //======================================================================
-QHash <QString, QVariant> hashSettings;
+QHash <QString, QVariant> GLOB_hashSettings;
 
-void Util::setSetting(const QString &key, const QVariant &value)
+void Util::setSetting (const QString &key, const QVariant &value)
 {
-	hashSettings.insert (key, value);
-	Settings::setUserSetting(key, value);
+	GLOB_hashSettings.insert (key, value);
+	Settings::setUserSetting (key, value);
 }
 //---------------------------------------------------------------------
-QVariant Util::getSetting(const QString &key, const QVariant &defaultValue)
+QVariant Util::getSetting (const QString &key, const QVariant &defaultValue)
 {
-	if (hashSettings.contains(key) )
+	if (GLOB_hashSettings.contains (key) )
 	{
-		return hashSettings.value (key);
+		return GLOB_hashSettings.value (key);
 	}
 	else
 	{
-		QVariant v = Settings::getUserSetting(key, defaultValue);
-		hashSettings.insert (key, v);
+		QVariant v = Settings::getUserSetting (key, defaultValue);
+		GLOB_hashSettings.insert (key, v);
 		return v;
 	}
 }
-
+//========================================================================
+QString Util::getSaveFileName (QWidget *parent, const QString &caption, 
+							const QString &dir, const QString &filter)
+{
+	#ifdef Q_OS_MACX
+		if ( QSysInfo::MacintoshVersion > QSysInfo::MV_10_8 )
+		{   // fix Mac OS X 10.9 (mavericks) QFileDialog bug
+			int useNative = Util::getSetting("mac_useNativeFileDialog", 999).toInt();
+			if (useNative == 999) {
+				Util::setSetting("mac_useNativeFileDialog", 0);
+				useNative = 0;
+			}
+			if (useNative == 0) {
+				return QFileDialog::getSaveFileName (parent, caption, dir, filter, 
+							0, QFileDialog::DontUseNativeDialog);
+			}
+		}
+	#endif
+	return QFileDialog::getSaveFileName (parent, caption, dir, filter);
+}
+//------------------------------------------------------------
+QString Util::getOpenFileName (QWidget *parent, const QString &caption, 
+							const QString &dir, const QString &filter)
+{
+	#ifdef Q_OS_MACX
+		if ( QSysInfo::MacintoshVersion > QSysInfo::MV_10_8 )
+		{   // fix Mac OS X 10.9 (mavericks) QFileDialog bug
+			int useNative = Util::getSetting("mac_useNativeFileDialog", 999).toInt();
+			if (useNative == 999) {
+				Util::setSetting("mac_useNativeFileDialog", 0);
+				useNative = 0;
+			}
+			if (useNative == 0) {
+				return QFileDialog::getOpenFileName (parent, caption, dir, filter, 
+							0, QFileDialog::DontUseNativeDialog);
+			}
+		}
+	#endif
+	return QFileDialog::getOpenFileName (parent, caption, dir, filter);
+}
+//------------------------------------------------------------
+QString Util::getServerName ()
+{
+	return "www.zygrib.org";
+}
+//------------------------------------------------------------
+void Util::setApplicationProxy ()
+{
+	QNetworkProxy proxy;
+	
+	if (Util::getSetting("httpUseProxy", false).toBool())
+	{
+		proxy.setType ((QNetworkProxy::ProxyType)
+						(Util::getSetting("httpProxyType", QNetworkProxy::HttpProxy).toInt()));
+		proxy.setHostName (Util::decode(Util::getSetting("httpProxyHostname", "").toString()));
+		proxy.setPort     (Util::getSetting("httpProxyPort", 0).toInt());
+		proxy.setUser     (Util::decode(Util::getSetting("httpProxyUsername", "").toString()));
+		proxy.setPassword (Util::decode(Util::getSetting("httpProxyUserPassword", "").toString()));
+		
+		if (proxy.type() == QNetworkProxy::DefaultProxy)
+		{
+			QNetworkProxyQuery proxq = QNetworkProxyQuery (QUrl("http://"+getServerName()+"/index.php"));
+			QList<QNetworkProxy> list = QNetworkProxyFactory::systemProxyForQuery (proxq);
+			if (list.size() > 0) {
+				QNetworkProxy pr2 = list [0];
+				QNetworkProxy::setApplicationProxy (pr2);
+			}
+			else {
+				QNetworkProxy::setApplicationProxy (proxy);
+			}
+		}
+		else {
+			QNetworkProxy::setApplicationProxy (proxy);
+		}
+	}
+	else {
+		proxy.setType (QNetworkProxy::NoProxy);
+		QNetworkProxy::setApplicationProxy (proxy);
+	}
+}
+//-------------------------------------------------------------------------------
+QNetworkRequest Util::makeNetworkRequest (QString url,double x0,double y0,double x1,double y1)
+{
+	QNetworkRequest request;
+	QString now = QTime::currentTime().toString("HHmmss");
+	if (url.contains('?'))
+		url += "&tm="+now;
+	else
+		url += "?tm="+now;
+	if (Util::getSetting ("strictHttpDownload", false).toBool()) {
+		QString zl = Util::encode(Util::decode(Util::getSetting("zyGribForumUserName", "").toString()),now);
+		QString zp = Util::encode(Util::decode(Util::getSetting("zyGribForumUserPassword", "").toString()),now);
+		url += "&zl="+zl+"&zp="+zp+"&md=h";
+		request.setRawHeader ("User-Agent","Mozilla/5.0 (Windows NT 5.1; rv:8.0) Gecko/20100101 Firefox/8.0");
+ 		request.setRawHeader ("Accept","text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8");
+		//request.setRawHeader ("Accept-Language","fr-ch,fr;q=0.8,fr-fr;q=0.6,en-us;q=0.4,en;q=0.2");
+		//request.setRawHeader ("Accept-Encoding","gzip, deflate");
+		//request.setRawHeader ("Accept-Charset","ISO-8859-1,utf-8;q=0.7,*;q=0.7");
+		//request.setRawHeader ("Cache-Control","max-stale=0");
+	}
+	else
+		request.setRawHeader ("User-Agent",qPrintable(Version::getAppName()+"/"+Version::getVersion()) );
+	request.setUrl (QUrl(url));
+	validArea (request,x0,y0,x1,y1);
+	return request;
+}
 //======================================================================
 bool Util::isDirWritable (const QDir &dir)
 {
@@ -68,59 +183,84 @@ bool Util::isDirWritable (const QDir &dir)
 }
     
 //======================================================================
+float Util::convertTemperature (float tempKelvin)
+{
+    QString unit = Util::getSetting("unitsTemp", QVariant(tr("°C"))).toString();
+    if (unit == tr("°K")) {
+        return tempKelvin;
+    }
+    else if (unit == tr("°F")) {
+        return 1.8*(tempKelvin-273.15)+32.0;
+    }
+    else  {
+        return tempKelvin-273.15;
+    }
+}
+//-------------------------------------------------------
 QString Util::formatTemperature (float tempKelvin, bool withUnit)
 {
-    QString unit = Util::getSetting("unitsTemp", QObject::tr("°C")).toString();
+    QString unit = Util::getSetting("unitsTemp", QVariant(tr("°C"))).toString();
     QString r;
-    if (unit == QObject::tr("°C")) {
-        r.sprintf("%.1f", tempKelvin-273.15);
+    if (unit == tr("°K")) {
+        r.sprintf("%.1f", tempKelvin);
     }
-    else if (unit == QObject::tr("°F")) {
+    else if (unit == tr("°F")) {
         r.sprintf("%.1f", 1.8*(tempKelvin-273.15)+32.0);
     }
-    else  {   // if (unit == QObject::tr("°K"))
-        unit = QObject::tr("°K");
-        r.sprintf("%.1f", tempKelvin);
+    else  {   // if (unit == tr("°K"))
+        unit = tr("°C");
+        r.sprintf("%.1f", tempKelvin-273.15);
     }
 	return (withUnit) ? r+""+unit : r;
 }
 //-------------------------------------------------------
 QString Util::formatTemperature_short(float tempKelvin, bool withUnit)
 {
-    QString unit = Util::getSetting("unitsTemp", QObject::tr("°C")).toString();
+    QString unit = Util::getSetting("unitsTemp", tr("°C")).toString();
     QString r;
-    if (unit == QObject::tr("°C")) {
-        r.sprintf("%d", qRound(tempKelvin-273.15) );
+    if (unit == tr("°K")) {
+        r.sprintf("%d", qRound(tempKelvin) );
     }
-    else if (unit == QObject::tr("°F")) {
+    else if (unit == tr("°F")) {
         r.sprintf("%d", qRound(1.8*(tempKelvin-273.15)+32.0) );
     }
-    else  {   // if (unit == QObject::tr("°K"))
-        unit = QObject::tr("°K");
-        r.sprintf("%d", qRound(tempKelvin) );
+    else  {   // if (unit == tr("°K"))
+        unit = tr("°C");
+        r.sprintf("%d", qRound(tempKelvin-273.15) );
     }
 	return (withUnit) ? r+""+unit : r;
 }
 //----------------------------------------------------------------
-QString Util::formatSpeed(float meterspersecond, bool withUnit)
+QString Util::formatSpeed (float meterspersecond, bool withUnit, QString unit)
 {
-    QString unit = Util::getSetting("unitsWindSpeed", QObject::tr("km/h")).toString();
     QString r;
     if (unit == "m/s") {
         r.sprintf("%.1f ", meterspersecond);
-        unit = QObject::tr("m/s");
+        unit = tr("m/s");
     }
     else if (unit == "km/h") {
         r.sprintf("%.1f ", meterspersecond*3.6);
-        unit = QObject::tr("km/h");
+        unit = tr("km/h");
     }
-    else  {   // if (unit == QObject::tr("kts"))
+    else  {   // if (unit == tr("kts"))
         r.sprintf("%.1f ", meterspersecond*3.6/1.852);
-        unit = QObject::tr("kts");
+        unit = tr("kts");
     }
 	return (withUnit) ? r+" "+unit : r;
 }
 //----------------------------------------------------------------
+QString Util::formatSpeed_Wind (float meterspersecond, bool withUnit)
+{
+    QString unit = Util::getSetting("unitsWindSpeed", tr("km/h")).toString();
+	return Util::formatSpeed (meterspersecond, withUnit, unit);
+}
+//----------------------------------------------------------------
+QString Util::formatSpeed_Current (float meterspersecond, bool withUnit)
+{
+    QString unit = Util::getSetting("unitsCurrentSpeed", tr("kts")).toString();
+	return Util::formatSpeed (meterspersecond, withUnit, unit);
+}
+//----------------------------------------------------------------
 QString Util::formatDirection (float angle, bool withUnit)
 {
     QString unite = tr("°");
@@ -131,10 +271,10 @@ QString Util::formatDirection (float angle, bool withUnit)
 //----------------------------------------------------------------
 QString Util::formatDistance (float mille, bool withUnit)
 {
-    QString unit = Util::getSetting("unitsDistance", QObject::tr("km")).toString();
+    QString unit = Util::getSetting("unitsDistance", tr("km")).toString();
     QString r;
     float d;
-    if (unit == QObject::tr("km")) {
+    if (unit == tr("km")) {
         unit = "km";
         d= mille*1.852;
     }
@@ -151,6 +291,46 @@ QString Util::formatDistance (float mille, bool withUnit)
 	return (withUnit) ? r+" "+unit : r;
 }
 //----------------------------------------------------------------
+QString Util::getDataUnit (int dataType)
+{
+	return getDataUnit (DataCode(dataType,LV_GND_SURF,0));
+}
+//----------------------------------------------------------------
+QByteArray Util::sha1 (const QByteArray &v) 
+{
+	return QCryptographicHash::hash(v, QCryptographicHash::Sha1).toHex();
+}
+//----------------------------------------------------------------
+QString Util::encode (const QString &v, const QString &k) {
+	QByteArray pv = v.toUtf8 ();
+	QString ke = k.trimmed();
+	if (ke == "") ke="zygrib";
+	QByteArray pk = Util::sha1 (ke.toUtf8 ());
+	QByteArray re;
+	int i, j=0;
+	for (i=0; i<pv.size(); i++) {
+		re.append (pv[i]^pk[j]);
+		j++;
+		if (j==pk.size()) j=0;
+	}
+	return QString(re.toBase64());
+}
+//----------------------------------------------------------------
+QString Util::decode (const QString &v, const QString &k) {
+	QByteArray pv = QByteArray::fromBase64 (v.toUtf8());
+	QString ke = k.trimmed();
+	if (ke == "") ke="zygrib";
+	QByteArray pk = Util::sha1 (ke.toUtf8 ());
+	QByteArray re;
+	int i, j=0;
+	for (i=0; i<pv.size(); i++) {
+		re.append (pv[i]^pk[j]);
+		j++;
+		if (j==pk.size()) j=0;
+	}
+	return QString(re);
+}
+//----------------------------------------------------------------
 QString Util::getDataUnit (const DataCode &dtc)
 {
 	QString unit;
@@ -181,34 +361,67 @@ QString Util::getDataUnit (const DataCode &dtc)
 		case GRB_TMAX         : 
 		case GRB_TMIN         : 
 		case GRB_DEWPOINT     : 
-		case GRB_DIFF_TEMPDEW : 
-		case GRB_THETA_E      : 
-			return Util::getSetting("unitsTemp", QObject::tr("°C")).toString();
+		case GRB_PRV_DIFF_TEMPDEW : 
+		case GRB_PRV_THETA_E      : 
+			return Util::getSetting("unitsTemp", tr("°C")).toString();
 			break;
-		case GRB_WIND_VX      : 
-		case GRB_WIND_VY      : 
-		case GRB_WIND_XY2D    : 
-			return Util::getSetting("unitsWindSpeed", QObject::tr("km/h")).toString();
+		case GRB_WIND_VX    : 
+		case GRB_WIND_VY    : 
+		case GRB_WIND_SPEED : 
+		case GRB_PRV_WIND_XY2D : 
+		case GRB_PRV_WIND_JET  : 
+			return Util::getSetting("unitsWindSpeed", tr("km/h")).toString();
+			break;
+		case GRB_CUR_VX      : 
+		case GRB_CUR_VY      : 
+		case GRB_PRV_CUR_XY2D    : 
+			return Util::getSetting("unitsCurrentSpeed", tr("kts")).toString();
 			break;
 		case GRB_CAPE 		  : 
-			return QObject::tr("J/kg");
+		case GRB_CIN 		  : 
+			return tr("J/kg");
 			break;
 		case GRB_SNOW_DEPTH   : 
-			unit = Util::getSetting("snowDepthUnit", QObject::tr("m")).toString();
-			if (unit == QObject::tr("m"))
-				unit = QObject::tr("cm");
+			unit = Util::getSetting("snowDepthUnit", tr("m")).toString();
+			if (unit == tr("m"))
+				unit = tr("cm");
 			return unit;
 			break;
 		case GRB_PRECIP_RATE  : 
 		case GRB_PRECIP_TOT   : 
-			return QObject::tr("mm/h");
+			return tr("mm/h");
 			break;
-		case GRB_PRESSURE     : 
-			return QObject::tr("hPa");
+		case GRB_PRESSURE_MSL : 
+			return tr("hPa");
+			break;
+		case GRB_WAV_SIG_HT : 
+		case GRB_WAV_WND_HT : 
+		case GRB_WAV_SWL_HT : 
+		case GRB_WAV_MAX_HT : 
+			return tr("m");
+			break;
+		case GRB_WIND_DIR : 
+		case GRB_WAV_WND_DIR : 
+		case GRB_WAV_SWL_DIR : 
+		case GRB_WAV_MAX_DIR : 
+		case GRB_WAV_PRIM_DIR : 
+		case GRB_WAV_SCDY_DIR : 
+			return tr("°");
+			break;
+		case GRB_WAV_WND_PER : 
+		case GRB_WAV_SWL_PER : 
+		case GRB_WAV_MAX_PER : 
+		case GRB_WAV_PRIM_PER : 
+		case GRB_WAV_SCDY_PER : 
+			return tr("s");
+			break;
+		case GRB_WAV_WHITCAP_PROB : 
+			return tr("%");
 			break;
 		default:
 			return "?";
 	}
+	return "?";
 }
 //----------------------------------------------------------------
 double Util::getDataCoef (const DataCode &dtc)
@@ -230,6 +443,7 @@ double Util::getDataCoef (const DataCode &dtc)
 		default:
 			return 1.0;
 	}
+	return 1.0;
 }
 //----------------------------------------------------------------
 QString Util::formatGeopotAltitude (float meter, bool withUnit)
@@ -253,11 +467,16 @@ QString Util::formatRain (float mmh, bool withUnit)
 	return (withUnit) ? r+" "+unite : r;
 }
 //----------------------------------------------------------------
-QString Util::formatPressure (float pasc, bool withUnit)
+QString Util::formatPressure (float pasc, bool withUnit, int precision)
 {
-    QString unite = Util::getDataUnit (DataCode(GRB_PRESSURE,LV_MSL,0));
+    QString unite = Util::getDataUnit (DataCode(GRB_PRESSURE_MSL,LV_MSL,0));
     QString r;
-	r.sprintf("%.1f", pasc/100.0);
+	if (pasc != GRIB_NOTDEF) {
+		if (precision > 0)
+			r.sprintf("%.1f", pasc/100.0);
+		else
+			r.sprintf("%.0f", pasc/100.0);
+	}
 	return (withUnit) ? r+" "+unite : r;
 }
 //----------------------------------------------------------------
@@ -265,16 +484,61 @@ QString Util::formatCAPEsfc (float jkg, bool withUnit)
 {
     QString unite = Util::getDataUnit (DataCode(GRB_CAPE,LV_GND_SURF,0));
     QString r;
-	r.sprintf("%d", (int) jkg);
+	r.sprintf("%d", qRound(jkg));
 	return (withUnit) ? r+" "+unite : r;
 }
 //----------------------------------------------------------------
-QString Util::formatSnowDepth(float meter, bool withUnit)
+QString Util::formatWaveHeight (float meter, bool withUnit)
 {
-    QString unit = Util::getSetting("snowDepthUnit", QObject::tr("m")).toString();
+    QString unit = Util::getSetting("waveHeightUnit", tr("m")).toString();
     QString r, unite;
     float d;
-    if (unit == QObject::tr("m")) {
+    if (unit == tr("m")) {
+		unite = unit;
+		d = meter;
+    }
+    else {
+        unite = "ft";
+        d= meter/0.3048;
+    }
+	if (d < 10)
+		r.sprintf("%.1f", d);
+	else
+		r.sprintf("%.0f", d);
+	return (withUnit) ? r+" "+unite : r;
+}
+//----------------------------------------------------------------
+QString Util::formatWavePeriod (float second, bool withUnit)
+{
+    QString unit = Util::getSetting("waveHeightPeriod", tr("s")).toString();
+    QString r;
+	r.sprintf("%.0f", second);
+	return (withUnit) ? r+" "+unit : r;
+}
+//----------------------------------------------------------------
+QString Util::formatWaveDirection (float angle, bool withUnit)
+{
+	return formatDirection (angle, withUnit);
+}
+//----------------------------------------------------------------
+QString Util::formatWhiteCap (float prb, bool withUnit)
+{
+    QString unite = "%";
+    QString r;
+	float v = inRange (0.0f, prb, 100.0f);
+	if (v < 10)
+		r.sprintf("%.1f", v);
+	else
+		r.sprintf("%.0f", v);
+	return (withUnit) ? r+" "+unite : r;
+}
+//----------------------------------------------------------------
+QString Util::formatSnowDepth (float meter, bool withUnit)
+{
+    QString unit = Util::getSetting("snowDepthUnit", tr("m")).toString();
+    QString r, unite;
+    float d;
+    if (unit == tr("m")) {
         unite = "cm";
         d= meter*100.0;
         if (d < 10)
@@ -290,14 +554,14 @@ QString Util::formatSnowDepth(float meter, bool withUnit)
 	return (withUnit) ? r+" "+unite : r;
 }
 //----------------------------------------------------------------
-QString Util::formatDegres(float x, bool inf100)     // 123.4 -> 123°24.00'
+QString Util::formatDegres (float x, bool inf100)     // 123.4 -> 123°24.00'
 {
 	const char *cdeg = "°";
     QString tunit = Util::getSetting("unitsPosition", "").toString();
-    QString unit = (tunit=="") ? QObject::tr("dd°mm'ss\"") : tunit;
+    QString unit = (tunit=="") ? tr("dd°mm'ss\"") : tunit;
     
     QString r;
-    if (unit == QObject::tr("dd°mm,mm'"))
+    if (unit == tr("dd°mm,mm'"))
     {
         int deg = (int) fabs(x);
         float min = (fabs(x) - deg)*60.0;
@@ -307,7 +571,7 @@ QString Util::formatDegres(float x, bool inf100)     // 123.4 -> 123°24.00'
 		else
         	r.sprintf("%c%03d%s%05.2f'", sign,deg,cdeg, min);
     }
-    else if (unit == QObject::tr("dd°mm'ss\""))
+    else if (unit == tr("dd°mm'ss\""))
     {
         int sec = (int) fabs(x*3600.0);  // total en secondes
         int min = sec / 60;              // nombre entier de minutes
@@ -320,7 +584,7 @@ QString Util::formatDegres(float x, bool inf100)     // 123.4 -> 123°24.00'
 		else
 	        r.sprintf("%c%03d%s%02d'%02d\"", sign,deg,cdeg, min,sec);
     }
-    else // if (unit == QObject::tr("dd,dd°"))
+    else // if (unit == tr("dd,dd°"))
     {
         if (inf100)
     	    r.sprintf("%05.2f%s",x,cdeg);
@@ -392,15 +656,25 @@ QString Util::formatLatitude(float y)
 //---------------------------------------------------------------------
 QString Util::formatPercentValue(float v, bool withUnit)
 {
+	if (v == GRIB_NOTDEF)
+		return withUnit ? "    %%": "   ";
     QString r;
     if (v<0)
         v=0;
     else if (v>100)
         v=100;    
-	if (withUnit)
-		r.sprintf("%d %%", (int)floor(v+0.5));
-	else
-		r.sprintf("%d", (int)floor(v+0.5));
+	if (withUnit) {
+		if (v < 10)
+			r.sprintf("%.1f %%", v);
+		else
+			r.sprintf("%.0f %%", v);
+	}
+	else {
+		if (v < 10)
+			r.sprintf("%.1f", v);
+		else
+			r.sprintf("%.0f", v);
+	}
     return r;
 }
 
@@ -510,7 +784,15 @@ QDateTime Util::applyTimeZone (time_t t, QString *suffix)
     
     return dt;
 }
-
+//-------------------------------------------------------------------------------
+void Util::validArea (QNetworkRequest &request,double x0,double y0,double x1,double y1)
+{
+	int c1 = (int)floor(fabs((17.6*x0+89.23*x1-7.23*y0)*y1));
+	int c2 = (int)ceil(fabs((37.12*x0-53.2*x1+7.23)*y0*y1));
+	QString formatrange;
+	QTextStream(&formatrange)<<c1<<"-"<<c2;
+	request.setRawHeader ("Range", qPrintable(formatrange));
+}
 //============================================================================
 double Util::distancePointSegment (double a,double b,     // point
 								  double x0, double y0,  // segment start
@@ -554,9 +836,29 @@ double Util::distancePointSegment (double a,double b,     // point
 	}
 	return dist;
 }
-
-
-
+//----------------------------------------------------------------
+double Util::roundMultiple (double x, double mul) 
+{
+	int sg;
+	if (x < 0) {
+		x = -x;
+		sg = -1;
+	}
+	else {
+		sg = 1;
+	}
+	return sg * ((int)floor(x/mul+0.5))*mul;
+}
+//----------------------------------------------------------------
+time_t Util::timestampSetHour (time_t t, int hour, int min, int sec)
+{
+	struct tm dt;
+	memcpy (&dt, gmtime(&t), sizeof(struct tm)); // no gmtime_r in windows mingw
+	dt.tm_hour = hour;
+	dt.tm_min = min;
+	dt.tm_sec = sec;
+	return mktime (&dt);
+}
 
 
 
